<?php

namespace Ems\Logic\ProcessamentoSolicitacao;

use Ems\Util\{ Utils, ExpectedException };
use Util\{ DateUtil, NovoCodigo, Repositories };

class ProcessamentoSolicitacaoAprovaReembolso extends ProcessamentoSolicitacao {

    public const EVENTO_REEMBOLSO = 890;
    public const MOVIMENTO_BENEFICIO = 21;
    
    public function efetivar() {
        $itens = $this->getItensAssociativeArray();
        
        $nrvinculom = $this->getSolicitacao()->getNrvinculom();
        $nrorg = Utils::getParamters()->getNrorg();
        $competenc = $itens['DTMESCOMPETENC'] ?? null;
        $vrReembolso = !empty($itens['VRREEMBOLSO']) ? (float) $itens['VRREEMBOLSO'] : null;
        
        if (empty($competenc)) {
            throw new ExpectedException('O mês de competência deve ser preenchido. Se necessário, crie uma nova solicitação');
        }
        if (empty($vrReembolso)) {
            throw new ExpectedException('O valor de reembolso deve ser preenchido. Se necessário, crie uma nova solicitação');
        }

        $dateTimeCompetenc = DateUtil::getDataDeString($competenc, DateUtil::FORMATO_BRASILEIRO, true);

        /* Validacao existencia de reembolso */

        $nrcalculo = Utils::getEntityManager()->getRepository(Repositories::FPA_CALCULOFOLHA)->getNrCalculo(
            $nrorg, $dateTimeCompetenc, self::MOVIMENTO_BENEFICIO
        );
        
        if(empty($nrcalculo)) {
            throw new ExpectedException('Não existe número de cálculo para o mês de referência selecionado. Confira o cálculo do vínculo para a competência ' . $competenc);
        }

        $reembolsoJaExistente = Utils::getEntityManager()->getRepository(Repositories::FPA_ITECALCFOLHA)->getValorEvento(
            $nrcalculo, $nrvinculom, $nrorg, '890', $dateTimeCompetenc
        );
        
        if (!empty($reembolsoJaExistente)) {
            throw new ExpectedException('Já existe um lançamento de reembolso na competência ' . $competenc . '. Realize a exclusão do evento antigo.');
        }

        /* Criacao do item */

        $this->criaEventoReembolso($nrcalculo, $competenc, $vrReembolso, $nrorg, $nrvinculom);
    }
    
    public function rejeitar() {}

    private function criaEventoReembolso(
        int $nrcalculo,
        string $competenc,
        float $vrReembolso,
        string $nrorg,
        string $nrvinculom
    ) {
        try {
            $itecalcfolha = Repositories::FPA_ITECALCFOLHA;
            $itecalc = new $itecalcfolha;
            $itecalc->setNritemcalcfolha(NovoCodigo::geraCodigo(Utils::getParamters()->getNrorg(), 'FPA_ITECALCFOLHA', 1, 12, true));
            $itecalc->setNrcalculofolha($nrcalculo);
            $itecalc->setNrvinculom($nrvinculom);
            $itecalc->setIdrefevalorcalc('NORMAL');
            $itecalc->setNreventom(self::EVENTO_REEMBOLSO);
            $itecalc->setVreventofolha($vrReembolso);
            $itecalc->setVrreffolha(1);
            $itecalc->setDtocorrencia(DateUtil::getDataDeString($competenc, DateUtil::FORMATO_BRASILEIRO, true));
            $itecalc->setNrorg($nrorg);
            $itecalc->setDtinclusao(DateUtil::getDataAtual(false));
            $itecalc->setNrorginclusao($nrorg);
            $itecalc->setCdoperinclusao(Utils::getParamters()->getCdoperador());
            $itecalc->setNrorgultatu($nrorg);
            $itecalc->setDtultatu(DateUtil::getDataAtual(false));
            $itecalc->setCdoperultatu(Utils::getParamters()->getCdoperador());
            
            Utils::getEntityManager()->persist($itecalc);
            Utils::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw new ExpectedException($e->getMessage());
        }
    }
}