<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

use Util\DateUtil;

class VinculoRepository {
    /** @var EntityManager */
    protected $entityManager;

    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->entityManager        = $entityManager;
        $this->connection           = $entityManager->getConnection();
    }

    public function getVinculoHistorico(int $nrorg, int $nrvinculom): Array {
        $sql = <<<SQL
            SELECT 
                VM.*,
                VH.NRESTRUTLEGAL,
                VH.NRESTRUTGEREN,
                VH.NRESTRUTSIND,
                VH.NRESCALATRABM,
                VH.NRCARGO,
                VH.IDTPPAGAMENTO,
                VH.IDREMUNERACAO,
                VH.IDMULTVINC,
                ATS.VRSALARIO,
                ATS.IDTPSALARIO,
                ATS.NRTPMODALIDSAL,
                OCUPM.NRTIPOOCUPACAO,
                TN.NRTURNO
            FROM GPE_VINCULOM VM
            JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG AND VH.NRVINCULOM = VM.NRVINCULOM AND VH.DTMESCOMPETENC = (
                SELECT MAX(DTMESCOMPETENC)
                  FROM GPE_VINCULOH VHH
                 WHERE VHH.NRORG = VH.NRORG
                       AND VHH.NRVINCULOM = VH.NRVINCULOM
            )
            LEFT JOIN GPE_OCUPACAOM OCUPM ON OCUPM.NRORG = VM.NRORG AND OCUPM.NROCUPACAOM = VH.NRCARGO
            LEFT JOIN GPE_TURNO TN ON TN.NRORG = VM.NRORG AND TN.NRESCALATRABM = VH.NRESCALATRABM
            LEFT JOIN GPE_ALTESALARIO ATS ON ATS.NRVINCULOM = :P_NRVINCULOM AND
                ATS.NRORG = :P_NRORG AND
                ATS.DTALTESALARIO = (
                    (SELECT DTALTESALARIO FROM 
                        (SELECT 
                            ATSS.DTALTESALARIO 
                        FROM 
                            GPE_ALTESALARIO ATSS 
                        WHERE 
                            ATSS.NRORG = VH.NRORG 
                            AND ATSS.NRVINCULOM = VH.NRVINCULOM 
                        ORDER BY ATSS.DTALTESALARIO DESC) 
                    WHERE ROWNUM = 1)
                )
             WHERE VM.NRORG = :P_NRORG AND
                   VM.NRVINCULOM = :P_NRVINCULOM
SQL;

        $query = $this->connection->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"       => $nrorg,
                    "P_NRVINCULOM"  => $nrvinculom
                )
            );

            $result = $query->fetchAll();

            return !empty($result) ? $result[0] : $result;
    }

    public function getVinculoMestreHistoricoByNrparcnegocio(int $nrorg, int $nrparcnegocio, string $dtmescompetenc): Array {
        $sql = <<<SQL
            SELECT VM.*,
                   VH.*
              FROM GPE_VINCULOM VM
              JOIN GPE_PESSOA PM ON PM.NRORG = VM.NRORG AND PM.NRPESSOA = VM.NRPESSOA
              JOIN PARCNEGOCIO PN ON PN.NRORG = VM.NRORG AND PN.NRPARCNEGOCIO = PM.NRPARCNEGOCIO
              JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG AND VH.NRVINCULOM = VM.NRVINCULOM AND VH.DTMESCOMPETENC = (
                (SELECT DTMESCOMPETENC FROM 
                        (SELECT 
                            VHH.DTMESCOMPETENC 
                         FROM 
                            GPE_VINCULOH VHH 
                         WHERE 
                           VHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC 
                           AND VHH.NRORG = VH.NRORG 
                           AND VHH.NRVINCULOM = VH.NRVINCULOM 
                         ORDER BY VHH.DTMESCOMPETENC DESC) 
                  WHERE ROWNUM = 1)
               )
             WHERE VM.NRORG = :P_NRORG
                   AND PN.NRPARCNEGOCIO = :P_NRPARCNEGOCIO
SQL;

        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"       => $nrorg,
                "P_NRPARCNEGOCIO"  => $nrparcnegocio,
                "P_DTMESCOMPETENC"  => $dtmescompetenc
            )
        );

        $result = $query->fetchAll();

        return !empty($result) ? $result[0] : $result;
    }


    public function getOperadorVinculo($nrorg, $nrvinculom) {
        $sql = <<<SQL
            SELECT OP.CDOPERADOR
              FROM GPE_VINCULOM VM
              JOIN GPE_PESSOA PE ON PE.NRORG = VM.NRORG
                                AND PE.NRPESSOA = VM.NRPESSOA
              JOIN OPERADOR OP ON OP.NRORG = PE.NRORG
                              AND OP.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
             WHERE VM.NRORG = :P_NRORG
                AND VM.NRVINCULOM = :P_NRVINCULOM
SQL;

        $query = $this->connection->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"       => $nrorg,
                    "P_NRVINCULOM"  => $nrvinculom
                )
            );

            $result = $query->fetch();

            return $result;
    }

    public function getVinculoMestreHistorico(string $nrorg, string $nrorgPadrao, string $nrvinculom) {
        $sql = <<<SQL
            SELECT VM.*,
                VH.*,
                TH.NMTIPOVINCULOH
            FROM GPE_VINCULOM VM
            JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG AND VH.NRVINCULOM = VM.NRVINCULOM AND VH.DTMESCOMPETENC = (
                (SELECT DTMESCOMPETENC FROM 
                    (SELECT 
                        VHH.DTMESCOMPETENC 
                    FROM 
                        GPE_VINCULOH VHH 
                    WHERE 
                        VHH.NRORG = VH.NRORG 
                        AND VHH.NRVINCULOM = VH.NRVINCULOM 
                    ORDER BY VHH.DTMESCOMPETENC DESC) 
                WHERE ROWNUM = 1)
            )
            LEFT JOIN GPE_TIPOVINCULOH TH ON (TH.NRORG = VM.NRORG OR TH.NRORG = :P_NRORG_PADRAO) AND TH.NRTIPOVINCULOM = VM.NRTIPOVINCULOM
                AND TH.DTMESCOMPETENC = (
                    SELECT MAX(DTMESCOMPETENC)
                    FROM GPE_TIPOVINCULOH
                    WHERE NRORG = TH.NRORG AND NRTIPOVINCULOM = TH.NRTIPOVINCULOM
                )
            WHERE VM.NRORG = :P_NRORG
                AND VM.NRVINCULOM = :P_NRVINCULOM
SQL;

        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_NRORG_PADRAO"    => $nrorgPadrao,
            )
        );

        $result = $query->fetchAll();

        return !empty($result) ? $result[0] : $result;
    }

    /**
     *  Rotina responsável por buscar a escala ativa de horários do funcionário. A escala é buscada a partir das alterações de escala registradas para o vínculo,
     * Disponíveis na tabela 'GPE_ALTEESCALA'
     *
     *
     *  @param  int     $nrOrg
     *  @param  int     $nrVinculo
     *  @param  string  $dtMesCompetenc = null
     *
     *  @return array|false
     */
    public function getHorarioBase(
        int     $nrOrg,
        int     $nrVinculo,
        string  $date
    ) {
        $sql = <<<SQL
            SELECT
                EH.PRIENTRADA,
                EH.PRISAIDA,
                EH.SEGENTRADA,
                EH.SEGSAIDA,
                PO.HRTEMPOTOLERANCIAANT,
                PO.HRTEMPOTOLERANCIAPOS
            FROM
                GPE_VINCULOM VM

                JOIN GPE_ALTEESCALA   AE ON AE.NRORG = VM.NRORG AND AE.NRVINCULOM = VM.NRVINCULOM AND AE.DTINIESCALA = (
                    SELECT MAX(DTINIESCALA) FROM GPE_ALTEESCALA
                    WHERE GPE_ALTEESCALA.DTINIESCALA <= TO_DATE(:P_DATE, 'DD/MM/YYYY')
                      AND GPE_ALTEESCALA.NRVINCULOM = AE.NRVINCULOM
                      AND GPE_ALTEESCALA.NRORG = AE.NRORG
                )

                JOIN
                GPE_ESCALATRABM EM ON
                    EM.NRORG = VM.NRORG AND
                    EM.NRESCALATRABM = AE.NRESCALATRABM
                JOIN
                GPE_ESCALATRABH EH ON
                    EM.NRESCALATRABM = EH.NRESCALATRABM AND
                    EM.NRORG = EH.NRORG AND
                    EH.IDATIVO = 'S' AND
                    EH.DTMESCOMPETENC = (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            GPE_ESCALATRABH
                        WHERE
                            NRORG = EH.NRORG AND
                            NRESCALATRABM = EH.NRESCALATRABM AND
                            DTMESCOMPETENC <= SYSDATE AND
                            (
                                EM.DTFIMVIGENCIA >= SYSDATE OR
                                EM.DTFIMVIGENCIA IS NULL
                            )
                    )
                JOIN
                GPE_PARAMORG PO ON
                    VM.NRORG = PO.NRORG
                WHERE
                    VM.NRVINCULOM = :P_NRVINCULO AND
                    VM.NRORG = :P_NRORG
SQL;

        $std = $this->connection->prepare($sql);

        $std->execute(
            array(
                'P_NRORG'           => $nrOrg,
                'P_NRVINCULO'       => $nrVinculo,
                'P_DATE'            => $date,
            )
        );

        $horarioBase = $std->fetch();

        return $horarioBase;
    }

    public function getNrParcNegocioVinculo(
        array $params
    ) {
        $sql = <<<SQL
            SELECT
                P.NRPARCNEGOCIO
            FROM
                GPE_VINCULOM VM
                JOIN
                GPE_PESSOA P ON
                    P.NRPESSOA = VM.NRPESSOA AND
                    P.NRORG = VM.NRORG
            WHERE
                VM.NRVINCULOM = :P_NRVINCULO AND
                VM.NRORG = :P_NRORG
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetch();
        $nrParcNegocio = $result['NRPARCNEGOCIO'];

        return $nrParcNegocio;
    }

    public function isDiaFeriadoVinculo(
        int     $nrOrg,
        int     $nrVinculo,
        string  $dtmescompetenc,
        string  $dia = null
    ){
        $sql = <<<SQL
            SELECT
                *
            FROM
                OCORCALENDARIO
            WHERE
                NRCALENDARIO = (
                    SELECT
                        NRCALENDARIO
                    FROM
                        ESTRUTURAH
                    WHERE
                        NRESTRUTURAM = (
                            SELECT MOV.NRESTRUTURAM
                            FROM GPE_MOVIMENTACAO MOV
                            JOIN GPE_PARAMORG PO ON PO.NRORG = MOV.NRORG
                            WHERE MOV.NRORG = :P_NRORG
                                AND MOV.NRVINCULOM = :P_NRVINCULOM
                                AND MOV.DTINIMOVIMENT <= LAST_DAY(TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY'))
                                AND (MOV.DTFIMMOVIMENT IS NULL OR MOV.DTFIMMOVIMENT >  LAST_DAY(TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')))
                                AND MOV.NRTIPOESTRUTURA IN (PO.NRTPESTRUTLEGAL, PO.NRTPESTRUTCALEND)
                            ORDER BY DECODE(MOV.NRTIPOESTRUTURA, PO.NRTPESTRUTCALEND, 1, PO.NRTPESTRUTLEGAL, 2, 3), MOV.DTINIMOVIMENT DESC
                            FETCH FIRST ROW ONLY
                        )
                        AND DTMESCOMPETENC IN (
                            SELECT
                                MAX(DTMESCOMPETENC)
                            FROM
                                ESTRUTURAH H1
                            WHERE
                                H1.NRESTRUTURAM = ESTRUTURAH.NRESTRUTURAM
                                AND H1.NRORG = ESTRUTURAH.NRORG
                                AND H1.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                        )
                        AND NRORG = :P_NRORG
                )
                AND NRORG = :P_NRORG
                AND ( TO_CHAR(DTCALENDARIO, 'DD/MM/YYYY') = :P_DTCALENDARIO
                      OR :P_DTCALENDARIO IS NULL )
SQL;

        $std = $this->connection->prepare($sql);

        $std->execute(
            array(
                "P_NRORG"           => $nrOrg,
                "P_NRVINCULOM"      => $nrVinculo,
                "P_DTMESCOMPETENC"  => $dtmescompetenc,
                "P_DTCALENDARIO"    => $dia,
            )
        );

        $result = $std->fetchAll();

        return $result;
    }

    public function setVinculo(
        array $params
    ) {
        $sql = <<<SQL
            INSERT INTO GPE_VINCULOM (
                NRVINCULOM,             NRPESSOA,           NRORG,
                DTADMISSAOVINC,         DTRESCISAOVINC,     CDMATRICULA,
                DTINCLUSAO,             DTULTATU,           NRORGINCLUSAO,
                CDOPERINCLUSAO,         NRORGULTATU,        CDOPERULTATU,
                NRTIPOVINCULOM,         NRVINCULOEMPREG,    NRTPADMISSAO,
                NRTPDEMISSAO,           DTOPCAOFGTS,        DTAPOSENTAFGTS,
                DTPRIMADMISSAO,         NMVINCULOM,         NRMOTIVORESC,
                DSOBSRESCISAO,          DTAVISOPREVIO,      NRDIASCONTRATOEXP,
                NRDIASCONTRATOEXPPRO,   DTEXAMEMEDICO,      DTAVISOTRCT,
                DSLIVROADMISSAO,        IDSEGDESEMPVINC,    NRDIASAPURVALES,
                NRTPAVIPRE,             DTENTREGAEPI,       DSOBSERVACAORESCISAO,
                IDVERIFICARESCISAO,     IDTPREINTEGRACAO,   NRPROCJUD,
                NRLEIANISTIA,           IDPAGTOJUIZO,       DTRETORNO,
                DTEFEITOFINANCEIRO,     OBSREINTEGRACAO,    DTCANCAVIPRE,
                NRMOTIVOCANCAP,         DSOBSCANCAVIPRE,    DTESOCIAL,
                DTVENCCONTREXPER,       NRCLAUSASSECURAT,   NRMOTIVOADMISSAO,
                NRVINCULOMSUBST
            )
            VALUES (
                :NRVINCULOM,            :NRPESSOA,          :NRORG,
                :DTADMISSAOVINC,        :DTRESCISAOVINC,    :CDMATRICULA,
                :DTINCLUSAO,            :DTULTATU,          :NRORGINCLUSAO,
                :CDOPERINCLUSAO,        :NRORGULTATU,       :CDOPERULTATU,
                :NRTIPOVINCULOM,        :NRVINCULOEMPREG,   :NRTPADMISSAO,
                :NRTPDEMISSAO,          :DTOPCAOFGTS,       :DTAPOSENTAFGTS,
                :DTPRIMADMISSAO,        :NMVINCULOM,        :NRMOTIVORESC,
                :DSOBSRESCISAO,         :DTAVISOPREVIO,     :NRDIASCONTRATOEXP,
                :NRDIASCONTRATOEXPPRO,  :DTEXAMEMEDICO,     :DTAVISOTRCT,
                :DSLIVROADMISSAO,       :IDSEGDESEMPVINC,   :NRDIASAPURVALES,
                :NRTPAVIPRE,            :DTENTREGAEPI,      :DSOBSERVACAORESCISAO,
                :IDVERIFICARESCISAO,    :IDTPREINTEGRACAO,  :NRPROCJUD,
                :NRLEIANISTIA,          :IDPAGTOJUIZO,      :DTRETORNO,
                :DTEFEITOFINANCEIRO,    :OBSREINTEGRACAO,   :DTCANCAVIPRE,
                :NRMOTIVOCANCAP,        :DSOBSCANCAVIPRE,   :DTESOCIAL,
                :DTVENCCONTREXPER,      :NRCLAUSASSECURAT,  :NRMOTIVOADMISSAO,
                :NRVINCULOMSUBST
            )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function setVinculoH(
        array $params
    ) {
        $sql = <<< SQL
            INSERT INTO GPE_VINCULOH (
                NRVINCULOH,             NRVINCULOM,         NRSITUFUNCM,
                NRCARGO,                NRESCALATRABM,      NRORG,
                DTMESCOMPETENC,         IDCONTRIBUISIND,    DTINCLUSAO,
                DTULTATU,               NRORGINCLUSAO,      CDOPERINCLUSAO,
                NRORGULTATU,            CDOPERULTATU,       NRDEPENDIR,
                NRDEPENDSFAM,           DTBASEFERIAS,       IDREMUNERACAO,
                IDTPPAGAMENTO,          NRESTRUTLEGAL,      NRESTRUTGEREN,
                NRTPMODALIDSAL,         IDMULTVINC,         NRFUNCAO,
                CDCONTRIBINDIVIDUAL,    NRTPMOVTRANSFM,     DTFIMCONTRDETERMIN,
                NRESTRUTSIND,           CONFIDENCIAL,       NRDIASAPURVALES,
                NRTURNO,                NRTPREGIMEPREV,     IDASSOCSINDICATO,
                CDMATRICULAESOCIAL,     NRFAIXA,            IDPAGPROPFER13,
                NRAGENTENOCIVO
            )
            VALUES (
                :NRVINCULOH,            :NRVINCULOM,        :NRSITUFUNCM,
                :NRCARGO,               :NRESCALATRABM,     :NRORG,
                :DTMESCOMPETENC,        :IDCONTRIBUISIND,   :DTINCLUSAO,
                :DTULTATU,              :NRORGINCLUSAO,     :CDOPERINCLUSAO,
                :NRORGULTATU,           :CDOPERULTATU,      :NRDEPENDIR,
                :NRDEPENDSFAM,          :DTBASEFERIAS,      :IDREMUNERACAO,
                :IDTPPAGAMENTO,         :NRESTRUTLEGAL,     :NRESTRUTGEREN,
                :NRTPMODALIDSAL,        :IDMULTVINC,        :NRFUNCAO,
                :CDCONTRIBINDIVIDUAL,   :NRTPMOVTRANSFM,    :DTFIMCONTRDETERMIN,
                :NRESTRUTSIND,          :CONFIDENCIAL,      :NRDIASAPURVALES,
                :NRTURNO,               :NRTPREGIMEPREV,    :IDASSOCSINDICATO,
                :CDMATRICULAESOCIAL,    :NRFAIXA,           :IDPAGPROPFER13,
                :NRAGENTENOCIVO
            )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function getVinculoTodos(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                VM.NRVINCULOM,
                VM.DTADMISSAOVINC,
                VM.DTRESCISAOVINC,
                VM.NMVINCULOM,
                VM.NRPESSOA,
                VM.NRORG,
                VM.NRTIPOVINCULOM,
                DECODE(VM.DTRESCISAOVINC, NULL, 0, 1) AS FILTRO
            FROM
                GPE_VINCULOM VM
            WHERE
                VM.NRORG = :P_NRORG
                AND EXISTS
                (
                    SELECT
                        1
                    FROM
                        GPE_VINCULOH
                    WHERE
                        GPE_VINCULOH.NRORG = VM.NRORG
                        AND GPE_VINCULOH.NRVINCULOM = VM.NRVINCULOM
                        AND GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                )
                AND VM.DTADMISSAOVINC <= LAST_DAY(TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY'))
                AND (
                    VM.DTRESCISAOVINC >= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') OR
                    VM.DTRESCISAOVINC IS NULL
                )
            ORDER BY
                FILTRO ASC,
                NMVINCULOM ASC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $vinculos = $std->fetchAll();

        return $vinculos;
    }

    public function getVinculoTodosResp(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                VM.NRVINCULOM,
                VM.DTADMISSAOVINC,
                VM.DTRESCISAOVINC,
                VM.NMVINCULOM,
                VM.NRPESSOA,
                VM.NRORG,
                VM.NRTIPOVINCULOM,
                DECODE(VM.DTRESCISAOVINC, NULL, 0, 1) AS FILTRO
            FROM
                GPE_VINCULOM VM
                JOIN
                GPE_VINCULOH VH ON
                    VH.NRORG = VM.NRORG AND
                    VH.NRVINCULOM = VM.NRVINCULOM AND
                    VH.DTMESCOMPETENC = (
                        SELECT
                            MAX(VH1.DTMESCOMPETENC)
                        FROM
                            GPE_VINCULOH VH1
                        WHERE
                            VH1.NRORG = VH.NRORG AND
                            VH1.NRVINCULOM = VH.NRVINCULOM AND
                            VH1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                    )
            WHERE
                VM.NRORG = :P_NRORG
                AND VM.DTADMISSAOVINC <= LAST_DAY(TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY'))
                AND EXISTS
                (
                    SELECT
                        1
                    FROM
                        GPE_VINCULOH
                    WHERE
                        GPE_VINCULOH.NRORG = VM.NRORG
                        AND GPE_VINCULOH.NRVINCULOM = VM.NRVINCULOM
                )
                AND (
                    VM.DTRESCISAOVINC > TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') OR
                    VM.DTRESCISAOVINC IS NULL
                )
                AND (INSTR(:P_NRESTRUTURAS, ',' || VH.NRESTRUTGEREN || ',')) > 0
            ORDER BY
                FILTRO ASC,
                NMVINCULOM ASC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $vinculos = $std->fetchAll();

        return $vinculos;
    }

    /**
     *  Rotina responsável por buscar a escala ativa de horários do funcionário. A escala é buscada a partir das alterações de escala registradas para o vínculo,
     * Disponíveis na tabela 'GPE_ALTEESCALA'
     *
     *
     *  @param  int     $nrOrg
     *  @param  int     $nrVinculo
     *  @param  string  $dtMesCompetenc = null
     *
     *  @return array|false
     */
    public function verificaFeriasVinculo(
        int     $nrOrg,
        int     $nrVinculo
    ) {
        $sql = <<<SQL
            SELECT 
                FG.DTINIGOZOFERIAS,
                FG.DTFIMGOZOFERIAS    
            FROM 
                FPA_FERIAS FF
            JOIN FPA_GOZOFERIAS FG
                ON FF.NRFERIAS = FG.NRFERIAS
                    AND FF.NRORG = FG.NRORG
            WHERE
                FF.NRVINCULOM = :P_NRVINCULO
                AND FF.NRORG = :P_NRORG
SQL;

        $std = $this->connection->prepare($sql);

        $std->execute(
            array(
                'P_NRORG'           => $nrOrg,
                'P_NRVINCULO'       => $nrVinculo
            )
        );

        $gozoFerias = $std->fetchAll();

        return $gozoFerias;
    }

    public function getVinculoByCPF(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                PM.NRPESSOA,
                VM.NRVINCULOM,
                VM.DTADMISSAOVINC,
                VM.DTRESCISAOVINC,
                VM.NMVINCULOM,
                VM.NRPESSOA,
                VM.NRORG,
                VM.NRTIPOVINCULOM,
                VM.IDVERIFICARESCISAO,
                DECODE(VM.DTRESCISAOVINC, NULL, 0, 1) AS FILTRO,
                CASE
                    WHEN SFH.IDAFASTSITUFUNC = 'S' OR (
                        VM.DTRESCISAOVINC IS NOT NULL
                        AND VM.DTRESCISAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
                    ) THEN 'S'
                    ELSE 'N'
                END AS IDDESLGOURESCD
            FROM
                GPE_PESSOA PM
                JOIN GPE_PESSOAH PH ON
                    PH.NRORG = PM.NRORG AND
                    PH.NRCPFPESSOA = :P_NRCPFPESSOA AND
                    PH.DTMESCOMPETENC = (
                        (SELECT DTMESCOMPETENC FROM 
                                (SELECT 
                                    PHH.DTMESCOMPETENC 
                                FROM 
                                    GPE_PESSOAH PHH 
                                WHERE 
                                PHH.NRORG = PH.NRORG AND
                                PHH.NRPESSOA = PH.NRPESSOA
                                ORDER BY PHH.DTMESCOMPETENC DESC) 
                        WHERE ROWNUM = 1)
                    )
                JOIN GPE_VINCULOM VM ON
                    VM.NRORG = PM.NRORG AND
                    VM.NRPESSOA = PM.NRPESSOA
                JOIN GPE_VINCULOH VH ON
                    VH.NRORG = VM.NRORG AND
                    VH.NRVINCULOM = VM.NRVINCULOM AND
                    VH.DTMESCOMPETENC = (
                        (SELECT DTMESCOMPETENC FROM 
                                (SELECT 
                                    VHH.DTMESCOMPETENC 
                                FROM 
                                    GPE_VINCULOH VHH 
                                WHERE 
                                VHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC AND
                                VHH.NRORG = VH.NRORG AND
                                VHH.NRVINCULOM = VH.NRVINCULOM 
                                ORDER BY VHH.DTMESCOMPETENC DESC) 
                        WHERE ROWNUM = 1)
                    )
                LEFT JOIN GPE_SITUFUNCH SFH ON ( SFH.NRORG IS NULL OR SFH.NRORG = :P_NRORG_PADRAO OR SFH.NRORG = VM.NRORG )
                AND SFH.NRSITUFUNCM = VH.NRSITUFUNCM
                AND SFH.DTEFETIVA = (
                SELECT
                        MAX(DTEFETIVA)
                    FROM
                        GPE_SITUFUNCH
                    WHERE
                            DTEFETIVA <= :P_DTMESCOMPETENC
                        AND ( SFH.NRORG IS NULL
                            OR SFH.NRORG = :P_NRORG_PADRAO
                            OR SFH.NRORG = VM.NRORG )
                        AND NRSITUFUNCM = SFH.NRSITUFUNCM
            )
            WHERE
                PM.NRORG = :P_NRORG AND
                PM.NRPESSOA = PH.NRPESSOA
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $vinculo = $std->fetch();

        return !empty($vinculo) ? $vinculo : [];
    }
}