<?php

namespace Ems\Util;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Doctrine\ORM\EntityManagerInterface;
use Util\Environment;
use Util\DateUtil;

class Utils {

    const ORDEM_ASC     = 'ASC';
    const ORDEM_DESC    = 'DESC';
    
    const SIM = 'S';
    const NAO = 'N';
    const SIM_TEXT = 'Sim';
    const NAO_TEXT = 'Não';

    const DEFAULT_FLOAT_DECIMAL_FIELDS_PRECISION = 3;
    
    const DATA_OPERATION_INSERT = 'INSERT';
    const DATA_OPERATION_UPDATE = 'UPDATE';
    const DATA_OPERATION_DELETE = 'DELETE';

    const REMOVER_ESPECIAIS = array(
        "ª" => "", "º" => "", "à" => "a","á" => "a","ã" => "a","â" => "a","é" => "e","ê" => "e","ì" => "i","í" => "i",
        "ó" => "o", "õ" => "o","ô" => "o","ú" => "u","ü" => "u","ç" => "c","À" => "A","Á" => "A","Ã" => "A","Â" => "A",
        "É" => "E","Ê" => "E","Í" => "I","Ó" => "O","Õ" => "O","Ô" => "O","Ù" => "U","Ú" => "U","Ü" => "U","." => ""
    );

    const REMOVER_INVALIDOS = array(
        "ª" => "", "º" => "", "à" => "a","á" => "a","ã" => "a","â" => "a","é" => "e","ê" => "e","ì" => "i","í" => "i",
        "ó" => "o", "õ" => "o","ô" => "o","ú" => "u","ü" => "u","ç" => "c","À" => "A","Á" => "A","Ã" => "A","Â" => "A",
        "É" => "E","Ê" => "E","Í" => "I","Ó" => "O","Õ" => "O","Ô" => "O","Ù" => "U","Ú" => "U","Ü" => "U", " " => "_",
        "." => ""
    );
    
    const ORG_TEKNISA = 20;

    const FIELDS_NOT_FILLED_BASE_MESSAGE = 'Os seguintes campos obrigatórios não foram preenchidos: ';

    public static function checkNovoCodigoIfExists(
        string  $tableName,
        bool    $considerNrorgOnMax = true
    ) {
        $nrOrg = self::getParamters()->getNrOrgTrab();

        $selectNovoCodigoExists =
        "SELECT * FROM NOVOCODIGO WHERE NRORG = $nrOrg AND CDCONTADOR = '$tableName'";

        $query = self::getEntityManager()->getConnection()->prepare($selectNovoCodigoExists);
        $query->execute();
              
        $novoCodigoTable = $query->fetchAll();
        if (!empty($novoCodigoTable)) {
            self::updateNovoCodigo($tableName, $considerNrorgOnMax);
        } else {
            self::createNovoCodigo($tableName, $considerNrorgOnMax);
        }
    }

    public static function updateNovoCodigo(
        string  $tableName,
        bool    $considerNrorgOnMax = true,
        string  $tableKey           = null
    ) {
        $tableKeyFinal = !empty($tableKey) ? $tableKey : self::getTableIDByTableName($tableName);
        $nrOrg = self::getParamters()->getNrOrgTrab();
        $nrOrgMaxCheckClause = $considerNrorgOnMax ? "AND NRORG = $nrOrg" : "";
        
        $updateNovoCodigoUpdate =
            "UPDATE NOVOCODIGO SET NRSEQUENCIAL = LPAD(NVL((SELECT MAX($tableKeyFinal) + 1 FROM $tableName WHERE 1=1 $nrOrgMaxCheckClause), 0),'20','0') WHERE NRORG = $nrOrg AND CDCONTADOR = '$tableName'";
        
        $query = self::getEntityManager()->getConnection()->prepare($updateNovoCodigoUpdate);

        return $query->execute();
    }

    private static function createNovoCodigo(
        string  $tableName,
        bool    $considerNrorgOnMax = true,
        string  $tableKey           = null
    )
    {   
        $tableKeyFinal = !empty($tableKey) ? $tableKey : self::getTableIDByTableName($tableName);
        $nrOrg = self::getParamters()->getNrOrgTrab();
        $nrOrgMaxCheckClause = $considerNrorgOnMax ? "AND NRORG = $nrOrg" : "";
        $cdOperInclusao =  self::getParamters()->getCdOperador();
        $dtUltAtu = DateUtil::getDataAtual();
        $cdOperUltAtu = self::getParamters()->getCdOperador();

        $insertNovoCodigo =
        "INSERT INTO NOVOCODIGO
        (CDCONTADOR, NRSEQUENCIAL, NRORG, CDOPERINCLUSAO, DTULTATU, CDOPERULTATU)
            VALUES
            ('$tableName',
            NVL((SELECT LPAD(MAX($tableKeyFinal) + 1,'20','0') FROM $tableName WHERE 1=1 $nrOrgMaxCheckClause), '00000000000000000000'),
            '$nrOrg',
            '$cdOperInclusao',
            '$dtUltAtu',
            '$cdOperUltAtu'
            )";

        $query = self::getEntityManager()->getConnection()->prepare($insertNovoCodigo);

        return $query->execute(); 
    }

    public static function getTableIDByTableName(
        string  $tableName
    ) {
        
        $tableKeyPrefix = 'NR';
        return $tableKeyPrefix . substr($tableName, 4);
    }
    
    public static function getInstance(string $instanceName) {
        $instanceManager = InstanceManager::getInstance();
        return $instanceManager->getService($instanceName);
    }

    public static function getInstanceByClassName(string $instanceName) {
        $shortInstanceName = basename(str_replace('\\', '/', $instanceName));
        $instanceManager = InstanceManager::getInstance();
        return $instanceManager->getService($shortInstanceName);
    }
    
    public static function getEntityManager(): ?EntityManagerInterface {
        return self::getInstance('entityManager');
    }
    
    public static function getParamters(): ?Environment {
        return self::getInstance('\Util\Environment');
    }

    protected static function getEntityPersistInstance(): ?EntityPersistInstance {
        return Utils::getInstance('doctrineEntityUtil');
    }

    public static function removeCaracteresEspeciais($str){
        return strtr($str, self::REMOVER_ESPECIAIS);
    }

    public static function removeCaracteresInvalidos($str){
        return strtr($str, self::REMOVER_INVALIDOS);
    }

    public static function fixName(
        string $name
    ): string {
        $name = trim($name);
        $name = self::removeCaracteresEspeciais($name);
        $name = strtoupper($name);

        return $name;
    }

    public static function getIdDiaSemana(string $date): string {
        $formattedDate = Utils::formatDateYMD($date);

        $dayOfWeek = date('w', strtotime($formattedDate));

        $idRelation = [
            '1000000',
            '0100000',
            '0010000',
            '0001000',
            '0000100',
            '0000010',
            '0000001',
        ];

        $idDiaSemana = $idRelation[$dayOfWeek];

        return $idDiaSemana;
    }

    public static function formatDateYMD(string $date): string {
        [$day, $month, $year] = explode('/', $date);
        $formattedDate = "{$year}-{$month}-{$day}";

        return $formattedDate;
    }

    public static function formatDateDMY(string $date): string {
        [$year, $month, $day] = explode('-', $date);
        $formattedDate = "{$day}/{$month}/{$year}";

        return $formattedDate;
    }

    public static function stringToDateTime(string $date): \DateTimeImmutable {
        $formattedDate = Utils::formatDateYMD($date);
        $dateTime = new \DateTimeImmutable($formattedDate);

        return $dateTime;
    }

    public static function addDaysToDate(string $date, int $days): string {
        $dateTime = Utils::stringToDateTime($date);
        $interval = new \DateInterval("P{$days}D");
        $newDateTime = $dateTime->add($interval);
        $newDate = $newDateTime->format('d/m/Y');

        return $newDate;
    }

    public static function subDaysFromDate(string $date, int $days): string {
        $dateTime = Utils::stringToDateTime($date);
        $interval = new \DateInterval("P{$days}D");
        $newDateTime = $dateTime->sub($interval);
        $newDate = $newDateTime->format('d/m/Y');

        return $newDate;
    }
    
    public static function areFloatsEqual($a, $b, $precision = self::DEFAULT_FLOAT_DECIMAL_FIELDS_PRECISION) {
        return abs(($a-$b)/$b) < pow(10, -1 * $precision);
    }

    public static function getSqlDateTimeString(
        string $date,
        string $time
    ): string {
        $dateTime = "$date $time:00";

        return $dateTime;
    }

    /**
     * Preenche os campos especificados da row com o valor escolhido caso não esteja definido.
     *
     * @param array $row A row a ser preenchida.
     * @param string[] $fields Os campos que, caso não estejam definidos, serão preenchidos com o valor especificado.
     * @param mixed $value O valor que preencherá os campos especificados.
     *
     * @return array A row enviada com os campos preenchidos.
     */
    public static function fillRowWithValueIfUndefined(
        array $row,
        array $fields,
        $value = null
    ): array {
        foreach ($fields as $field) {
            $row[$field] = $row[$field] ?? $value;
        }

        return $row;
    }

    /**
     * Remove os pontos e vírgulas da string.
     *
     * @param string $str A string a ter a pontuação removida.
     *
     * @return string A string passado com a pontuação removida.
     */
    public static function removePunctuation(
        string $str
    ): string {
        $search = ['.', ',', '-'];
        $str = str_replace($search, '', $str);

        return $str;
    }

    public static function currencyStringToNumber(
        string $value
    ) {
        $value = str_replace(['R', 'r', '.', '$'], '', $value);
        $value = str_replace(',', '.', $value);

        $valueNumber = floatval($value);

        return $valueNumber;
    }

    public static function getIdOcorrenciaDescription(
        string $idConfirmada
    ) {
        $relation = [
            'P' => 'Pendente',
            'A' => 'Aprovada',
            'R' => 'Rejeitada',
        ];

        $description = $relation[$idConfirmada] ?? false;

        return $description;
    }
    
    public static function decompoeNomePessoa($nmpessoa)
    {
        $arrayNmpessoa = explode(' ', $nmpessoa);
        $nomeMeio      = '';

        $primeiroNome = $arrayNmpessoa[0];
        for ($i = 1; $i < count($arrayNmpessoa) - 1; $i++) {
            $nomeMeio .= ' ' . $arrayNmpessoa[$i];
            $nomeMeio = trim($nomeMeio);
        }
        $ultimoNome = $arrayNmpessoa[count($arrayNmpessoa) - 1];
        $nomeMeio   = ($nomeMeio) ? : $ultimoNome;
        return array($primeiroNome, $nomeMeio, $ultimoNome);
    }
    
    public static function getPronomeTratamento($cdEstadoCivil,$idSexoPessoa) {
        if ($idSexoPessoa == 'M'){
            $pronomeTratamento = 'Sr.';
            return $pronomeTratamento;
        }
        else if ($idSexoPessoa == 'F' && $cdEstadoCivil == 'C'){
            $pronomeTratamento = 'Sra.';
            return $pronomeTratamento;
        }
        else {
            $pronomeTratamento = 'Srta.';
            return $pronomeTratamento;
        }
    }
    
    public static function sanitizeCPF(string $cpf): string {
        return str_replace(array('.','/','-'), '', $cpf);
    }
    
    public static function sanitizeUrl(string $url): string {
        return strtr(
            $url,
            [' ' => '%20']
        );
    }
    
    /**
     * Transforma um array em uma string a ser usada em consultas que esperam um conjunto de valores separados por caracter
     * @param array $list
     * @return string
     */
    public static function arrayToStringForQueryParam(?array $list): ?string {
        if (empty($list)) {
            return null;
        }
        return ',' . implode(',', $list) . ',';
    }

    /**
     *  Rotina que verifica se todos os campos informados estão preenchidos no array informado
     * 
     *  @param  Array   $row
     *  @param  Array   $atLeastOneFields
     *  @param  bool    $throwError         = true
     * 
     *  @return bool
     * 
     *  @throws \Exception
     * 
     */
    public static function checkRequiredFields(Array $row, Array $requiredFields, bool $throwError = true) {
        $fieldsNotFilled = [];
        foreach ($requiredFields as $field) {
            if (!isset($row[$field]) || empty($row[$field])) {
                $fieldsNotFilled[] = $field;
            }
        }
        
        $requiredFieldsNotFilled = count($fieldsNotFilled) > 0;
        if ($requiredFieldsNotFilled && $throwError) {
            throw new \Exception(self::FIELDS_NOT_FILLED_BASE_MESSAGE . implode(', ', $fieldsNotFilled));
        }
        
        return ['isValid' => !$requiredFieldsNotFilled, 'fields' => $fieldsNotFilled];
    }

    /**
     * Indica se o conteudo em questao nao eh vazio para o conteudo de uma foto
     * @param string photoContent
     * @return bool
     */
    public static function isPhotoNotEmpty(string $photoContent): bool {
        return !empty($photoContent) && strlen($photoContent) > 10;
    }


    /**
     * Adiciona novos campos em um array com outros nomes ou de acordo com callbacks
     * @param array $data
     * @param array $fieldsToNewFields
     * @return array
     */
    public static function addFieldsToArray(
        array $data,
        array $fieldsToNewFields
    ): array {
        $newData = $data;
        foreach ($newData as &$newDataItem) {
            foreach ($fieldsToNewFields as $newField => $currentField) {
                /* Se a parametrizacao preve uma acao que retornara o valor do campo */
                if (is_callable($currentField)) {
                    $getDataForNewField = $currentField;
                    $newDataItem[$newField] = $getDataForNewField(
                        $newDataItem,
                        $newData
                    );
                    continue;
                }

                /* Se a parametrizacao indicar somente o nome de outro campo */
                $newDataItem[$newField] = $newDataItem[$currentField] ?? null;
            }
        }
        return $newData;
    }

    /**
     * Traduz os identificadores de Sim ou Nao para seus textos completos
     * @param ?string $value
     * @return string
     */
    public static function getSOrNText(?string $value): string {
        switch ($value) {
            case self::SIM:
                return self::SIM_TEXT;
            default:
                return self::NAO_TEXT;
        }
    }
}