<?php

namespace HCM\FPA\ObrigacaoLegal\CNAB\Santander;

use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;
use HCM\Geral\Entity\GpepessoaEntity;
use HCM\Geral\Entity\GpevinculomEntity;
use HCM\Geral\Entity\GpevinculoempregEntity;
use HCM\Geral\Entity\GpeOcupacaomEntity;
use HCM\Geral\Entity\GpemovimentacaoEntity;
use HCM\Geral\Entity\GpetpmovtransfmEntity;
use HCM\Geral\Entity\GpetpmovtransfhEntity;
use HCM\Geral\Entity\GpeGrauinstrEntity;
use HCM\Geral\Entity\GpeescalatrabmEntity;
use HCM\Geral\Entity\GpeRacapessoaEntity;
use HCM\Geral\Entity\GpeCondfispesEntity;
use HCM\Geral\Entity\GpeCboEntity;
use HCM\Geral\Entity\EnderecoparcEntity;
use HCM\FPA\Calculos\Geral\Estruturacao;
use HCM\Geral\Entity\EstruturamEntity;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FpaTpdemissaoEntity;
use HCM\FPA\Entity\FpaeventomEntity;
use HCM\FPA\Entity\FpaFormcnabEntity;
use HCM\FPA\ObrigacaoLegal\CNAB\Santander\registro\Registro0CNABSantander;
use HCM\FPA\ObrigacaoLegal\CNAB\Santander\registro\Registro1CNABSantander;
use HCM\FPA\ObrigacaoLegal\CNAB\Santander\registro\Registro3ACNABSantander;
use HCM\FPA\ObrigacaoLegal\CNAB\Santander\registro\Registro3BCNABSantander;
use HCM\FPA\ObrigacaoLegal\CNAB\Santander\registro\Registro5CNABSantander;
use HCM\FPA\ObrigacaoLegal\CNAB\Santander\registro\Registro9CNABSantander;
use HCM\FPA\Registro\RegistroCNAB;
use HCM\FPA\Registro\FPARegistro;

class CNABSantander{

    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_mesCompetencia;
    private $_dtInicio;
    private $_dtFim;
    private $_dtAux;
    private $_estrutura;
    private $_movimentacao;
    private $_estruturacao;
    private $_tpMovTransfM;
    private $_tpMovTransfH;
    private $_itemGrupo;
    private $_listaCalculoFolha;
    private $_evento;
    private $_registro0;
    private $_registro1;
    private $_registro3A;
    private $_registro3B;
    private $_registro5;
    private $_registro9;
    private $_cnabMagnetico;
    private $_totalVinculos;
    private $_dadoVinculo = array();
    private $_cnabTeste;
    private $_codConvenio;
    private $_itemCalculo;
    private $_pessoa;
    private $_vinculo;
    private $_vinculoEmpreg;
    private $_ocupacao;
    private $_tpDemissao;
    private $_grauInstrucao;
    private $_escalaTrabalho;
    private $_raca;
    private $_condicaoFisica;
    private $_cbo;
    private $_enderecoParc;

    private $_nrTpMovimentoFormulario;

    private $_cdOperador;

    const CODIGO_BANCO_SANTANDER = '033';

    public function __construct(
        $nrOrg, $nrOrgPadrao, $mesCompetencia, $cnabMagnetico, $dataSet, $entityManager, $hcmParameters
    ){


        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->_nrOrg = $nrOrg;
        $this->_nrOrgPadrao = $nrOrgPadrao;
        
        $this->_mesCompetencia = $mesCompetencia;

        $this->_dtInicio = $this->_mesCompetencia;
        $this->_dtFim = DateUtil::getUltimoDiaDoMes($this->_mesCompetencia);
        
        $this->_itemCalculo = new FpaitecalcfolhaEntity($entityManager, $hcmParameters);
        $this->_pessoa = new GpepessoaEntity($entityManager, $hcmParameters);
        $this->_vinculo = new GpevinculomEntity($entityManager, $hcmParameters);
        $this->_vinculoEmpreg = new GpevinculoempregEntity($entityManager, $hcmParameters);
        $this->_ocupacao = new GpeOcupacaomEntity($entityManager, $hcmParameters);
        $this->_tpDemissao = new FpaTpdemissaoEntity($entityManager, $hcmParameters);
        $this->_movimentacao = new GpemovimentacaoEntity($entityManager, $hcmParameters);
        $this->_tpMovTransfM = new GpetpmovtransfmEntity(
            $entityManager, $hcmParameters
        );
        $this->_tpMovTransfH = new GpetpmovtransfhEntity(
            $entityManager, $hcmParameters
        );
        $this->_grauInstrucao = new GpeGrauinstrEntity($entityManager, $hcmParameters);
        $this->_escalaTrabalho = new GpeescalatrabmEntity($entityManager, $hcmParameters);
        $this->_raca = new GpeRacapessoaEntity($entityManager, $hcmParameters);
        $this->_condicaoFisica = new GpeCondfispesEntity($entityManager, $hcmParameters);
        $this->_cbo = new GpeCboEntity($entityManager, $hcmParameters);
        $this->_enderecoParc = new EnderecoparcEntity($entityManager, $hcmParameters);
        $this->_evento = new FpaeventomEntity($entityManager, $hcmParameters);
        $this->_estruturacao = new Estruturacao($entityManager, $hcmParameters);
        $this->_estrutura = new EstruturamEntity($entityManager, $hcmParameters);

        $this->_cnabMagnetico = $cnabMagnetico;

        $this->_registro0 = new Registro0CNABSantander();
        $this->_registro1 = new Registro1CNABSantander();
        $this->_registro3A = new Registro3ACNABSantander();
        $this->_registro3B = new Registro3BCNABSantander();
        $this->_registro5 = new Registro5CNABSantander();
        $this->_registro9 = new Registro9CNABSantander();

        $this->_cdOperador = $hcmParameters->getCdOperador();

        $this->_inicializaVariaveis($dataSet);
    }

    private function _inicializaVariaveis($dataSet){

        $this->_totalVinculos = array();
        $this->_nrTpMovimentoFormulario = $dataSet['NRTIPOMOVIMENT'];
        $this->_cnabTeste = $dataSet['IDTESTE'];
        $this->_codConvenio = $dataSet['CODCONVENIO'];
    }

    public function geraCNAB($estrutura, $pessoa, $vinculo){
        $this->_estrutura = $estrutura;
        $this->_pessoa = $pessoa;
        $this->_vinculo = $vinculo;
        $this->_preparaRegistro0();
        $this->_preparaRegistro1();
        $this->_preparaRegistro3A();
        $this->_preparaRegistro3B();
        $this->_preparaRegistro5();
        $this->_preparaRegistro9();
    }

    private function _preparaRegistro0() {

        $posicao = $this->_cnabMagnetico->procuraRegistroLista(
            $this->_estrutura->nrInscricao, Registro0CNABSantander::REGISTRO_0, self::CODIGO_BANCO_SANTANDER,
            $this->_nrTpMovimentoFormulario, $this->_cdOperador, $this->_nrOrg
        );

        if ($posicao === false) {
            if ($this->_cnabTeste === FpaFormcnabEntity::IDTESTE_NAO) {
                $ultimoSequencial = $this->_cnabMagnetico->retornaUltimoSequencialRegistro0(
                    $this->_mesCompetencia, $this->_nrOrg, self::CODIGO_BANCO_SANTANDER, $this->_cdOperador
                );
                if ($ultimoSequencial) {
                    $sequencia = $ultimoSequencial;
                    $sequencia++;
                } else {
                    $sequencia = FPARegistro::CARACTER_ONZE;
                }
            } else {
                $sequencia = FPARegistro::CARACTER_UM;
            }
            $this->_registro0->setSequencia((string)$sequencia);
            $this->_registro0->setEstrutura($this->_estrutura);
            $this->_registro0->setCodConvenioBanco($this->_codConvenio);

            $this->_registro0->geraRegistro0();
            
            $this->_cnabMagnetico->insereRegistroLista(
                $this->_nrOrg, $this->_cdOperador, $this->_mesCompetencia, Registro0CNABSantander::REGISTRO_0, $this->_estrutura->nrTpInscricao,
                $this->_estrutura->nrInscricao, $this->_registro0->getLinhaRegistro(), self::CODIGO_BANCO_SANTANDER,
                $this->_nrTpMovimentoFormulario, $sequencia
            );

            $this->_registro0->setLinhaRegistro();
        }
    }

    private function _preparaRegistro1() {

        $posicao = $this->_cnabMagnetico->procuraRegistroLista(
            $this->_estrutura->nrInscricao, Registro1CNABSantander::REGISTRO_1, self::CODIGO_BANCO_SANTANDER,
            $this->_nrTpMovimentoFormulario, $this->_cdOperador, $this->_nrOrg
        );

        if ($posicao === false) {

            $this->_registro1->setEstrutura($this->_estrutura);
            $this->_registro1->setCodConvenioBanco($this->_codConvenio);

            $this->_registro1->geraRegistro1();

            $this->_cnabMagnetico->insereRegistroLista(
                $this->_nrOrg, $this->_cdOperador, $this->_mesCompetencia, Registro1CNABSantander::REGISTRO_1, $this->_estrutura->nrTpInscricao,
                $this->_estrutura->nrInscricao, $this->_registro1->getLinhaRegistro(), self::CODIGO_BANCO_SANTANDER,
                $this->_nrTpMovimentoFormulario
            );

            $this->_registro1->setLinhaRegistro();
        }
    }

    private function _preparaRegistro3A() {

        $this->_registro3A->setPessoa($this->_pessoa);
        $this->_registro3A->setVinculo($this->_vinculo);
        $this->_registro3A->setCnpjEmpresa($this->_estrutura->nrInscricao);

        $this->_registro3A->geraRegistro3A();

        $this->_cnabMagnetico->persisteRegistro(
            $this->_nrOrg, $this->_mesCompetencia, RegistroCNAB::REGISTRO_3, Registro3ACNABSantander::SEGMENTO_A,
            $this->_estrutura->nrTpInscricao, $this->_estrutura->nrInscricao, $this->_registro3A->getLinhaRegistro(),
            $this->_vinculo->getNrvinculom(), self::CODIGO_BANCO_SANTANDER, $this->_nrTpMovimentoFormulario,
            $this->_vinculo->valorPagamento, $this->_pessoa->nrTpInscricao, $this->_pessoa->historico->getNrcpfpessoa(), $this->_cdOperador
        );

        $this->_registro3A->setLinhaRegistro();
    }

    private function _preparaRegistro3B() {

        $this->_registro3B->setPessoa($this->_pessoa);
        $this->_registro3B->setVinculo($this->_vinculo);

        $this->_registro3B->geraRegistro3B();

        $this->_cnabMagnetico->persisteRegistro(
            $this->_nrOrg, $this->_mesCompetencia, RegistroCNAB::REGISTRO_3, Registro3BCNABSantander::SEGMENTO_B,
            $this->_estrutura->nrTpInscricao, $this->_estrutura->nrInscricao, $this->_registro3B->getLinhaRegistro(),
            $this->_vinculo->getNrvinculom(), self::CODIGO_BANCO_SANTANDER, $this->_nrTpMovimentoFormulario,
            $this->_vinculo->valorPagamento, $this->_pessoa->nrTpInscricao, $this->_pessoa->historico->getNrcpfpessoa(), $this->_cdOperador
        );

        $this->_registro3B->setLinhaRegistro();
    }

    private function _preparaRegistro5() {

        $posicao = $this->_cnabMagnetico->procuraRegistroLista(
            $this->_estrutura->nrInscricao, Registro5CNABSantander::REGISTRO_5, self::CODIGO_BANCO_SANTANDER,
            $this->_nrTpMovimentoFormulario, $this->_cdOperador, $this->_nrOrg
        );

        if ($posicao === false) {

            $this->_registro5->geraRegistro5();

            $this->_cnabMagnetico->insereRegistroLista(
                $this->_nrOrg, $this->_cdOperador, $this->_mesCompetencia, Registro5CNABSantander::REGISTRO_5, $this->_estrutura->nrTpInscricao,
                $this->_estrutura->nrInscricao, $this->_registro5->getLinhaRegistro(), self::CODIGO_BANCO_SANTANDER,
                $this->_nrTpMovimentoFormulario
            );

            $this->_registro5->setLinhaRegistro();
        }
    }

    private function _preparaRegistro9() {

        $posicao = $this->_cnabMagnetico->procuraRegistroLista(
            $this->_estrutura->nrInscricao, Registro9CNABSantander::REGISTRO_9, self::CODIGO_BANCO_SANTANDER
            , $this->_nrTpMovimentoFormulario, $this->_cdOperador, $this->_nrOrg
        );

        if ($posicao === false) {

            $this->_registro9->geraRegistro9();

            $this->_cnabMagnetico->insereRegistroLista(
                $this->_nrOrg, $this->_cdOperador, $this->_mesCompetencia, Registro9CNABSantander::REGISTRO_9, $this->_estrutura->nrTpInscricao,
                $this->_estrutura->nrInscricao, $this->_registro9->getLinhaRegistro(), self::CODIGO_BANCO_SANTANDER,
                $this->_nrTpMovimentoFormulario
            );

            $this->_registro9->setLinhaRegistro();
        }
    }

    public function modificaLinhasConsolidacaoSantander($tipoRegistro, $linha, $valorTotalPagamento){

        if ($tipoRegistro === Registro5CNABSantander::REGISTRO_5) {

            // Quantidade de Registros do Lote.
            $qtdRegistrosLote = $this->_cnabMagnetico->totalRegistrosPorTipos(
                $this->_mesCompetencia, $this->_nrOrg, RegistroCNAB::LISTA_TIPO_REGISTROS_LOTE,
                self::CODIGO_BANCO_SANTANDER, $this->_nrTpMovimentoFormulario, $this->_cdOperador
            );

            // Bloco 5 | 018-023 | Quantidade de Registros do Lote (Registros Tipo 1, 3, 5).
            $linha = $this->_registro5->alteraDado(
                $linha, (string)$qtdRegistrosLote, (18 -1), 6, Registro5CNABSantander::CARACTER_ZERO, Registro5CNABSantander::PREENCHE_ESQUERDA
            );

            // Valor obtido pela somatoria dos valores de Creditos / Pagamentos dos registros de detalhe (Registros do Tipo 3 / Segmento A).
            $linha = $this->_registro5->alteraDado(
                $linha, $valorTotalPagamento, (24 -1), 18, Registro5CNABSantander::CARACTER_ZERO, Registro5CNABSantander::PREENCHE_ESQUERDA
            );

            // Bloco 7 | 042-059 | Somatoria Quantidade Moeda (Registro Tipo 3). Mesmo valor do bloco anterior (6).
            $linha = $this->_registro5->alteraDado(
                $linha, $valorTotalPagamento, (42 -1), 18, Registro5CNABSantander::CARACTER_ZERO, Registro5CNABSantander::PREENCHE_ESQUERDA
            );
        } elseif ($tipoRegistro === Registro9CNABSantander::REGISTRO_9) {

            // Quantidade de lotes do arquivo.
            $qtdLotesArquivo = $this->_cnabMagnetico->totalRegistrosPorTipos(
                $this->_mesCompetencia, $this->_nrOrg, RegistroCNAB::LISTA_TIPO_REGISTRO_CABECALHO_LOTE,
                self::CODIGO_BANCO_SANTANDER, $this->_nrTpMovimentoFormulario, $this->_cdOperador
            );

            // Quantidade de registros no arquivo.
            $qtdRegistrosArquivo = $this->_cnabMagnetico->totalRegistrosPorTipos(
                $this->_mesCompetencia, $this->_nrOrg, RegistroCNAB::LISTA_TIPO_REGISTROS_ARQUIVO,
                self::CODIGO_BANCO_SANTANDER, $this->_nrTpMovimentoFormulario, $this->_cdOperador
            );

            // Bloco 5 | 018-023 | Quantidade de lotes do arquivo (Registros Tipo 1).
            $linha = $this->_registro9->alteraDado(
                $linha, (string)$qtdLotesArquivo, (18 -1), 6, Registro9CNABSantander::CARACTER_ZERO, Registro9CNABSantander::PREENCHE_ESQUERDA
            );

            // Bloco 6 | 024-029 | Quantidade de registros no arquivo (Registros Tipo 0, 1, 3, 5 e 9).
            $linha = $this->_registro9->alteraDado(
                $linha, (string)$qtdRegistrosArquivo, (24 -1), 6, Registro9CNABSantander::CARACTER_ZERO, Registro9CNABSantander::PREENCHE_ESQUERDA
            );
        }
        return $linha;
    }
}