<?php

namespace HCM\FPA\ObrigacaoLegal\Rais;

use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use HCM\Util\FPA;
use Geral\Util\NovoCodigo;
use HCM\FPA\ObrigacaoLegal\Rais\Entities\Tipo00Base;
use Teknisa\Model\Gestaopessoas\Entities\FpaRaismagnet;
use \Setforms2\Shared\ORM\DateTime;


class Tipo00 extends Tipo00Base
{
    private $nrorg;
    private $ano;
    public $tipo01;
    public $tipo09;
    private $qtd02;
    private $nrformrais;
    private $idProjetaRescisao;



    function __construct($hcmParameters,$entityManager,$responsEstrut, $responsVinc, $nrorg, $ano, $nrformrais, $retifica = false, $idProjetaRescisao = 0)
    {
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager;
        $this->nrorg = $nrorg;
        $this->ano = $ano;
        $this->setDefaultInfo($retifica);
        $this->setRespDataVinc($responsVinc);
        $this->setRespDataEst($responsEstrut);
        $this->qtd02 = 0;
        $this->nrformrais = $nrformrais;
        $this->idProjetaRescisao = $idProjetaRescisao;
    }

    private function setPriEmpresaInfo()
    {
        $info = $this->tipo01[0]->getInfoZero();
        $this->cnpjPrimeiro = $info['CNPJ'];
        $this->prefixo = $info['PREFIXO'];

    }
    
    private function setDefaultInfo($retifica)
    {
        $this->espacos = '';
        $this->espacos2 = '';
        $this->tipoRegistro = 0;
        $this->constante = 1;
        $this->dataGeracao = DateUtil::getDataAtual();
        if ($retifica == 'S') {
            $this->retificacao = 1;
            $this->dataRetificacao = DateUtil::getDataAtual();
        } else {
            $this->retificacao = 2;
        }
    }

    private function setRespDataVinc( $pessoah)
    {
        try {
            $email = $this->entityManager->getRepository(Repositories::COMUNICAPARC)->findOneBy(
                array(
                'nrparcnegocio' => $pessoah->getNrparcnegocio(),
                'nrorg' => $this->nrorg,
                'cdformacomu' => self::FORMACOMU_EMAIL
                )
            );
            if(!$email){
                throw new \Exception("Não foi possível encontrar o e-mail para o (a) responsável pela RAIS", 10);
                
            }
            $this->cpfResp = $pessoah->historico->getNrcpfpessoa();
            $this->nomeResp = $pessoah->historico->getNmpessoa();
            $this->dataNascResp = $pessoah->historico->getDtnascpessoa();
            $this->emailResp = ($email ? $email->getCdcomunicaparc() : null);
            if ($pessoah->historico->getNrconselhoreg() == self::NUM_ID_CREA) {
                $this->creaRetif = $pessoah->getNrinscrconsreg();
            }
        } catch (Exception $e) {
            throw $e;
        }

    }

    private function setRespDataEst( $estruturah)
    {
        try {
            $estruturam = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findOneBy(
                array(
                'nrestruturam' => $estruturah->getNrestruturam(),
                'nrorg' => $estruturah->getNrorg()
                )
            );
            $endereco = $this->entityManager->getRepository(Repositories::ENDERECOPARC)->findOneBy(
                array(
                'nrparcnegocio' => $estruturam->getNrparcnegocio(),
                'nrorg' => $this->nrorg
                )
            );
            $telefone = $this->entityManager->getRepository(Repositories::COMUNICAPARC)->findOneBy(
                array(
                'nrparcnegocio' => $estruturam->getNrparcnegocio(),
                'nrorg' => $this->nrorg,
                'cdformacomu' => self::FORMACOMU_TEL
                )
            );
            if (!is_null($estruturah->getCdcnpjestrut())) {
                $this->cnpj = $estruturah->getCdcnpjestrut();
                $this->tipoInscricao = 1;
            } else {
                $this->cnpj = $estruturah->getCdceiestrut();
                $this->tipoInscricao = 3;
            }
            $this->razaoSocial = $estruturah->getNmrazsocestrut();
            if ($endereco) {
                $this->logradouro = $endereco->getDsendereco();
                $this->numero = $endereco->getNrenderecoparc();
                $this->complemento = $endereco->getDscompleendereco();
                $this->uf = $endereco->getSgestado();
                $this->cep = $endereco->getNrcependereco();
                $this->bairro = $endereco->getNmbairroendereco();
                
                if ($endereco->getCdmunicipio()) {
                    $municipio = $this->entityManager->getRepository(Repositories::MUNICIPIO)->findOneBy(
                        array(
                        "cdmunicipio" => $endereco->getCdmunicipio(),
                        "cdpais" => $endereco->getCdpais(),
                        "sgestado" => $endereco->getSgestado()
                            )
                        );
                    $this->codMunicipio = $municipio->getCdmunicibge();
                    $this->municipio = $municipio->getNmmunicipio();
                }else{
                    throw new \Exception("Não foi possível encontrar o municipio para a estrutura".$estruturam->getNrestruturam()."-".$estruturam->getNmestruturam(), 10);
                }
            } else {
                throw new \Exception("Não foi possível encontrar o endereço da estrutura responsável pela RAIS", 10);
            }
            if ($telefone) {
                $this->ddd = $telefone->getCdprefixcomuparc();
                $this->numeroTel = $telefone->getCdcomunicaparc();
            }else{
                throw new \Exception("Não foi possível encontrar o telefone da estrutura responsável pela RAIS", 10);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function setTipo01($nrestruturasm = array() , $centralPgto = null, $sindicato = null) {
        
        try {
            $sindicato = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findBy(
                array(
                'nrestruturam' => $sindicato,
                'nrorg' => $this->nrorg
                )
            );
            if(!$sindicato){
                throw new \Exception("Não foi possível encontrar histórico para a estrutura sindical patronal", 10);
            }

            if($nrestruturasm) {
                $estruturas = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findBy(array(
                    'nrorg'=>$this->nrorg,
                    'nrtipoestrutura'=>FPA::getTipoEstruturaLegalOrg(),
                    'nrestruturam'=> $nrestruturasm
                ));
            }else {
                $estruturas = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findBy(array(
                    'nrorg'=>$this->nrorg,
                    'nrtipoestrutura'=>FPA::getTipoEstruturaLegalOrg()
                ));
            }
            if ($estruturas) {
                foreach ($estruturas as $estruturam) {
                    $estruturah = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaHistorico(
                        $estruturam->getNrorg(),
                        $estruturam->getNrestruturam(),
                        $this->hcmParameters->getDtCompetencia()
                    );

                    if(!$estruturah){
                        throw new \Exception("Não foi possível encontrar um histórico para a estrutura ".$estruturam->getNrestruturam()."-".$estruturam->getNmestruturam(), 10);
                    }
                    if ($estruturam->getNrestruturam() == $centralPgto) {
                        $centralPgto = true;
                    }

                    $this->tipo01[] = new Tipo01($this->hcmParameters,$this->entityManager,$estruturam, $estruturah, $this->ano, $centralPgto, $sindicato, $this->nrformrais, $this->idProjetaRescisao);
                }
                $this->setPriEmpresaInfo();
            }else{
                throw new \Exception("Nenhuma estrutura foi retornada para o filtro de busca selecionado", 10);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function setTipo02($grupoIncidencias, $vinculos) {
        foreach($this->tipo01 as $tipo01){
            $nrcalc = $tipo01->setRais02($grupoIncidencias, $vinculos);
            $this->qtd02 += $tipo01->getQuantDois();
        }
        return $nrcalc;
    }

    public function setTipo09() {
        $tipo01 = end($this->tipo01);
        $info = $tipo01->getInfoZero();
        $this->tipo09 = new Tipo09($info['CNPJ'], count($this->tipo01), $this->qtd02, $info['PREFIXO']);

    }


    public function getMagnetico($idgeramagnet = null) {
        $linhas = array();
        $linhas[] = $this->getRow();
        foreach($this->tipo01 as $tipo01){
            $linhas[] = $tipo01->getRow();
            $tipo02mags = $tipo01->getTipo02();
            for($n = count($tipo02mags), $i = 0; $i < $n; ++$i){
                $linha = $tipo02mags[$i];
                $linhas[] = $linha;
                $dados = $tipo01->tipo02[$i];
                if($idgeramagnet == 'S') {
                    $this->insereMagnetico($linha, $dados->getCnpj(), $dados->getCodigoPisPasep(),
                        $dados->getDataAdmissao(), $dados->getTipoRegistro(), $dados->getNrvinculom());
                }
            }
            if($idgeramagnet == 'S'){
                $this->entityManager->flush();
            }
        }
        $linhas[] = $this->tipo09->getRow();
        return $linhas;
    }


    public function insereMagnetico($dslinharegistro = null, $cdinscestrut = null, $nrpispaseppes = null,
                                    $dtadmissaovinc = null, $idtiporegistro = null, $nrvinculom = null){ 
        $obj = new FpaRaismagnet;

        if(!($dtadmissaovinc instanceof DateTime)){
            $dtadmissaovinc = DateUtil::getDataDeString($dtadmissaovinc);
        }
        $nrraismagnet     = NovoCodigo::geraCodigo($this->nrorg,'FPA_RAISMAGNET'.$this->nrorg, 1, 22, true ); 
        $dtmescompetencia = DateUtil::getDataDeString(FPA::getDtCompetencia());
        $dtinclusao       = DateUtil::getDataAtual();
        $cdoperador       = $this->hcmParameters->getCdoperador();

        $obj->setNrraismagnet($nrraismagnet);
        $obj->setNrorg($this->nrorg);
        $obj->setMescompetencia($dtmescompetencia);
        $obj->setIdativo('S');
        $obj->setDtinclusao($dtinclusao);
        $obj->setDtultatu($dtinclusao);
        $obj->setCdoperinclusao($cdoperador);
        $obj->setCdoperultatu($cdoperador);
        $obj->setNrorginclusao($this->nrorg);
        $obj->setNrorgultatu($this->nrorg);

        $obj->setDslinharegistro($dslinharegistro);
        $obj->setCdinscestrut($cdinscestrut);
        $obj->setNrpispaseppes($nrpispaseppes);
        $obj->setDtadmissaovinc($dtadmissaovinc);
        $obj->setIdtiporegistro($idtiporegistro);
        $obj->setNrvinculom($nrvinculom);

        $this->entityManager->persist($obj);
    }
} 