<?php

namespace HCM\Geral\Services;

use Zeedhi\Framework\HTTP\Kernel;
use HCM\VAB\IntegracaoVT\Controlador;
use HCM\Util\DateUtil;
use HCM\Geral\Services\FileService;
use HCM\Util\FPA;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Geral\Services\UploadFileService;
use Doctrine\ORM\EntityManager;



class ConsultaQualificaService {

    protected $hcmParameters;

    public function __construct(Kernel $httpKernel,HCMParameters $hcmParameters,EntityManager $entitymanager, FPA $fpa, FileService $fileService,  UploadFileService $uploadsFileService) {
        $this->httpKernel = $httpKernel;
        $this->entityManager = $entitymanager;
        $this->hcmParameters = $hcmParameters;
        $this->fpa = $fpa;
        $this->fileService = $fileService;
        $this->uploadsFileService = $uploadsFileService;
    }
    
    private static $layout = array(
        'P' => array('CPF', 'NIS', 'NOME', 'DN', 'COD_NIS_INV', 'COD_CPF_INV', 'COD_NOME_INV', 'COD_DN_INV', 'COD_CNIS_NIS', 'COD_CNIS_DN', 'COD_CNIS_OBITO', 'COD_CNIS_CPF', 'COD_CNIS_CPF_NAO_INF', 'COD_CPF_NAO_CONSTA', 'COD_CPF_NULO', 'COD_CPF_CANCELADO', 'COD_CPF_SUSPENSO', 'COD_CPF_DN', 'COD_CPF_NOME', 'COD_ORIENTACAO_CPF', 'COD_ORIENTACAO_NIS'),
        'R' => array('CPF', 'NIS', 'NOME', 'DN', 'COD_CPF_INV', 'COD_NIS_INV', 'COD_NOME_INV', 'COD_DN_INV', 'SEPARADOR', 'REG_DESFORMATADO'                                                                                                                                                                                                                 ));
    
    private static $dominio = array(
        'P' => array(
            'COD_NIS_INV'          => array( '0' => null, '1' => 'NIS inválido' ),
            'COD_CPF_INV'          => array( '0' => null, '1' => 'CPF inválido' ),
            'COD_NOME_INV'         => array( '0' => null, '1' => 'NOME inválido' ),
            'COD_DN_INV'           => array( '0' => null, '1' => 'DN inválida' ),
            'COD_CNIS_NIS'         => array( '0' => null, '1' => 'NIS inconsistente' ),
            'COD_CNIS_DN'          => array( '0' => null, '1' => 'Data de Nascimento informada diverge da existente no CNIS' ),
            'COD_CNIS_OBITO'       => array( '0' => null, '1' => 'NIS com óbito no CNIS' ),
            'COD_CNIS_CPF'         => array( '0' => null, '1' => 'CPF informado diverge do existente no CNIS' ),
            'COD_CNIS_CPF_NAO_INF' => array( '0' => null, '1' => 'CPF não preenchido no CNIS' ),
            'COD_CPF_NAO_CONSTA'   => array( '0' => null, '1' => 'CPF informado não consta o Cadastro CPF' ),
            'COD_CPF_NULO'         => array( '0' => null, '1' => 'CPF informado NULO no Cadastro CPF' ),
            'COD_CPF_CANCELADO'    => array( '0' => null, '1' => 'CPF informado CANCELADO no Cadastro CPF' ),
            'COD_CPF_SUSPENSO'     => array( '0' => null, '1' => 'CPF informado SUSPENSO no Cadastro CPF' ),
            'COD_CPF_DN'           => array( '0' => null, '1' => 'Data de Nascimento informada diverge da existente no Cadastro CPF' ),
            'COD_CPF_NOME'         => array( '0' => null, '1' => 'NOME informado diverge do existente no Cadastro CPF (Obs: Informa o nome que consta no CPF)' ),
            'COD_ORIENTACAO_CPF'   => array( '0' => null, '1' => 'Conveniadas da RFB: Correios, Banco do Brasil ou CAIXA' ),
            'COD_ORIENTACAO_NIS'   => array( '0' => null, '1' => 'NIS no INSS – Ligar 135 para agendar atendimento', '2' => 'Atualizar o Cadastro NIS da CAIXA – Utilizar Cadastro NIS Empresa pelo Conectividade Social ou uma agência da CAIXA', '3' => 'Atualizar o Cadastro NIS em uma agência do Banco do Brasil')),
        'R' => array(
            'COD_CPF_INV'      => array( '0' => null, '1' => 'CPF inconsistente'),
            'COD_NIS_INV'      => array( '0' => null, '1' => 'NIS inconsistente'),
            'COD_NOME_INV'     => array( '0' => null, '1' => 'NOME inconsistente'),
            'COD_DN_INV'       => array( '0' => null, '1' => 'DN inconsistente'),
            'SEPARADOR'        => array( '0' => null, '1' => 'SEPARADOR INVÁLIDO'),
            'REG_DESFORMATADO' => array( '0' => null, '1' => 'FORMATAÇÃO inválida'))
    );

    public function importarCQC($row) {
        try {
            $arquivo = $row['DSPATH'][0];
            $file = $this->httpKernel->factoryFromFileData($arquivo, FPA::getTempDirectory()."/teste");
    
            if (!$file->isFile()) {
                throw new \Exception("Falha ao gravar o arquivo.");
            }
            
            $info = file_get_contents($file->getPathname());  
            
            $info = explode("\n", $info);     //Array delimitado, uma string por linha
            $aux = count($info);                 //Número de linhas do arquivo
            unset($info[0       ]);              //Exclui a 1ª linha que é um cabeçalho
            unset($info[$aux - 1]);              //Exclui a penúltima linha que é em branco
            unset($info[$aux - 2]);              //Excluia última linha que é um totalizador de registros
    
            $aux = explode(';', $info[1]);       //Adquirindo os campos
    
            switch(count($aux))                     //Números de campos, determiandno o tipo de arquivo de retorno
            {
                case 21: $tipo = 'P'; $nomeTipo = 'PROCESSADO'; break;    //Processado: Cabeçalho: 11 + 11 + 60 + 8 + Restante: 17 = 107
                case 10: $tipo = 'R'; $nomeTipo = 'REJEITADO';  break;    //Rejeitado : Cabeçalho: 11 + 11 + 60 + 8 + Restante: 6  = 96
                default: $tipo = null;          //Formato desconhecido, inválido
            }
            if(is_null($tipo))
            {
                throw new \Exception("O tipo de arquivo é inválido. Espera-se um arquivo do tipo 'PROCESSADO' ou 'REJEITADO'.", Messages::ALERT_MESSAGE_CODE);
            }
    
            //Monta array de saída em 'estado bruto'
            $nro = 0;           //Número da linha do registro
            $registros = array();
            foreach($info as $linha)
            {
                $i = 0;                         //Índice auxiliar para os registros
                $aux = explode(';', $linha);    //Adquiri os campos de uma linha
                foreach(self::$layout[$tipo] as $valLayout) //Monta um registro
                    $registros[$nro][$valLayout] = $aux[$i++];  //recebe campo a campo como key do array layout
                $registros[$nro]['DN'] = substr($registros[$nro]['DN'], 0, 2) . '/' . substr($registros[$nro]['DN'], 2, 2) . '/' . substr($registros[$nro]['DN'], 4, 4);
                $registros[$nro++]['TIPO_RETORNO'] = $nomeTipo; //Nome do tipo do arquivo de retorno, Próximo registro
            }
    
            //Refina o array de saída, resume as últimas colunas segundso o array de 'dominio'
            $saida = array();           //Array de saida para o datasource
            foreach($registros as $registro)
            {
                $aux = null;         //Auxiliar para armazenas o dominínio e concatená-lo posteriormente
                foreach(self::$dominio[$tipo] as $keyDom => $valDom)
                {
                    $x = $valDom[intval($registro[$keyDom])];   //Adiquir a descrição do domínio (se houver)
                    if(!is_null($x))                    //Verifica se existe descrição do domínio
                        $aux[] = $x;
                }
                if(!is_null($aux))
                    $registro['DOMINIO'] = implode(";\r\n", $aux) . ".";   //Descrição do dominínio (erros encontrados)
                $saida[] = $registro;                   //Salva o registro no array de saída
            }
            return $saida;
            
        } catch (\Exception $e) {
            throw $e;
        }
    }

}
