<?php

namespace Vab\Services;

use Doctrine\ORM\EntityManager;
use Ems\Util\Utils;
use Folha\Util\Valor;
use Util\Environment;
use Util\Messages;
use Vab\Repository\CalculoValeTransporteRepository;

use Folha\Entities\GpePessoa;
use Folha\Entities\FpaItecalcfolha;
use Folha\Entities\VabHistvalevinc;
use Folha\Entities\GpeHorarioturno;

use Folha\Repositories\GpeEscalatrabm;

use HCM\Geral\Entity\GpealtesitufuncEntity;
use HCM\Geral\Entity\GpesitufuncmEntity;
use HCM\Geral\Entity\GpemovimentacaoEntity;

use HCM\HcmParameters;
use Ems\Services\{ EmailService};

use Util\{
    DateUtil,
    EmailMessages,
    //HCMParameters,
    Repositories,
    //Util
    NovoCodigo
};

use Vab\Logic\CalculoBeneficioUtil;

use \Zeedhi\Framework\ORM\DateTime;

class CalculoValeTransporteService {
    
    /** @var EntityManager */
    protected $entityManager;

    /** @var Environment */
    protected $environment;
    
    /** @var Repository */
    protected $calculoValeTransporteRepository;
    
    /** @var const MOVIMENTO_BENEFICIO */
    public const MOVIMENTO_BENEFICIO = 21;
    
    private $nrorg;
    private $vinculos;
    private $dataPagamento;
    private $dataInicio;
    private $dataFinal;
    private $competencia;
    private $nrtipolinhavinc;
    private $hcmParameters;
    private $qtdiasvalefixo;
    private $idcalccomplementar;
    private $idcalccredindev;
    private $dtiniciocredindev;
    private $dtfinalcredindev;
    
    public function __construct(
        EntityManager      $entityManager,
        Environment        $environment,
        CalculoValeTransporteRepository $calculoValeTransporteRepository,
        EmailService $emailService
    ) {
        $this->entityManager            = $entityManager;
        $this->environment              = $environment;
        $this->calculoValeTransporteRepository  = $calculoValeTransporteRepository;
        $this->emailService            = $emailService;
    }
    
    private function inicializaVariaveis($row){
        $this->row = $row;
        $this->idMemoria = $this->row['IDMEMORIACALC'];
        $this->dataInicio = DateUtil::getDataDeString(substr($this->row['DTINICIOVALE'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        $this->dataFinal = DateUtil::getDataDeString(substr($this->row['DTFINALVALE'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        $this->dataPagamento = DateUtil::getDataDeString(substr($this->row['DTPAGAMENTOVALE'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        $this->nrorg = $this->row['NRORG'];
        $this->competencia = $this->row['DTCOMPETENC'];
        $this->nrtipolinhavinc = $this->row['NRTIPOLINHAVINC'];
        $this->vinculos = $this->row['VINCULOS'];
        $this->qtdiasvalefixo = $this->row['QTDIASVALE'];
        $this->idcalccomplementar = !empty($this->row['IDCALCCOMPLEMENTAR']) ? $this->row['IDCALCCOMPLEMENTAR'] : 'N';
        $this->idcalccredindev = !empty($this->row['IDCALCCREDINDEV']) ? $this->row['IDCALCCREDINDEV'] : 'N';
        $this->dtiniciocredindev = $this->row['DTINICIOCREDINDEV'];
        $this->dtfinalcredindev = $this->row['DTFINALCREDINDEV'];
        $this->cdoperador = $this->row['CDOPERADOR'];
        $this->tpestruturalegalorg = $this->row['TIPOESTRUTURALEGALORG'];
        
        //!!!!!!!! IMPORTANTE, como estamos no HCMSERVICES precisamos de criar o hcmParameters
        $this->hcmParameters = new HcmParameters($this->nrorg, $this->competencia, 0, $this->cdoperador, $this->tpestruturalegalorg);
    }
    
    public function calcularValeTransporte($row){
        try {
            
            //Inicializa variaveis da classe
            $this->inicializaVariaveis($row);
            
            //Busca parametro de Vale Transporte. retorna um ARRAY
            $paramVale = $this->calculoValeTransporteRepository->getParametroValeTransporte($this->nrorg, $this->competencia);
            
            $row = array();
            $messages = array();
            
            foreach ($this->vinculos as $nrvinculo) {
                $vinculo = Utils::getEntityManager()->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array('nrorg' => $this->nrorg,
                'nrvinculom' => $nrvinculo));
                
                if($this->idcalccomplementar == 'N'){
                    $historicos = $this->retornaHistoricoValeEntrePeriodos($vinculo, $this->competencia);
                    //So deve verificar os historicos entre periodos se o flag calcComplementar não estiver marcado
                    if(!empty($historicos)){
                        /*$row[] = [
                          "NRMSGCALC"   => 0,
                          "DSDESCRICAO" => "Vínculo: ".$historicos[0]->getNrvinculom()." possui calculo de vale transporte na competência!! periodo Inicial: ".$historicos[0]->getDtiniapurvale()->format('d/m/Y')." periodo final: ".$historicos[0]->getDtfinapurvale()->format('d/m/Y')." e com data de pagamento em: ".$historicos[0]->getDtpgtovale()->format('d/m/Y').".", 
                          "IDTIPOMSG"   => 'Alerta',
                          "DTINCLUSAO"  => $historicos[0]->getDtinclusao()->format('d/m/Y'),
                        ];*/
                        continue;
                    }
                } elseif($this->idcalccomplementar == 'S') {
                    $historicosPgto = $this->retornaHistoricoPgto($vinculo, $this->competencia, $this->dataPagamento);
                    //So deve verificar os historicos com pagamento igual se o flag calcComplementar estiver marcado
                    if(!empty($historicosPgto)){
                        /*$row[] = [
                          "NRMSGCALC"   => 0,
                          "DSDESCRICAO" => "Vínculo: ".$historicos[0]->getNrvinculom()." possui calculo de vale transporte com mesma data de Pagamento!! Data de pagamento em: ".$historicos[0]->getDtpgtovale()->format('d/m/Y').".", 
                          "IDTIPOMSG"   => 'Alerta',
                          "DTINCLUSAO"  => $historicos[0]->getDtinclusao()->format('d/m/Y'),
                        ];*/
                        continue;
                    }
                }
                
                #MemoriaVT::getInstance()->inicializarDados($vinculo, $this->hcmParameters, $this->dataInicio, $this->dataFinal, $this->dataPagamento);
                
                #try {
                    //Se não estiver vazio considera utilização do parametro de vale transporte.
                    !empty($paramVale) ? $params = new ValeVinculo($vinculo, $this->nrorg, $this->dataInicio, $this->dataFinal, $this->nrtipolinhavinc, $this->hcmParameters, $paramVale[0], $this->qtdiasvalefixo, $this->idcalccomplementar, $this->idcalccredindev, $this->dtiniciocredindev, $this->dtfinalcredindev) 
                    //Caso estiver vazio calcula sem o parametro de vale transporte.
                    : $params = new ValeVinculo($vinculo, $this->nrorg, $this->dataInicio, $this->dataFinal, $this->nrtipolinhavinc, $this->hcmParameters, null, $this->qtdiasvalefixo, $this->idcalccomplementar, $this->idcalccredindev, $this->dtiniciocredindev, $this->dtfinalcredindev);
                #} catch (\Exception $e){
                    //Qualquer throw do valeVinculo deve ser continuado.
                #    continue;
                #}
                
                $infos = $params->getValeTransporte();
                
                if (!empty($infos)) {
                    foreach ($infos as $info) {
                        $params->addHistorico($info, $vinculo->getNrvinculom(), $this->dataPagamento);
                    }
                }
                
                if($params->getMessages()){
                    $messages[] = $params->getMessages();
                }
                
                #MemoriaVT::getInstance()->concatenarMensagemVinc();
            }
            
            if(count($messages) > 0){
                foreach($messages as $message){
                    foreach($message as $messages){
                        $row[] = [
                          "NRMSGCALC"   => 0,
                          "DSDESCRICAO" => $messages, 
                          "IDTIPOMSG"   => 'Alerta/Erro',
                          "DTINCLUSAO"  => DateUtil::getDataAtual()->format('d/m/Y'),
                        ];
                    }
                }
            }
            
            #MemoriaVT::getInstance()->salvarMsgArquivo();
            
            #if($this->idMemoria == 'S') $this->response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('memoriaCalculo', array(MemoriaVT::getInstance()->getCaminhoArquivo())) );
            
            \Setforms2\Shared\Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e; 
        }
    }
    
    private function retornaHistoricoValeEntrePeriodos($vinculo, $competencia){
        try{
            $competencia = DateUtil::getDataDeString($competencia)->format('d/m/Y');
            $vinculo = $vinculo->getNrvinculom();
            $dtinicioForm = $this->dataInicio->format('d/m/Y');
            $dtfimForm = $this->dataFinal->format('d/m/Y');
            $nrtipolinhavincParam = null;
            
            if(is_null($this->nrtipolinhavinc)){
                $nrtipolinhavincParam = 'A';
            }
            $param = array('P_NRORG' => $this->nrorg);
            
            $historicos = VabHistvalevinc::getObjectsByWhere(
                "NRORG = $this->nrorg AND
                DTMESCOMPAPVALE = '$competencia' AND 
                DTINIAPURVALE <= '$dtfimForm' AND 
                DTFINAPURVALE >= '$dtinicioForm' AND
                (NRTIPOLINHAVINC = '$this->nrtipolinhavinc' OR '$nrtipolinhavincParam' = 'A') AND
                NRVINCULOM IN ($vinculo)", $param
            );
            
            if($historicos){
                return $historicos;
            } else {
                return array();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function retornaHistoricoPgto($vinculo, $competencia, $dtPagamento){
        try{
            $competencia = DateUtil::getDataDeString($competencia)->format('d/m/Y');
            $dtPagamento = $dtPagamento->format('d/m/Y');
            $vinculo = $vinculo->getNrvinculom();
            $dtinicioForm = DateUtil::getDataDeString(substr($this->row['DTINICIOVALE'], 0, 10))->format('d/m/Y');
            $dtfimForm = DateUtil::getDataDeString(substr($this->row['DTFINALVALE'], 0, 10))->format('d/m/Y');
            $nrtipolinhavincParam = null;
            
            if(is_null($this->nrtipolinhavinc)){
                $nrtipolinhavincParam = 'A';
            }
            $param = array('P_NRORG' => $this->nrorg);
            
            $historicos = VabHistvalevinc::getObjectsByWhere(
                "NRORG = $this->nrorg AND
                DTMESCOMPAPVALE = '$competencia' AND 
                DTPGTOVALE = '$dtPagamento' AND
                (NRTIPOLINHAVINC = '$this->nrtipolinhavinc' OR '$nrtipolinhavincParam' = 'A') AND
                NRVINCULOM IN ($vinculo)", $param
            );
            
            if($historicos){
                return $historicos;
            } else {
                return array();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
}