<?php

namespace HCM\Geral\Logic;

use HCM\Util\Repositories;
use Symfony\Component\DependencyInjection\SimpleXMLElement;
use Teknisa\Model\Geral\Entities\Estruturam;
use \HCM\Geral\Services\ParcnegocioService;
use \HCM\Geral\Services\EstruturasService;
use HCM\Util\DateUtil;
use Zeedhi\Framework\ORM\DateTime;


class ImportacaoXmlEsocialLogic {
    
    const MSG_ERRO_XML = "Estrutura do xml do evento incorreta";
    
    private $dataXml = array();
    private $countChild  = 0;
    
    public function __construct($entityManager, $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        
        $this->estruturam = Repositories::ESTRUTURAM;
        $this->estruturah = Repositories::ESTRUTURAH;
        $this->parcnegocio = Repositories::PARCNEGOCIO;
        $this->comunicaParc = Repositories::COMUNICAPARC;
        $this->gpepessoa = Repositories::GPE_PESSOA;
        $this->gpepessoah = Repositories::GPE_PESSOAH;
        $this->ocupacaom = Repositories::GPE_OCUPACAOM;
        $this->ocupacaoh = Repositories::GPE_OCUPACAOH;
        $this->enderecoParc = Repositories::ENDERECOPARC;
        $this->relacionaparc = Repositories::RELACIONAPARC;
        $this->gpevinculom = Repositories::GPE_VINCULOM;
        $this->gpevinculoh = Repositories::GPE_VINCULOH;
        $this->gpemovimentacao = Repositories::GPE_MOVIMENTACAO;
        $this->gpealteOcupacao = Repositories::GPE_ALTEOCUPACAO;
        $this->gpealtesalario = Repositories::GPE_ALTESALARIO;
        $this->gpealtesitufunc = Repositories::GPE_ALTESITUFUNC;
        $this->fpadepvinculo = Repositories::FPA_DEPVINCULO;
        $this->gpealteEscala = Repositories::GPE_ALTEESCALA;
 	    $this->gpehordiam = Repositories::GPE_HORDIAM;
 	    $this->gpehordiah = Repositories::GPE_HORDIAH;
 	    $this->gpentervaloshorario = Repositories::GPE_INTERVALOSHORARIO;
 	    $this->fpaferias = Repositories::FPA_FERIAS;
 	    $this->fpagozoferias = Repositories::FPA_GOZOFERIAS;
 	    $this->desFormacao = Repositories::DES_FORMACAO;
    }
    
    
    public function insertXmlS1000($xml, $nrInsc, $idoptsimples){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $dtcompetencia = $this->hcmParameters->getDtCompetencia();
        $nrtipoestrutura = $this->hcmParameters->getTipoEstruturaLegalOrg();
        $evtInfoEmpregador = (array) $xml->evtInfoEmpregador;
        
        $estrutMesmaInsc = $this->verificaEstruturaMesmaInscricao($nrInsc, $nrtipoestrutura, $nrorg, $dtcompetencia);
        if(!empty($estrutMesmaInsc)){
            return array('status' => false, 
                         'message' => 'Foi encontrada a estrutura '.$estrutMesmaInsc[0]->getNrestruturam().' com o mesmo número de inscrição.');   
        }
        if($evtInfoEmpregador){
            $ideEmpregador = (array) $evtInfoEmpregador['ideEmpregador'];
            $infoCadastro = (array) $evtInfoEmpregador['infoEmpregador']->inclusao->infoCadastro;
            $softwareHouse = (array) $evtInfoEmpregador['infoEmpregador']->inclusao->infoCadastro->softwareHouse;
            $contato = (array) $evtInfoEmpregador['infoEmpregador']->inclusao->infoCadastro->contato;
            $idePeriodo = (array) $evtInfoEmpregador['infoEmpregador']->inclusao->idePeriodo; 
            
            // Utiliza as informações do XML
            if($ideEmpregador['tpInsc'] == 1){
                $idpessoafisica = 'N'; 
                $cdtipoinscricao = 'CNPJ';
            }else{
                $idpessoafisica = 'S'; 
                $cdtipoinscricao = 'CPF';
            }
            
            if(strlen($idePeriodo['iniValid']) < 10){
                $idePeriodo['iniValid'] = $this->alterDateFormat($idePeriodo['iniValid']);
            }
            
            if(array_key_exists('fimValid', $idePeriodo) && !empty($idePeriodo['fimValid'])){
                $idePeriodo['fimValid'] = strlen($idePeriodo['fimValid']) < 10 ? $this->alterDateFormat($idePeriodo['fimValid']) : $idePeriodo['fimValid'];
            }else{
                $idePeriodo['fimValid'] = null;
            }
            $idativo = $this->retonaAtivo($idePeriodo['fimValid']);
            
            $nmestrutura = $infoCadastro['nmRazao'];
            $nmfantasia = $infoCadastro['nmRazao'];
            $classtribut = $this->entityManager->getRepository(Repositories::CLASSTRIBUT)->findOneBy(array('cdesocial' => $infoCadastro['classTrib']));
            $cdcnae = null;
            $cdnatujuri = $infoCadastro['natJurid'];
            
            // Utiliza as informações do WebService substituindo as do XML caso existam
            $webservice = false;
            $retorno = $this->consultaDadoRF($nrInsc);
            if($retorno['Status'] == true){
                $webservice = true;
            }
            if(isset($retorno['RazaoSocial']) && $retono['RazaoSocial'] != ""){
                $nmestrutura = $retorno['RazaoSocial'];
            }
            if(isset($retorno['NomeFantasia']) && $retono['NomeFantasia'] != ""){
                $nmfantasia = $retorno['NomeFantasia'];
            }
            if(isset($retorno['DataFundacao']) && $retono['DataFundacao'] != ""){
                $iniValid = $retorno['DataFundacao'];
            }
            if(isset($retorno['MatrizFilial']) && $retono['MatrizFilial'] != ""){
                if($retorno['MatrizFilial'] == "MATRIZ"){
                    $idMatriz = 'S';
                }else{
                    $idMatriz = 'N';
                }
            }
            if(isset($retorno['CodigoAtividadeEconomica']) && $retono['CodigoAtividadeEconomica'] != ""){
                $cdcnae = str_replace(array("-","/"), "", $retorno['CodigoAtividadeEconomica']);
            }
            if(isset($retorno['CodigoNaturezaJuridica']) && $retono['CodigoNaturezaJuridica'] != ""){
                $cdnatujuri = $retorno['CodigoNaturezaJuridica'];
            }
            /*["Enderecos"]=>
            array(11) {
                ["Tipo"]=> string(9) "Comercial"
                ["Logradouro"]=> string(13) "PC RIO BRANCO"
                ["Numero"]=> string(3) "100"
                ["Complemento"]=> string(17) "PAVMTO2 QUIOSQHE6"
                ["Bairro"]=> string(6) "CENTRO"
                ["Cidade"]=> string(14) "BELO HORIZONTE"
                ["Estado"]=> string(2) "MG"
                ["CEP"]=> string(8) "30111050"
                ["GeoLocalizacao"]=>
                array(2) {
                    ["Latitude"]=> string(11) "-19.9138499"
                    ["Longitude"]=> string(11) "-43.9421293"
                }
                ["DataAtualizacao"]=> string(25) "2020-04-27T00:00:00-03:00"
                ["CodigoIBGE"]=> int(3106200)
            }
            ["Email"]=> string(26) "keilaribeiro@redefc.com.br"
            ["Telefone"]=> string(10) "3134126990"*/
            
            try{
                $this->entityManager->beginTransaction();
                
                // Salva Pessoa Responsável
                $pessoa = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->retornaPessoaCpf($nrorg, $dtcompetencia, $contato['cpfCtt']);
                
                if(empty($pessoa)){
                    $parcnegocioPessoa = $this->novoParcNegocio($nrorg, $contato['nmCtt'], $contato['nmCtt'], 'PESSOA', 'S', 'N',
                                                                 'CPF', $contato['cpfCtt'], null, 'S');
                    $pessoa = $this->novaPessoaResponsável($nrorg, $contato['nmCtt'], $contato['cpfCtt'], DateUtil::getDataDeString($idePeriodo['iniValid'],DateUtil::FORMATO_BRASILEIRO,true), $parcnegocioPessoa);         
                    $parceiroResp = $pessoa->getNrparcnegocio();
                }else{
                    $parceiroResp = $pessoa[0]['NRPARCNEGOCIO'];
                }
                
                // Salva Parceiro de Negócio
                $parcnegocio = $this->novoParcNegocio($nrorg, $nmestrutura, $nmestrutura, 'ESTRUTURA', $idpessoafisica, 'N',
                                                $cdtipoinscricao, $nrInsc, DateUtil::getDataDeString($idePeriodo['iniValid'],DateUtil::FORMATO_BRASILEIRO,true), $idativo);
                                             
                // Salva Estrutura
                $estrutura = $this->novaEstrutura($nrtipoestrutura, $idePeriodo['iniValid'], $idativo, $idoptsimples, $parcnegocio->getNrparcnegocio(), $nmestrutura, $nrInsc, $idePeriodo['fimValid'], null, null, null, null,
                                               null, null, null, null, null, null, null, null, null, null, $nmestrutura, $cdnatujuri, $cdcnae, null, null, null, null, null, null, $parceiroResp, null, null, $nmfantasia, $nmestrutura,
                                               null, null, null, null, $classtribut->getNrclasstribut(), null, null, null, null, null, null, null, null, null, null, null, null);
                
                // Salva Comunicação do parceiro
                $rowsComunicaParc = $this->formatComunicaParc($contato, $parcnegocio);
                $this->novasFormasComunica($rowsComunicaParc);
                
                $this->entityManager->commit();
            }catch (\Exception $e) {   
                $this->entityManager->rollback();
                throw $e;
            }
            
            return array('status' => true,     
                         'messages' => ['Dados importados corretamente! Número da estrutura: '.$estrutura[0]->getNrestruturam()]);
            // return array('status' => true, 'NRESTRUTURAM' => $estrutura[0]->getNrestruturam());
            
        }else{
            return array('status' => false,     
                         'message' => self::MSG_ERRO_XML . " (Ausência da tag 'evtInfoEmpregador' no arquivo).");
        }
    }
    
    public function insertXmlS1005($xml, $idoptsimples){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $dtcompetencia = $this->hcmParameters->getDtCompetencia();
        $nrtipoestrutura = $this->hcmParameters->getTipoEstruturaLegalOrg();
        $evtTabEstab  = (array) $xml->evtTabEstab;
        
        if($evtTabEstab){
            $ideEmpregador = (array) $evtTabEstab['ideEmpregador'];
            $ideEstab = (array) $evtTabEstab['infoEstab']->inclusao->ideEstab; 
            $dadosEstab = (array) $evtTabEstab['infoEstab']->inclusao->dadosEstab;
            $aliqGilrat = (array) $evtTabEstab['infoEstab']->inclusao->dadosEstab->aliqGilrat;
            
            $nrInsc = $ideEstab['nrInsc'];
            $estrutMesmaInsc = $this->verificaEstruturaMesmaInscricao($nrInsc, $nrtipoestrutura, $nrorg, $dtcompetencia);
            
            // Utiliza as informações do XML
            if($ideEstab['tpInsc'] == 1 || $ideEstab['tpInsc'] == 3 || $ideEstab['tpInsc'] == 4){
                $idpessoafisica = 'N'; 
                $cdtipoinscricao = 'CNPJ';
                if(substr($nrInsc, 8, 4) == '0001'){
                    $idMatriz = 'S';
                }else{
                    $idMatriz = 'N';
                }
            }else{
                $idMatriz = 'S';
                $idpessoafisica = 'S'; 
                $cdtipoinscricao = 'CPF';
            }
            
            if(strlen($ideEstab['iniValid']) < 10){
                $ideEstab['iniValid'] = $this->alterDateFormat($ideEstab['iniValid']);
            }
            
            if(array_key_exists('fimValid', $ideEstab) && !empty($ideEstab['fimValid'])){
                $ideEstab['fimValid'] = strlen($ideEstab['fimValid']) < 10 ? $this->alterDateFormat($ideEstab['fimValid']) : $ideEstab['fimValid'];
            }else{
                $ideEstab['fimValid'] = null;
            }
            $idativo = $this->retonaAtivo($ideEstab['fimValid']);
            
            $cdcnae = $dadosEstab['cnaePrep'];
            
            $vrfap = $aliqGilrat['fap'];
                    
            // Utiliza as informações do WebService substituindo as do XML caso existam
            $webservice = false;
            $retorno = $this->consultaDadoRF($nrInsc);
            if($retorno['Status'] == true){
                $webservice = true;
            }
            if(isset($retorno['CodigoAtividadeEconomica']) && $retono['CodigoAtividadeEconomica'] != ""){
                $cdcnae = str_replace(array("-","/"), "", $retorno['CodigoAtividadeEconomica']);
            }
            
            if($idMatriz == 'S'){
                if(empty($estrutMesmaInsc)){
                    return array('status' => false, 
                                 'message' => 'Não foi encontrado o registro dessa estrutura matriz para ser atualizado.');   
                }else{
                    try{
                        $this->entityManager->beginTransaction();
                        
                        $estrutura = $estrutMesmaInsc;
                        $estruturah = $estrutMesmaInsc[1];
                        
                        $estruturah->setCdcnae($cdcnae);
                        $estruturah->setVrfap($vrfap);
                        
                        $this->entityManager->persist($estruturah);
                        $this->entityManager->flush();
                        
                        $this->entityManager->commit();
                    }catch (\Exception $e) {   
                        $this->entityManager->rollback();
                        throw $e;
                    }
                }
            }else{
                if(!empty($estrutMesmaInsc)){
                   return array('status' => false, 
                                'message' => 'Já foi localizado o registro de número '.$estrutMesmaInsc[0]->getNrestruturam().' para essa filial .');    
                }
                
                try{
                    $this->entityManager->beginTransaction();
                    
                    $estrututaMatriz = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaMatrizPorFilial($nrorg, $ideEmpregador['nrInsc'], $nrtipoestrutura, $dtcompetencia)[0];
                    if(empty($estrututaMatriz)){
                        return array('status' => false, 
                                     'message' => 'Não foi encontrada no sistema nenhuma estrutura matriz para essa filial.'); 
                    }
                    
                    if($ideEstab['tpInsc'] == 3){
                        $cdceiestrut = null;
                        $cdcaepfestrut = $nrInsc; 
                        $cdcnpjestrut = $estrututaMatriz['CDCNPJESTRUT'];
                    }else if($ideEstab['tpInsc'] == 4){
                        $cdceiestrut = $nrInsc;
                        $cdcaepfestrut = null; 
                        $cdcnpjestrut = $estrututaMatriz['CDCNPJESTRUT'];
                    }else{
                        $cdceiestrut = null;
                        $cdcaepfestrut = null; 
                        $cdcnpjestrut = $nrInsc;
                    }
                    
                    $nmestrutura = $this->montaNomeEstrutura($estrututaMatriz['NMRAZSOCESTRUT'], $nrInsc, 'FILIAL');
                    $nmfantasia = $nmestrutura;
                    $cdnatujuri = null;
                    
                    // Utiliza as informações do WebService substituindo as do XML caso existam
                    if(isset($retorno['RazaoSocial']) && $retono['RazaoSocial'] != ""){
                        $nmestrutura = $retorno['RazaoSocial'];
                    }
                    if(isset($retorno['NomeFantasia']) && $retono['NomeFantasia'] != ""){
                        $nmfantasia = $retorno['NomeFantasia'];
                    }
                    if(isset($retorno['DataFundacao']) && $retono['DataFundacao'] != ""){
                        $iniValid = $retorno['DataFundacao'];
                    }
                    if(isset($retorno['CodigoNaturezaJuridica']) && $retono['CodigoNaturezaJuridica'] != ""){
                        $cdnatujuri = $retorno['CodigoNaturezaJuridica'];
                    }
                    
                    // Salva Parceiro de Negócio
                    $parcnegocio = $this->novoParcNegocio($nrorg, $nmestrutura, $nmestrutura, 'ESTRUTURA', $idpessoafisica, 'N',
                                                          $cdtipoinscricao, $nrInsc, DateUtil::getDataDeString($ideEstab['iniValid'],DateUtil::FORMATO_BRASILEIRO,true), $idativo);
                                                    
                    // Salva Estrutura
                    $estrutura = $this->novaEstrutura($nrtipoestrutura, $ideEstab['iniValid'], $idativo, $idoptsimples, $parcnegocio->getNrparcnegocio(), $nmestrutura, $cdcnpjestrut, $ideEstab['fimValid'], null, null, null, null,
                                                      null, null, null, null, null, null, null, null, null, $cdceiestrut, $nmestrutura, $cdnatujuri, $cdcnae, null, null, null, null, null, null, null, null, null, $nmfantasia, $nmestrutura,
                                                      null, null, null, null, null, $vrfap, null, null, null, null, null, null, null, null, null, $cdcaepfestrut, null);
                    
                    $this->entityManager->commit();
                }catch (\Exception $e) {   
                    $this->entityManager->rollback();
                    throw $e;
                }
                
            }
            return array('status' => true,     
                         'messages' => ['Dados importados corretamente! Número da estrutura: '.$estrutura[0]->getNrestruturam()]);
            
            // return array('status' => true, 'NRESTRUTURAM' => $estrutura[0]->getNrestruturam());
            
        }else{
            return array('status' => false,     
                         'message' => self::MSG_ERRO_XML . " (Ausência da tag 'evtTabEstab' no arquivo).");
        }
    }
    
    public function insertXmlS1010($xml){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
        $competencia = $this->hcmParameters->getDtCompetencia();
        $evtTabRubrica  = (array) $xml->evtTabRubrica;
        $mensagens = array();
        
        if($evtTabRubrica){
            $infoRubrica = (array) $evtTabRubrica['infoRubrica']->inclusao;
            $ideRubrica = (array) $infoRubrica['ideRubrica'];
            $dadosRubrica = (array) $infoRubrica['dadosRubrica'];
            $ideProcessoCP = array_key_exists('ideProcessoCP', $dadosRubrica) ? $dadosRubrica['ideProcessoCP'] : null;
            $ideProcessoIRRF = array_key_exists('ideProcessoIRRF', $dadosRubrica) ? $dadosRubrica['ideProcessoIRRF'] : null;
            $ideProcessoFGTS = array_key_exists('ideProcessoFGTS', $dadosRubrica) ? $dadosRubrica['ideProcessoFGTS'] : null;
            $ideProcessoSIND = array_key_exists('ideProcessoSIND', $dadosRubrica) ? $dadosRubrica['ideProcessoSIND'] : null;
            
        }else{
            return array('status' => false,     
                         'message' => self::MSG_ERRO_XML . " (Ausência da tag 'evtTabRubrica' no arquivo).");
        }
    }
    
    public function insertXmlS1020($xml, $nrInsc, $idoptsimples){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $dtcompetencia = $this->hcmParameters->getDtCompetencia();
        $evtTabLotacao  = (array) $xml->evtTabLotacao;
        $nrtipoestruturalegal = $this->hcmParameters->getTipoEstruturaLegalOrg();
        
        if($evtTabLotacao){
            $ideEmpregador = (array) $evtTabLotacao['ideEmpregador'];
            $ideLotacao = (array) $evtTabLotacao['infoLotacao']->inclusao->ideLotacao; 
            $dadosLotacao = (array) $evtTabLotacao['infoLotacao']->inclusao->dadosLotacao;
            $fpasLotacao = (array) $evtTabLotacao['infoLotacao']->inclusao->dadosLotacao->fpasLotacao;
            
            $tplotatribut = $this->entityManager->getRepository(Repositories::ESO_TPLOTATRIBUT)->findOneBy(array('cdesocial' => $dadosLotacao['tpLotacao']));
            if(empty($tplotatribut)){
                return array('status' => false, 
                             'message' => 'Não foi encontrado o tipo de lotação informado ('.$dadosLotacao['tpLotacao'].')');
            }
            
            $tipoestruturaLotacao = $this->entityManager->getRepository(Repositories::TIPOESTRUTURA)->findBy(array('nrtplotatribut' => $tplotatribut->getNrtplotatribut(), 'nrorg' => array($nrorg, $this->hcmParameters->getNrorgPadrao())));
            if(empty($tipoestruturaLotacao)){
                return array('status' => false, 
                             'message' => 'Não foi encontrado tipo de estrutura para o tipo de lotação informado. ('.$dadosLotacao['tpLotacao'].')');
            }
            
            $nrtipoestrutura = $this->retonarTipoEstruturaInserida($tipoestruturaLotacao);
            if(empty($nrtipoestrutura)){
                return array('status' => false, 
                             'message' => 'Não foi encontrado tipo de estrutura legal nem tomador para o tipo de lotação informado. ('.$dadosLotacao['tpLotacao'].')');
            }
            
            $estrutMesmaInsc = $this->verificaEstruturaMesmaInscricao($nrInsc, $nrtipoestrutura['numero'], $nrorg, $dtcompetencia);
            
            // Utiliza as informações do XML
            if($nrtipoestrutura['tipo'] == 'LEGAL'){
                $idTomador = 'N';
            }else{
                $idTomador = 'S';
            }
            
            if(strlen($nrInsc) == 14){
                $idpessoafisica = 'N'; 
                $cdtipoinscricao = 'CNPJ';
            }else{
                $idpessoafisica = 'S'; 
                $cdtipoinscricao = 'CPF';
            }
            
            if(strlen($ideLotacao['iniValid']) < 10){
                $ideLotacao['iniValid'] = $this->alterDateFormat($ideLotacao['iniValid']);
            }
            
            if(array_key_exists('fimValid', $ideLotacao) && !empty($ideLotacao['fimValid'])){
                $ideLotacao['fimValid'] = strlen($ideLotacao['fimValid']) < 10 ? $this->alterDateFormat($ideLotacao['fimValid']) : $ideLotacao['fimValid'];
            }else{
                $ideLotacao['fimValid'] = null;
            }
            
            $idativo = $this->retonaAtivo($ideLotacao['fimValid']);
            
            if($idTomador == 'N'){
                if(empty($estrutMesmaInsc)){
                    return array('status' => false, 
                                 'message' => 'Não foi encontrado o registro dessa estrutura matriz para ser atualizado.');   
                }else{
                    try{
                        $this->entityManager->beginTransaction();
                        
                        $estrutura = $estrutMesmaInsc;
                        $estruturah = $estrutMesmaInsc[1];
                        
                        $estruturah->setCdterceiro($fpasLotacao['codTercs']);
                        $estruturah->setCdfpas($fpasLotacao['fpas']);
                        
                        $this->entityManager->persist($estruturah);
                        $this->entityManager->flush();
                        
                        $this->entityManager->commit();
                    }catch (\Exception $e) {   
                        $this->entityManager->rollback();
                        throw $e;
                    }
                }
            }else{
                if(!empty($estrutMesmaInsc)){
                   return array('status' => false, 
                                 'message' => 'Já foi localizado o registro de número '.$estrutMesmaInsc[0]->getNrestruturam().' para esse tomador.');    
                }
                
                try{
                    $this->entityManager->beginTransaction();
            
                    $nmestrutura = $this->montaNomeEstrutura('', $nrInsc, 'TOMADOR');
                    $nmfantasia = $nmestrutura;
                    $cdcnae = null;
                    $cdnatujuri = null;
                    
                    // Utiliza as informações do WebService substituindo as do XML caso existam
                    $webservice = false;
                    $retorno = $this->consultaDadoRF($nrInsc);
                    if($retorno['Status'] == true){
                        $webservice = true;
                    }
                    if(isset($retorno['RazaoSocial']) && $retono['RazaoSocial'] != ""){
                        $nmestrutura = $retorno['RazaoSocial'];
                    }
                    if(isset($retorno['NomeFantasia']) && $retono['NomeFantasia'] != ""){
                        $nmfantasia = $retorno['NomeFantasia'];
                    }
                    if(isset($retorno['DataFundacao']) && $retono['DataFundacao'] != ""){
                        $iniValid = $retorno['DataFundacao'];
                    }
                    if(isset($retorno['CodigoAtividadeEconomica']) && $retono['CodigoAtividadeEconomica'] != ""){
                        $cdcnae = str_replace(array("-","/"), "", $retorno['CodigoAtividadeEconomica']);
                    }
                    if(isset($retorno['CodigoNaturezaJuridica']) && $retono['CodigoNaturezaJuridica'] != ""){
                        $cdnatujuri = $retorno['CodigoNaturezaJuridica'];
                    }
                    
                    // Salva Parceiro de Negócio
                    $parcnegocio = $this->novoParcNegocio($nrorg, $nmestrutura, $nmestrutura, 'ESTRUTURA', $idpessoafisica, 'N',
                                                          $cdtipoinscricao, $nrInsc, DateUtil::getDataDeString($ideLotacao['iniValid'],DateUtil::FORMATO_BRASILEIRO,true), $idativo);
                    
                    // Salva Estrutura
                    $estrutura = $this->novaEstrutura($nrtipoestrutura['numero'], $ideLotacao['iniValid'], $idativo, $idoptsimples, $parcnegocio->getNrparcnegocio(), $nmestrutura, $nrInsc, $ideLotacao['fimValid'], $ideLotacao['codLotacao'], null, null, null,
                                                      null, null, null, null, null, null, null, null, null, null, $nmestrutura, $cdnatujuri, $cdcnae, null, null, null, null, null, null, null, $fpasLotacao['fpas'], null, $nmfantasia, $nmestrutura,
                                                      null, null, null, $fpasLotacao['codTercs'], null, null, null, null, null, null, null, null, null, null, null, null, null);
                    
                    $this->entityManager->commit();
                }catch (\Exception $e) {   
                    $this->entityManager->rollback();
                    throw $e;
                }
                
            }
            
            return array('status' => true,     
                         'messages' => ['Dados importados corretamente! Número da estrutura: '.$estrutura[0]->getNrestruturam()]);
            
            // return array('status' => true, 'NRESTRUTURAM' => $estrutura[0]->getNrestruturam());
            
        }else{
            return array('status' => false,     
                         'message' => self::MSG_ERRO_XML . " (Ausência da tag 'evtTabLotacao' no arquivo).");
        }
    }
    
    public function insertXmlS1030($xml){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $dtcompetencia = $this->hcmParameters->getDtCompetencia();
        $evtTabCargo  = (array) $xml->evtTabCargo;
        
        if($evtTabCargo){
            $infoCargo = (array) $evtTabCargo['infoCargo'];
            $ideCargo = (array) $evtTabCargo['infoCargo']->inclusao->ideCargo; 
            $dadosCargo = (array) $evtTabCargo['infoCargo']->inclusao->dadosCargo;
            
            $ocupacao = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOH)->retornaUltimoHistoricoCdintegracao($ideCargo['codCargo'], $nrorg, $dtcompetencia);
            if(!empty($ocupacao)){
                return array('status' => false, 
                             'message' => 'Já existe uma ocupação cadastrada com esse código de integração.');   
            }
            
            $dtinivigencia = $this->alterDateFormat($ideCargo['iniValid']);
            
            if(array_key_exists('fimValid', $ideCargo) && !empty($ideCargo['fimValid'])){
                $dtfimvigencia = strlen($ideCargo['fimValid']) < 10 ? $this->alterDateFormat($ideCargo['fimValid']) : $ideCargo['fimValid'];
            }else{
                $dtfimvigencia = null;
            }
            
            $dtmescompetenc = DateUtil::truncateData(DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataDeString($dtinivigencia,DateUtil::FORMATO_BRASILEIRO,true)));
            
            try{
                $this->entityManager->beginTransaction();
        
                // Salva Ocupação
                $ocupacao = $this->novaOcupacao(1, $dtinivigencia, $dtmescompetenc, $dadosCargo['nmCargo'], $dtfimvigencia, null, null, $ideCargo['codCargo'], 
                                                   $dadosCargo['codCBO'], null, null, null, null , null);

                $this->entityManager->commit();
            }catch (\Exception $e) {   
                $this->entityManager->rollback();
                throw $e;
            }
            
            return array('status' => true,     
                         'messages' => ['Dados importados corretamente! Número da ocupação: '.$ocupacao[0]->getNrocupacaom()]);
            // return array('status' => true, 'NROCUPACAOM' => $ocupacao[0]->getNrocupacaom());
        }else{
            return array('status' => false,     
                         'message' => self::MSG_ERRO_XML . " (Ausência da tag 'evtTabCargo' no arquivo).");
        }
    }
    
    public function insertXmlS1040($xml){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $dtcompetencia = $this->hcmParameters->getDtCompetencia();
        $evtTabFuncao  = (array) $xml->evtTabFuncao;
        
        if($evtTabFuncao){
            $infoFuncao = (array) $evtTabFuncao['infoFuncao'];
            $ideFuncao = (array) $evtTabFuncao['infoFuncao']->inclusao->ideFuncao; 
            $dadosFuncao = (array) $evtTabFuncao['infoFuncao']->inclusao->dadosFuncao;
            
            $ocupacao = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOH)->retornaUltimoHistoricoCdintegracao($ideFuncao['codFuncao'], $nrorg, $dtcompetencia);
            if(!empty($ocupacao)){
                return array('status' => false, 
                             'message' => 'Já existe uma ocupação cadastrada com esse código de integração.');   
            }
            
            $dtinivigencia = $this->alterDateFormat($ideFuncao['iniValid']);
            
            if(array_key_exists('fimValid', $ideFuncao) && !empty($ideFuncao['fimValid'])){
                $dtfimvigencia = strlen($ideFuncao['fimValid']) < 10 ? $this->alterDateFormat($ideFuncao['fimValid']) : $ideFuncao['fimValid'];
            }else{
                $dtfimvigencia = null;
            }
            
            $dtmescompetenc = DateUtil::truncateData(DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataDeString($dtinivigencia,DateUtil::FORMATO_BRASILEIRO,true)));
            
            try{
                $this->entityManager->beginTransaction();
        
                // Salva Ocupação
                $ocupacao = $this->novaOcupacao(2, $dtinivigencia, $dtmescompetenc, $dadosFuncao['dscFuncao'], $dtfimvigencia, null, null, $ideFuncao['codFuncao'], 
                                                   $dadosFuncao['codCBO'], null, null, null, null , null);

                $this->entityManager->commit();
            }catch (\Exception $e) {   
                $this->entityManager->rollback();
                throw $e;
            }
            return array('status' => true,     
                         'messages' => ['Dados importados corretamente! Número da ocupação: '.$ocupacao[0]->getNrocupacaom()]);
            // return array('status' => true, 'NROCUPACAOM' => $ocupacao[0]->getNrocupacaom());
        }else{
            return array('status' => false,     
                         'message' => self::MSG_ERRO_XML . " (Ausência da tag 'evtTabFuncao' no arquivo).");
        }
    }
    
    public function insertXmlS1050($xml){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $dtcompetencia = $this->hcmParameters->getDtCompetencia();
        $evtTabHorTur  = (array) $xml->evtTabHorTur;
        
        if($evtTabHorTur){
            $infoHorContratual = (array) $evtTabHorTur['infoHorContratual'];
            $ideHorContratual = (array) $infoHorContratual['inclusao']->ideHorContratual;
            $dadosHorContratual = (array) $infoHorContratual['inclusao']->dadosHorContratual;
            if(array_key_exists('horarioIntervalo', $dadosHorContratual)){
                $horarioIntervalo = is_array($dadosHorContratual['horarioIntervalo']) ? $dadosHorContratual['horarioIntervalo'] :  [$dadosHorContratual['horarioIntervalo']];
                $horarioIntervalo = $this->ordenaDados($horarioIntervalo);
            }else{
                $horarioIntervalo = [];
            }
            
            $horario = $this->entityManager->getRepository(Repositories::GPE_HORDIAM)->findOneBy(array('nrorg' => $nrorg, 'cdintegracao' => $ideHorContratual['codHorContrat']));
            
            if(empty($horario)){
                try{
                    $this->entityManager->beginTransaction();
                    
                    $dtinivigencia = DateUtil::getDataDeString($this->alterDateFormat($ideHorContratual['iniValid'], 1), DateUtil::FORMATO_BRASILEIRO, true);
    
                    if(array_key_exists('fimValid', $ideHorContratual)){
                        $dtfimvigencia = DateUtil::getDataDeString($this->alterDateFormat($ideHorContratual['fimValid'], 1), DateUtil::FORMATO_BRASILEIRO, true);
                    }else{
                        $dtfimvigencia = null;
                    }
                    
                    $nmhordiah = $dadosHorContratual['hrEntr'].' - '.$dadosHorContratual['hrSaida'];
            
                    // Salva Horário
                    $horario = $this->novoHorario($nrorg, $dtinivigencia, $dtinivigencia, $nmhordiah, 'N', $dtfimvigencia, $ideHorContratual['codHorContrat'], '0600');
                    
                    $entrada = $dadosHorContratual['hrEntr'];
                    $saida = $dadosHorContratual['hrSaida'];
                    $hrintervalo = null;
                    $dsintervalo = null;
                    $nrtpintervalo = (intval($dadosHorContratual['hrSaida']) - intval($dadosHorContratual['hrEntr'])) < 400 ? 1 : 3;
                    $intervalosHorario = [];
                    
                    // 1	Sem intervalo
                    // 2	Intervalo em horário fixo
                    // 3	Intervalo em horário variável
                    
                    for($i = 0; $i< count($horarioIntervalo)+1; $i++){
                        $dados =  array_key_exists($i, $horarioIntervalo) ? (array) $horarioIntervalo[$i] : null;
                        
                        if(count($horarioIntervalo) == $i && count($horarioIntervalo) > 0){
                            $saida = $dadosHorContratual['hrSaida'];
                            $hrintervalo = null;
                            $dsintervalo = null;
                            $nrtpintervalo = null;
                        }else if(count($horarioIntervalo) >= $i && !empty($dados)){
                            $saida = $dados['iniInterv'];
                            $hrintervalo = $this->getHoraDeMinuto($dados['durInterv']);
                            $dsintervalo = $dados['durInterv'] >= '60' ? 'Almoço/Descanso' : 'Lanche';
                            $nrtpintervalo = 2;
                        }
                        
                        $intervalosHorario[] = $this->novoIntervaloHorario($nrorg, $horario[0]->getNrhordiam(), ($i+1), $entrada, $entrada, $entrada, $saida, $saida, $saida, $entrada, $saida, 'DC', 'DC', 'N', 'N', 'N', 'N', $hrintervalo, 'S', $dsintervalo, $nrtpintervalo);
                                                                        
                        $entrada = !empty($dados) ? $dados['termInterv'] : null;
                    }
                    
                    $this->entityManager->commit();
                }catch (\Exception $e) {   
                    $this->entityManager->rollback();
                    throw $e;
                }
                
                return array('status' => true,     
                             'messages' => ['Dados importados corretamente! Número do horário: '.$horario[0]->getNrhordiam()]);
                // return array('status' => true, 'NRHORDIAM' => $horario[0]->getNrhordiam());
            }else{
                return array('status' => false, 
                             'message' => 'Já existe um horário cadastrado com esse código de integração.');   
            }
            
        }else{
            return array('status' => false,     
                         'message' => self::MSG_ERRO_XML . " (Ausência da tag 'evtTabFuncao' no arquivo).");
        }
    }
    
    public function insertXmlS2200($xml){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
        $competencia = $this->hcmParameters->getDtCompetencia();
        $evtAdmissao  = (array) $xml->evtAdmissao;
        $mensagens = array();
        
        if($evtAdmissao){
            $trabalhador = (array) $evtAdmissao['trabalhador'];
            $vinculo = (array) $evtAdmissao['vinculo'];
            $admissao = (array) $vinculo['infoRegimeTrab']->infoCeletista->dtAdm;
            $dtmescompetenc = DateUtil::getDataDeString($this->alterDateFormat($admissao[0], 1),DateUtil::FORMATO_BRASILEIRO,true);
            
            $pessoaExistente = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->retornaPessoaCpf($nrorg, $dtmescompetenc->format('d/m/Y'), $trabalhador['cpfTrab']);
            $vinculoExistente = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaVinculoPorMatriculaESocial($nrorg, null, $vinculo['matricula'], $dtmescompetenc, null);
            
            if(!empty($pessoa) && !empty($vinculoExistente)){
                return array('status' => false,     
                             'message' => "Esse vínculo já foi inserido. Número:".$vinculoExistente->getNrvinculom()."");
            }
            
            try{
                $this->entityManager->beginTransaction();
                
                // inserção de Pessoa
                if(empty($pessoaExistente)){
                    
                    $nascimento = (array) $trabalhador['nascimento'];
                    $documentos = (array) $trabalhador['documentos'];
                    $ctps = array_key_exists('CTPS', $documentos) ? (array) $documentos['CTPS'] : null;
                    $rg = array_key_exists('RG', $documentos) ? (array) $documentos['RG'] : null;
                    $rne = array_key_exists('RNE', $documentos) ? (array) $documentos['RNE'] : null;
                    $cnh = array_key_exists('CNH', $documentos) ? (array) $documentos['CNH'] : null;
                    $endereco = array_key_exists('endereco', $trabalhador) ? (array) $trabalhador['endereco'] : null;
                    $infoDeficiencia = array_key_exists('infoDeficiencia', $trabalhador) ? (array) $trabalhador['infoDeficiencia'] : null;
                    $contato = array_key_exists('contato', $trabalhador) ? (array) $trabalhador['contato'] : null;
                    $trabEstrangeiro = array_key_exists('trabEstrangeiro', $trabalhador) ? (array) $trabalhador['trabEstrangeiro'] : null;
                    
                    $estadoCivil = $this->entityManager->getRepository(Repositories::ESTADOCIVIL)->findOneBy(array('cdesocial' => $trabalhador['estCiv']));
                    $raca = $this->entityManager->getRepository(Repositories::GPE_RACAPESSOA)->findOneBy(array('cdesocial' => $trabalhador['racaCor']));
                    $grauinstr = $this->entityManager->getRepository(Repositories::GPE_GRAUINSTR)->findOneBy(array('cdesocial' => $trabalhador['grauInstr']));
                    $paisNac = $this->entityManager->getRepository(Repositories::PAIS)->findOneBy(array('cdesocial' => $nascimento['paisNac'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                    $nacionalidade = $this->entityManager->getRepository(Repositories::GPE_NACIONALIDADE)->findOneBy(array('cdpais' => $paisNac->getCdpais())); 
                    $paisNascto = $this->entityManager->getRepository(Repositories::PAIS)->findOneBy(array('cdesocial' => $nascimento['paisNascto'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                    $codMunic = $this->entityManager->getRepository(Repositories::MUNICIPIO)->findOneBy(array('cdmunicibge' => $nascimento['codMunic'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                    
                    $dtnascimento = $this->alterDateFormat($nascimento['dtNascto'], 2);
                    $nrCondicaoFisica = $this->retornaCondicaoFisica($infoDeficiencia);
                    
                    $validaPis = $this->validaPis($trabalhador['nisTrab']);
                    
                    if($validaPis){
                        $nrPIS = $trabalhador['nisTrab'];
                        $nrnitpessoa = null;
                    }else{
                        $nrPIS = null;
                        $nrnitpessoa = $trabalhador['nisTrab'];
                    }
                    
                    if(empty($ctps)){
                        $ctps['nrCtps'] = null;
                        $ctps['serieCtps'] = null;
                        $ctps['ufCtps'] = null;
                    }
                    
                    if(empty($rg)){
                        $rg['nrRg'] = null;
                        $rg['orgaoEmissor'] = null;
                        $rg['dtExped'] = null;
                    }else{
                        $rg['dtExped'] = $this->alterDateFormat($rg['dtExped'], 2);
                    }
                    
                    if(empty($rne)){
                        $rne['nrRne'] = null;
                        $rne['orgaoEmissor'] = null;
                        $rne['dtExped'] = null;
                    }else{
                        $rne['dtExped'] = $this->alterDateFormat($rne['dtExped'], 2);
                    }
                    
                    if(empty($cnh)){
                        $cnh['nrRegCnh'] = null;
                        $cnh['dtExped'] = null;
                        $cnh['ufCnh'] = null;
                        $cnh['dtValid'] = null;
                        $cnh['dtPriHab'] = null;
                        $cnh['categoriaCnh'] = null;
                    }else{
                        $cnh['dtExped'] = $this->alterDateFormat($cnh['dtExped'], 2);
                        
                    }
                    
                    if(empty($trabEstrangeiro)){
                        $trabEstrangeiro['dtChegada'] = null;
                        $trabEstrangeiro['classTrabEstrang'] = null;
                    }else{
                        $classTrabEstrang = $this->entityManager->getRepository(Repositories::GPE_CLASSESTRANG)->findOneBy(array('cdesocial' => $trabEstrangeiro['classTrabEstrang']));
                        $trabEstrangeiro['classTrabEstrang'] = $classTrabEstrang->getNrclassestrang();
                    }
                        
                    // insere parceiro de negocio
                    $parceiro = $this->novoParcNegocio($nrorg, $trabalhador['nmTrab'], $trabalhador['nmTrab'], 'PESSOA', 'S', 'N', 'CPF', $trabalhador['cpfTrab'], DateUtil::getDataDeString($dtnascimento), 'S', 'N');
                    
                    // insere pessoa 
                    $pessoa = $this->novaPessoa($nrorg, $parceiro->getNrparcnegocio(), $dtmescompetenc, $trabalhador['nmTrab'], null, null, DateUtil::getDataDeString($dtnascimento), $trabalhador['sexo'], $nrCondicaoFisica, $trabalhador['cpfTrab'], $ctps['nrCtps'], $ctps['serieCtps'], 
                                                $ctps['ufCtps'], null, $nrPIS, null, null, null, $estadoCivil->getCdestacivil(), null, null, null, null, null, null, $cnh['nrRegCnh'], DateUtil::getDataDeString($cnh['dtExped']), DateUtil::getDataDeString($cnh['dtValid']), $cnh['categoriaCnh'],
                                                $cnh['ufCnh'], null, null, null, null, null, $raca->getNrracapessoa(), null, null, $paisNascto->getCdpais(), $nascimento['uf'], $codMunic->getCdmunicipio(), $rg['nrRg'], $rg['orgaoEmissor'], null, null, DateUtil::getDataDeString($rg['dtExped']),
                                                $nacionalidade->getNrnacionalidade(), $grauinstr->getNrgrauinstr(), null, null, $rne['nrRne'], null, null, null, null, null, null, null, null, null, null, null, null, null, $trabEstrangeiro['classTrabEstrang'], $rne['orgaoEmissor'], 
                                                DateUtil::getDataDeString($trabEstrangeiro['dtChegada']), $infoDeficiencia['infoCota'], $nrnitpessoa);
                    
                    // insere enderecoparc
                    if($endereco && array_key_exists('brasil', $endereco)){
                        $enderecoBrasil = (array) $endereco['brasil'];
                        $cidade = $this->entityManager->getRepository(Repositories::MUNICIPIO)->findOneBy(array('cdmunicibge' => $enderecoBrasil['codMunic'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                        $tpLograd = $this->entityManager->getRepository(Repositories::LOGRADOURO)->findOneBy(array('cdesocial' => $enderecoBrasil['tpLograd'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                        
                        $enderecoBrasil = $this->novoEndereco($parceiro->getNrparcnegocio(), '0055', $enderecoBrasil['uf'], $cidade->getCdmunicipio(), $tpLograd->getCdlogradouro(), $enderecoBrasil['bairro'], 'PRINCIPAL', $enderecoBrasil['cep'], $enderecoBrasil['complemento'], 
                                                              $enderecoBrasil['nrLograd'], $enderecoBrasil['dscLograd'], null);
                        unset($cidade);
                        unset($tpLograd);
                    }
                    if($endereco && array_key_exists('exterior', $endereco)){
                        $enderecoExterior = (array) $endereco['exterior']; 
                        $cidade = $this->entityManager->getRepository(Repositories::MUNICIPIO)->findOneBy(array('cdmunicibge' => $enderecoExterior['codMunic'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                        $tpLograd = $this->entityManager->getRepository(Repositories::LOGRADOURO)->findOneBy(array('cdesocial' => $enderecoExterior['tpLograd'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                        $pais = $this->entityManager->getRepository(Repositories::PAIS)->findOneBy(array('cdesocial' => $enderecoExterior['paisResid'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                        
                        $enderecoExterior = $this->novoEndereco($parceiro->getNrparcnegocio(), $pais->getCdpais(), null, $cidade->getCdmunicipio(), $tpLograd->getCdlogradouro(), $enderecoExterior['bairro'], 'EXTERIOR', $enderecoExterior['codPostal'], $enderecoExterior['complemento'], 
                                                                $enderecoExterior['nrLograd'], $enderecoExterior['dscLograd'], null);
                        unset($cidade);
                        unset($tpLograd);
                        unset($pais);
                    }
                    // insere formas de contato
                    
                    $arrayContato = [];
                    if($contato && array_key_exists('fonePrinc', $contato)){
                        $fonePrinc['CDFORMACOMU'] = '01';
                        $fonePrinc['NRPARCNEGOCIO'] = $parceiro->getNrparcnegocio();
                        
                        if(strlen($contato['fonePrinc']) > 9){
                            $fonePrinc['CDPREFIXCOMUPARC'] = substr($contato['fonePrinc'], 0, 2);
                            $fonePrinc['CDCOMUNICAPARC'] = substr($contato['fonePrinc'], 2, (strlen($contato['fonePrinc'])-2));
                        }else{
                            $fonePrinc['CDPREFIXCOMUPARC'] = '';
                            $fonePrinc['CDCOMUNICAPARC'] = $contato['fonePrinc'];
                        }
                        
                        array_push($arrayContato, $fonePrinc);
                        unset($fonePrinc);
                    }
                    if($contato && array_key_exists('foneAlternat', $contato)){
                        $foneAlternat['CDFORMACOMU'] = '02';
                        $foneAlternat['NRPARCNEGOCIO'] = $parceiro->getNrparcnegocio();
                        if(strlen($contato['foneAlternat']) > 9){
                            $foneAlternat['CDPREFIXCOMUPARC'] = substr($contato['foneAlternat'], 0, 2);
                            $foneAlternat['CDCOMUNICAPARC'] = substr($contato['foneAlternat'], 2, (strlen($contato['foneAlternat'])-2));
                        }else{
                            $foneAlternat['CDPREFIXCOMUPARC'] = '';
                            $foneAlternat['CDCOMUNICAPARC'] = $contato['foneAlternat'];
                        }
                        
                        array_push($arrayContato, $foneAlternat);
                        unset($foneAlternat);
                    }
                    if($contato && array_key_exists('emailPrinc', $contato)){
                        
                        $emailPrinc['CDFORMACOMU'] = '05';
                        $emailPrinc['NRPARCNEGOCIO'] = $parceiro->getNrparcnegocio();
                        $emailPrinc['CDPREFIXCOMUPARC'] = '';
                        $emailPrinc['CDCOMUNICAPARC'] = $contato['emailPrinc'];
                        
                        array_push($arrayContato, $emailPrinc);
                        unset($emailPrinc);
                    }
                    
                    if(!empty($arrayContato)){
                        $this->novasFormasComunica($arrayContato);
                    }
                    
                    // inserir o pais
                    $parceiroMae = $this->novoParcNegocio($nrorg, $nascimento['nmMae'], $nascimento['nmMae'], 'PESSOA', 'S', 'N', 'LIVRE', null, null, 'S', 'N'); 
                    $pessoaMae = $this->novaPessoa($nrorg, $parceiroMae->getNrparcnegocio(), $dtmescompetenc, $nascimento['nmMae'], null, null, null, 'F', null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, 
                                                  null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null);
                    $relacionaMae = $this->novoRelacionaparc($nrorg, 1, $parceiro->getNrparcnegocio(), null, $parceiroMae->getNrparcnegocio());
                    
                    $parceiroPai = $this->novoParcNegocio($nrorg, $nascimento['nmPai'], $nascimento['nmPai'], 'PESSOA', 'S', 'N', 'LIVRE', null, null, 'S', 'N'); 
                    $pessoaPai = $this->novaPessoa($nrorg, $parceiroPai->getNrparcnegocio(), $dtmescompetenc, $nascimento['nmPai'], null, null, null, 'M', null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, 
                                                  null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null);
                    $relacionaPai = $this->novoRelacionaparc($nrorg, 2, $parceiro->getNrparcnegocio(), null, $parceiroPai->getNrparcnegocio());
                    
                    $nrPessoa = $pessoa[0]->getNrpessoa();
                    $nrParcnegocio = $parceiro->getNrparcnegocio();
                    
                }else{
                    $nrPessoa = $pessoaExistente[0]['NRPESSOA'];
                    $nrParcnegocio = $pessoaExistente[0]['NRPARCNEGOCIO'];
                }
                
                $infoCeletista = (array) $vinculo['infoRegimeTrab']->infoCeletista;
                $infoContrato = (array) $vinculo['infoContrato'];
                $horario = (array) $infoContrato['horContratual'];
                $remuneracao = (array) $infoContrato['remuneracao'];
                $FGTS = (array) $infoCeletista['FGTS'];
                $duracao = array_key_exists('duracao', $infoContrato) ? (array) $infoContrato['duracao'] : null;
                $desligamento = array_key_exists('desligamento', $vinculo) ? (array) $vinculo['desligamento'] : null;
                $dependentes = array_key_exists('dependente', $trabalhador) ? $trabalhador['dependente'] : null;
                $afastamento = array_key_exists('afastamento', $vinculo) ? (array) $vinculo['afastamento'] : null;
                $sucessaoVinc = array_key_exists('sucessaoVinc', $vinculo) ? (array) $vinculo['sucessaoVinc'] : null;
                // $transfDom = (array) $vinculo['transfDom'];
                // $mudancaCPF = (array) $vinculo['mudancaCPF'];
                
                $dtAdmissao = $this->alterDateFormat($infoCeletista['dtAdm'], 2);
                $nrtpmovtransfm = $trabalhador['indPriEmpr'] == 'S' ? 1 : 2;
                $nrvinculoempreg = $infoCeletista['natAtividade'] == 1 ? 1 : 4;
                
                $estrutSind = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaPorInscricaoTipoEstrutura($nrorg, $infoCeletista['cnpjSindCategProf'], 10, $dtmescompetenc);
                $nrestrutsind = !empty($estrutSind) ? $estrutSind[0]->getNrestruturam() : null; 
                
                $dtOpcaoFGTS = $FGTS['opcFGTS'] == 1 ? DateUtil::getDataDeString($this->alterDateFormat($FGTS['dtOpcFGTS'], 2)) : null;
                
                $cargo = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOH)->retornaUltimoHistoricoCdintegracao($infoContrato['codCargo'], $nrorg, $dtmescompetenc);
                $nrcargo = !empty($cargo) ? $cargo->getNrocupacaom() : null; 
                
                $funcao = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOH)->retornaUltimoHistoricoCdintegracao($infoContrato['codFuncao'], $nrorg, $dtmescompetenc); 
                $nrfuncao = !empty($funcao) ? $funcao->getNrocupacaom() : null; 
                
                switch($remuneracao["undSalFixo"]){
                    case "1":
                        $idTipoSalario = 'HORA';
                        $idremuneracao = 'SAL2';
                        break;
                    case "2":
                        $idTipoSalario = 'DIARIO';
                        $idremuneracao = 'SAL1';
                        break;
                    case "3":
                        $idTipoSalario = 'SEMANAL';
                        $idremuneracao = 'SAL1';
                        break;
                    case "4":
                        $idTipoSalario = 'QUINZENAL';
                        $idremuneracao = 'SAL1';
                        break;
                    case "5":
                        $idTipoSalario = 'MENSAL';
                        $idremuneracao = 'SAL1';
                        break;
                    case "6":
                        $idTipoSalario = 'TAREFA';
                        $idremuneracao = 'SAL1';
                        break;
                }

                
                $observacaoSalario = (array) $infoContrato['remuneracao'];
                $observacaoSalario = array_key_exists('dscSalVar', $observacaoSalario) ? $observacaoSalario['dscSalVar'] : null;
                
                $nrinsc = (array) $infoContrato['localTrabalho']->localTrabGeral->nrInsc;
                $estrutlegal = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaPorInscricaoTipoEstrutura($nrorg, $nrinsc[0], $this->hcmParameters->getTipoEstruturaLegalOrg(), $dtmescompetenc);
                $nrestrutlegal = !empty($estrutlegal) ? $estrutlegal[0]->getNrestruturam() : null; 
                
                $hora = !empty($horario['horario']) ? (array) $horario['horario'][0] : null;
                $escalaTrab = null;
                if($hora){
                    $escalaTrab = $this->entityManager->getRepository(Repositories::GPE_ESCALATRABM)->retornaEscalaTrabalhoPorHorario($hora['dia'], $hora['codHorContrat'], $horario['qtdHrsSem'], $horario['tpJornada'], $nrorg, $competencia);
                }
                $nrEscalaTrab = !empty($escalaTrab) ? $escalaTrab['nrescalatrabm'] : null;
                
                if($duracao){
                    $dtfimcontrdetermin = !empty($duracao['dtTerm']) ? : null;
                }else{
                    $dtfimcontrdetermin = null;
                }
                
                if($desligamento){
                    $nrsitufuncm = 13;
                    $dtRescisao = $desligamento['dtDeslig'];
                }else{
                    $nrsitufuncm = 1;
                    $dtRescisao = null;
                }
                
                $estrutPosterior = null;
                if($sucessaoVinc){
                    $estrutAnterior = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaPorInscricaoTipoEstrutura($nrorg, $sucessaoVinc['cnpjEmpregAnt'], $this->hcmParameters->getTipoEstruturaLegalOrg(), $dtmescompetenc);
                    if($estrutAnterior){
                        $estrutPosterior = $nrestrutlegal;
                        $nrestrutlegal = $estrutAnterior[0]->getNrestruturam();
                    }
                }
                
                // insere vínculo
                $vinculo = $this->novoVinculo($nrPessoa, DateUtil::getDataDeString($dtAdmissao), null, 1, $infoCeletista['tpAdmissao'], $nrvinculoempreg, DateUtil::getDataDeString($dtOpcaoFGTS), DateUtil::getDataDeString($dtAdmissao), 
                                              DateUtil::getDataDeString($dtAdmissao), $trabalhador['nmTrab'], null, null, DateUtil::getDataDeString($dtAdmissao), $dtmescompetenc, $nrsitufuncm, $nrcargo, $nrEscalaTrab, null, null, null, DateUtil::getDataDeString($dtAdmissao), $idremuneracao,
                                              $idTipoSalario, $nrestrutlegal, null, 1, 'UNICO', $nrfuncao, null, $nrtpmovtransfm, DateUtil::getDataDeString($dtfimcontrdetermin), $nrestrutsind, 'N', $nrorg, $dtRescisao, null, DateUtil::getDataDeString($dtAdmissao), $vinculo['matricula'], 
                                              null, null, null);
                array_push($mensagens, 'Dados importados corretamente! Número do vínculo: '.$vinculo[0]->getNrvinculom().'.');
                array_push($mensagens, $vinculo[0]->getNrvinculom().': Inserir movimentação para uma estrutura gerencial.');
                
                // insere altesalario
                $alteSalario = $this->novaAlteSalario($nrorg, $vinculo[0]->getNrvinculom(), DateUtil::getDataDeString($dtAdmissao), 1, 1, $remuneracao["vrSalFx"], $idTipoSalario, 1, $observacaoSalario, null, null, null);
                
                // insere movimentacao sindical
                if($nrestrutsind){
                    $movimSind = $this->novaMovimentacao($nrorg, 1, $vinculo[0]->getNrvinculom(), $nrestrutsind, DateUtil::getDataDeString($dtAdmissao), 10, $nrtpmovtransfm, null, null, 0, null, null);
                }else{
                    array_push($mensagens, $vinculo[0]->getNrvinculom().': Não foi possível cadastrar uma movimentação com a estrutura sindical de CNPJ '.$infoCeletista['cnpjSindCategProf'].'.');
                }
                
                //  insere movimentacao legal
                if($nrestrutlegal){
                    $movimLegal = $this->novaMovimentacao($nrorg, 1, $vinculo[0]->getNrvinculom(), $nrestrutlegal, DateUtil::getDataDeString($dtAdmissao), $this->hcmParameters->getTipoEstruturaLegalOrg(), $nrtpmovtransfm, null, null, 0, null, null);
                    if(!empty($estrutPosterior)){
                        $dtfimmoviment = $this->alterDateFormat($sucessaoVinc['dtTransf']);
                        $movimLegalsaida = $this->novaMovimentacao($nrorg, 1, $vinculo[0]->getNrvinculom(), $estrutPosterior, DateUtil::getDataDeString($dtfimmoviment), $this->hcmParameters->getTipoEstruturaLegalOrg(), 9, null, null, 0, DateUtil::getDataDeString($dtfimmoviment), $sucessaoVinc['observacao']);
                        $movimLegalentrada = $this->novaMovimentacao($nrorg, 1, $vinculo[0]->getNrvinculom(), $estrutPosterior, DateUtil::adicionaIntervalo($dtfimmoviment, 1, DateUtil::DIA), $this->hcmParameters->getTipoEstruturaLegalOrg(), 8, null, null, 0, null, $sucessaoVinc['observacao']);
                        
                        $vinculoh = $vinculo[1];
                        $vinculoh->setDtmescompetenc(DateUtil::truncateData(DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataDeString($dtfimmoviment, DateUtil::FORMATO_BRASILEIRO, true))));
                        $vinculoh->setNrvinculoh(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_VINCULOH', 1, 12, true));
                        $vinculoh->setNrestrutlegal($estrutPosterior);
                        
                        $this->entityManager->persist($movimLegalsaida);
                        $this->entityManager->persist($movimLegalentrada);
                        $this->entityManager->persist($vinculoh);
                        $this->entityManager->flush();
                        
                    }
                }else{
                    array_push($mensagens, $vinculo[0]->getNrvinculom().': Não foi possível cadastrar uma movimentação com a estrutura legal de CNPJ '.$nrinsc[0].'.');
                }
                
                // insere altesitufunc
                $situFuncional = $this->novaAltesitufunc($nrorg, $vinculo[0]->getNrvinculom(), 1, DateUtil::getDataDeString($dtAdmissao), null, null, null, null, null, null, null, null, null, null);
                
                // if($afastamento){
                //     $iniAfastamento = $this->alterDateFormat($afastamento['dtIniAfast'], 2);
                //     $motivoAfastamento = $this->entityManager->getRepository(Repositories::GPE_MOTIVOAFASTA)->findBy(array('nrorg' => array($nrorg, $nrorgpadrao), 'cdesocial' => $afastamento['codMotAfast']));
                //     if(count($motivoAfastamento) == 1){
                //         $situFuncional->setDtfimsitufunc(DateUtil::subtraiIntervalo($iniAfastamento, 1, DateUtil::DIA));
                //         $alteafastamento = $this->novaAltesitufunc($nrorg, $vinculo->getNrvinculom(), $motivoAfastamento[0]->getNrmotivoafasta(), DateUtil::getDataDeString($iniAfastamento), null, null, null, null, null, null, null, null, null, null);
                //         array_push($mensagens, $vinculo->getNrvinculom().': Complete o cadastro de alteração de situação funcional (número:'.$alteafastamento->getNraltesitufunc().').');
                //     }else if(count($motivoAfastamento) == 0){
                //         array_push($mensagens, $vinculo->getNrvinculom().': Não foi possível cadastrar uma alteração de situação funcional, pois não foi encontrado afastamento de CDESOCIAL '.$afastamento['codMotAfast'].'.');
                //     }else if(count($motivoAfastamento) > 1){
                //         array_push($mensagens, $vinculo->getNrvinculom().': Não foi possível cadastrar uma alteração de situação funcional, pois foram encontrados vários afastamentos de CDESOCIAL '.$afastamento['codMotAfast'].'.');
                //     }
                // }
                
                if($desligamento){
                    $situFuncional->setDtfimsitufunc(DateUtil::subtraiIntervalo($dtRescisao, 1, DateUtil::DIA));
                    $situFuncDesligamento = $this->novaAltesitufunc($nrorg, $vinculo[0]->getNrvinculom(), 13, DateUtil::getDataDeString($dtRescisao), null, null, null, null, null, null, null, null, null, null);
                }
                
                // insere alteocupacao
                if($nrcargo){
                    $ocupacaom = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOM)->findOneBy(array('nrorg' => $nrorg, 'nrocupacaom' => $nrcargo));
                    $alteOcupacao = $this->novaAlteOcupacao($nrorg, $vinculo[0]->getNrvinculom(), $nrcargo, DateUtil::getDataDeString($dtAdmissao), $ocupacaom->getNrtipoocupacao(), 1, null, null, null, null);
                }
                
                // insere alteescala
                if($nrEscalaTrab){
                    $alteEscala = $this->novaAlteEscala($nrorg, $vinculo[0]->getNrvinculom(), $nrEscalaTrab, DateUtil::getDataDeString($dtAdmissao), null, null, null, null, $escalaTrab['nrturno'], null);
                }else{
                    array_push($mensagens, $vinculo[0]->getNrvinculom().': Não foi encontrada escala para adicionar a alteração de escala de trabalho.');
                }
                
                if($dependentes){
                    $dependentes = is_array($dependentes) ? $dependentes : [$dependentes]; 
                    foreach($dependentes as $dependente){
                        $dependente = (array) $dependente;
                        
                        $dtNascimento = $this->alterDateFormat($dependente['dtNascto'], 2);
                        $tiporelaciona = $this->entityManager->getRepository(Repositories::TIPORELACIONA)->findOneBy(array('nrorg' => array($nrorg, $nrorgpadrao), 'cdesocial' => $dependente['tpDep']));
                        
                        $parceiroDepend = $this->novoParcNegocio($nrorg, $dependente['nmDep'], $dependente['nmDep'], 'PESSOA', 'S', 'N', 'LIVRE', null, null, 'S', 'N'); 
                        $pessoaDepend = $this->novaPessoa($nrorg, $parceiroDepend->getNrparcnegocio(), $dtmescompetenc, $dependente['nmDep'], null, null, DateUtil::getDataDeString($dtNascimento), null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, 
                                                          null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, 
                                                          null, null, null, null, null, null, null);
                        $relacionaparceiroDepend = $this->novoRelacionaparc($nrorg, $tiporelaciona->getNrtiporelaciona(), $nrParcnegocio, null, $parceiroDepend->getNrparcnegocio());
                        
                        if($dependente['tpDep'] == '03'){
                            // Imposto de Renda
                            $dependenteVincIRRF = $this->novoDepVinculo($nrorg, $pessoaDepend[0]->getNrpessoa(), 2, $vinculo[0]->getNrvinculom());
                            // Salário Família
                            $dependenteVincSF = $this->novoDepVinculo($nrorg, $pessoaDepend[0]->getNrpessoa(), 1, $vinculo[0]->getNrvinculom());
                            array_push($mensagens, $vinculo[0]->getNrvinculom().': Verifique se as dependências estão corretas.(Pessoa dependende: '.$pessoaDepend[0]->getNrpessoa().')');
                        }
                    }
                    
                    
                }
                
                $this->entityManager->commit(); 
                
            }catch (\Exception $e) {   
                $this->entityManager->rollback();
                throw $e;
            }    
            
            return array('status' => true,     
                         'messages' => $mensagens);
            
        }else{
            return array('status' => false,     
                         'message' => self::MSG_ERRO_XML . " (Ausência da tag 'evtAdmissao' no arquivo).");
        }
    }
    
    public function insertXmlS2205($xml){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
        $competencia = $this->hcmParameters->getDtCompetencia();
        $evtAltCadastral  = (array) $xml->evtAltCadastral;
        $mensagens = array();
        
        if($evtAltCadastral){
            $ideTrabalhador = array_key_exists('ideTrabalhador', $evtAltCadastral) ? (array) $evtAltCadastral['ideTrabalhador'] : null;
            $alteracao = (array) $evtAltCadastral['alteracao'];
            $dtAlteracao = $this->alterDateFormat($alteracao['dtAlteracao'], 2);
            $compAlteracao = DateUtil::getDataDeString($this->alterDateFormat($alteracao['dtAlteracao'], 1), DateUtil::FORMATO_BRASILEIRO, true);
            $dadosTrabalhador = (array) $alteracao['dadosTrabalhador'];
            
            $pessoa = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->retornaPessoaCpf($nrorg, $dtAlteracao, $ideTrabalhador['cpfTrab']);
            
            if($pessoa){
                
                try{
                    $this->entityManager->beginTransaction();
                    
                    // dados pessoa
                    $nascimento = (array) $dadosTrabalhador['nascimento'];
                    $documentos = (array) $dadosTrabalhador['documentos'];
                    $ctps = array_key_exists('CTPS', $documentos) ? (array) $documentos['CTPS'] : null;
                    $rg = array_key_exists('RG', $documentos) ? (array) $documentos['RG'] : null;
                    $rne = array_key_exists('RNE', $documentos) ? (array) $documentos['RNE'] : null;
                    $cnh = array_key_exists('CNH', $documentos) ? (array) $documentos['CNH'] : null;
                    $endereco = array_key_exists('endereco', $dadosTrabalhador) ? (array) $dadosTrabalhador['endereco'] : null;
                    $infoDeficiencia = array_key_exists('infoDeficiencia', $dadosTrabalhador) ? (array) $dadosTrabalhador['infoDeficiencia'] : null;
                    $contato = array_key_exists('contato', $dadosTrabalhador) ? (array) $dadosTrabalhador['contato'] : null;
                    $trabEstrangeiro = array_key_exists('trabEstrangeiro', $dadosTrabalhador) ? (array) $dadosTrabalhador['trabEstrangeiro'] : null;
                    
                    $estadoCivil = $this->entityManager->getRepository(Repositories::ESTADOCIVIL)->findOneBy(array('cdesocial' => $dadosTrabalhador['estCiv']));
                    $raca = $this->entityManager->getRepository(Repositories::GPE_RACAPESSOA)->findOneBy(array('cdesocial' => $dadosTrabalhador['racaCor']));
                    $grauinstr = $this->entityManager->getRepository(Repositories::GPE_GRAUINSTR)->findOneBy(array('cdesocial' => $dadosTrabalhador['grauInstr']));
                    $paisNac = $this->entityManager->getRepository(Repositories::PAIS)->findOneBy(array('cdesocial' => $nascimento['paisNac'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                    $nacionalidade = $this->entityManager->getRepository(Repositories::GPE_NACIONALIDADE)->findOneBy(array('cdpais' => $paisNac->getCdpais())); 
                    $paisNascto = $this->entityManager->getRepository(Repositories::PAIS)->findOneBy(array('cdesocial' => $nascimento['paisNascto'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                    $codMunic = $this->entityManager->getRepository(Repositories::MUNICIPIO)->findOneBy(array('cdmunicibge' => $nascimento['codMunic'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                    
                    $dtnascimento = $this->alterDateFormat($nascimento['dtNascto'], 2);
                    $nrCondicaoFisica = $this->retornaCondicaoFisica($infoDeficiencia);
                    
                    $validaPis = $this->validaPis($dadosTrabalhador['nisTrab']);
                    
                    if($validaPis){
                        $nrPIS = $dadosTrabalhador['nisTrab'];
                        $nrnitpessoa = null;
                    }else{
                        $nrPIS = null;
                        $nrnitpessoa = $dadosTrabalhador['nisTrab'];
                    }
                    
                    if(empty($ctps)){
                        $ctps['nrCtps'] = null;
                        $ctps['serieCtps'] = null;
                        $ctps['ufCtps'] = null;
                    }
                    
                    if(empty($rg)){
                        $rg['nrRg'] = null;
                        $rg['orgaoEmissor'] = null;
                        $rg['dtExped'] = null;
                    }else{
                        $rg['dtExped'] = $this->alterDateFormat($rg['dtExped'], 2);
                    }
                    
                    if(empty($rne)){
                        $rne['nrRne'] = null;
                        $rne['orgaoEmissor'] = null;
                        $rne['dtExped'] = null;
                    }else{
                        $rne['dtExped'] = $this->alterDateFormat($rne['dtExped'], 2);
                    }
                    
                    if(empty($cnh)){
                        $cnh['nrRegCnh'] = null;
                        $cnh['dtExped'] = null;
                        $cnh['ufCnh'] = null;
                        $cnh['dtValid'] = null;
                        $cnh['dtPriHab'] = null;
                        $cnh['categoriaCnh'] = null;
                    }else{
                        $cnh['dtExped'] = $this->alterDateFormat($cnh['dtExped'], 2);
                        
                    }
                    
                    if(empty($trabEstrangeiro)){
                        $trabEstrangeiro['dtChegada'] = null;
                        $trabEstrangeiro['classTrabEstrang'] = null;
                    }else{
                        $classTrabEstrang = $this->entityManager->getRepository(Repositories::GPE_CLASSESTRANG)->findOneBy(array('cdesocial' => $trabEstrangeiro['classTrabEstrang']));
                        $trabEstrangeiro['classTrabEstrang'] = $classTrabEstrang->getNrclassestrang();
                    }
                    
                    $pessoah = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->retornaHistoricoAtualPorPessoa($nrorg, $pessoa[0]['NRPESSOA'], $compAlteracao);
                    
                    if($pessoah->getDtmescompetenc() == $compAlteracao){
                        
                        list($primeiroNome, $meioNome, $ultimoNome) = self::decompoeNomePessoa($dadosTrabalhador['nmTrab']);
                        
                        $pessoah->setNmpessoa($dadosTrabalhador['nmTrab']);
                        $pessoah->setDsprinomepess($primeiroNome);
                        $pessoah->setDsnomemeiopess((strlen($meioNome)> 20) ? substr($meioNome, 0, 20) : $meioNome);
                        $pessoah->setDsultnomepess($ultimoNome);
                        $pessoah->setDtnascpessoa(DateUtil::getDataDeString($dtnascimento, DateUtil::FORMATO_BRASILEIRO, true));
                        $pessoah->setIdsexopessoa($dadosTrabalhador['sexo']);
                        $pessoah->setNrcondfispes($nrCondicaoFisica);
                        $pessoah->setNrctpspessoa($ctps['nrCtps']);
                        $pessoah->setNrseriectpspes($ctps['serieCtps']);
                        $pessoah->setSgufctpspes($ctps['ufCtps']);
                        $pessoah->setNrpispaseppes($nrPIS);
                        $pessoah->setCdestacivil($estadoCivil->getCdestacivil());
                        $pessoah->setNrcarthabpes($cnh['nrRegCnh']);
                        $pessoah->setDthabcartpes(DateUtil::getDataDeString($cnh['dtExped'], DateUtil::FORMATO_BRASILEIRO, true));
                        $pessoah->setDtvalhabcartpes(DateUtil::getDataDeString($cnh['dtValid'], DateUtil::FORMATO_BRASILEIRO, true));
                        $pessoah->setDscateghabcart($cnh['categoriaCnh']);
                        // $pessoah->set($cnh['ufCnh']);
                        $pessoah->setNrracapessoa($raca->getNrracapessoa());
                        $pessoah->setCdpais($paisNascto->getCdpais());
                        $pessoah->setSgestado($nascimento['uf']);
                        $pessoah->setCdmunicipio($codMunic->getCdmunicipio());
                        $pessoah->setNrrgpessoa($rg['nrRg']);
                        $pessoah->setCdexrgpessoa($rg['orgaoEmissor']);
                        $pessoah->setDtexrgpessoa(DateUtil::getDataDeString($rg['dtExped'], DateUtil::FORMATO_BRASILEIRO, true));
                        $pessoah->setNrnacionalid($nacionalidade->getNrnacionalidade());
                        $pessoah->setNrgrauinstr($grauinstr->getNrgrauinstr());
                        $pessoah->setNrdocuestrang($rne['nrRne']);
                        $pessoah->setNrclassestrang($trabEstrangeiro['classTrabEstrang']);
                        $pessoah->setDsexpdocestr($rne['orgaoEmissor']);
                        $pessoah->setDsexpdocestr(DateUtil::getDataDeString($trabEstrangeiro['dtChegada'], DateUtil::FORMATO_BRASILEIRO, true));
                        $pessoah->setIdcotapcd($infoDeficiencia['infoCota']);
                        $pessoah->setDsexpdocestr($nrnitpessoa);
                        
                        $this->entityManager->persist($pessoah);
                        $this->entityManager->flush();
                        
                    }else{
                        $this->pessoaHistorico($nrorg, $pessoah->getNrpessoa(), $compAlteracao, $dadosTrabalhador['nmTrab'], $pessoah->getDsapelidopess(), $pessoah->getNrcertnascpes(), DateUtil::getDataDeString($dtnascimento, DateUtil::FORMATO_BRASILEIRO, true), $dadosTrabalhador['sexo'], 
                                               $nrCondicaoFisica, $pessoah->getNrcpfpessoa(), $ctps['nrCtps'], $ctps['serieCtps'], $ctps['ufCtps'], $pessoah->getDtctpspessoa(), $nrPIS, $pessoah->getDtinscpispasep(), $pessoah->getNrinscinsspes(), $pessoah->getNrinscisspes(), 
                                               $estadoCivil->getCdestacivil(), $pessoah->getNrcertcasapes(), $pessoah->getDtcasapessoa(), $pessoah->getIdisentituelei(), $pessoah->getNrtitueleipes(), $pessoah->getNrsecaeleipes(), $pessoah->getNrzonaeleipes(), $cnh['nrRegCnh'], 
                                               DateUtil::getDataDeString($cnh['dtExped'], DateUtil::FORMATO_BRASILEIRO, true), DateUtil::getDataDeString($cnh['dtValid'], DateUtil::FORMATO_BRASILEIRO, true), $cnh['categoriaCnh'], $cnh['ufCnh'], $pessoah->getNrcatmilitarpe(), 
                                               $pessoah->getNrcertresepes(), $pessoah->getDscategcertrese(), $pessoah->getDtcertresepes(), $pessoah->getCdexcertresepes(), $raca->getNrracapessoa(), $pessoah->getIdgrupsangupes(), $pessoah->getIdfatorrhpes(), $paisNascto->getCdpais(), 
                                               $nascimento['uf'], $codMunic->getCdmunicipio(), $rg['nrRg'], $rg['orgaoEmissor'], $pessoah->getSgufrgpessoa(), $pessoah->getDslocalexrgpes(), DateUtil::getDataDeString($rg['dtExped'], DateUtil::FORMATO_BRASILEIRO, true),
                                               $nacionalidade->getNrnacionalidade(), $grauinstr->getNrgrauinstr(), $pessoah->getDspronometrat(), $pessoah->getNranochegapais(), $rne['nrRne'], $pessoah->getIdpesnaturaliza(), $pessoah->getDtnaturalizapes(), $pessoah->getIdposvistoperm(),
                                               $pessoah->getDtpermavisto(), $pessoah->getNrconselhoreg(), $pessoah->getNrinscrconsreg(), $pessoah->getDtexconsreg(), $pessoah->getSgufconsreg(), $pessoah->getCdtpbaixapag(), $pessoah->getNrlivrocertnasc(), $pessoah->getNrfolhacertnasc(),
                                               $pessoah->getDscomplcertnasc(), $pessoah->getNrregctpspes(), $trabEstrangeiro['classTrabEstrang'], $rne['orgaoEmissor'], DateUtil::getDataDeString($trabEstrangeiro['dtChegada'], DateUtil::FORMATO_BRASILEIRO, true), $infoDeficiencia['infoCota'], 
                                               $nrnitpessoa);  
                    }
                                           
                    // dados endereço
                    $enderecoBr = $this->entityManager->getRepository(Repositories::ENDERECOPARC)->findOneBy(array('cdpais' => '0055', 'nrparcnegocio' => $pessoa[0]['NRPARCNEGOCIO'], 'nrorg' => $nrorg));
                    if(array_key_exists('brasil', $endereco) && empty($enderecoBr)){
                        $enderecoBrasil = (array) $endereco['brasil'];
                        $cidade = $this->entityManager->getRepository(Repositories::MUNICIPIO)->findOneBy(array('cdmunicibge' => $enderecoBrasil['codMunic'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                        $tpLograd = $this->entityManager->getRepository(Repositories::LOGRADOURO)->findOneBy(array('cdesocial' => $enderecoBrasil['tpLograd'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                        
                        $enderecoBrasil = $this->novoEndereco($pessoa[0]['NRPARCNEGOCIO'], '0055', $enderecoBrasil['uf'], $cidade->getCdmunicipio(), $tpLograd->getCdlogradouro(), $enderecoBrasil['bairro'], 'PRINCIPAL', $enderecoBrasil['cep'], $enderecoBrasil['complemento'], 
                                                              $enderecoBrasil['nrLograd'], $enderecoBrasil['dscLograd'], null);
                        unset($cidade);
                        unset($tpLograd);
                    }
                    
                    $enderecoEx = $this->entityManager->getRepository(Repositories::ENDERECOPARC)->retornaEnderecoParcExterior($pessoa[0]['NRPARCNEGOCIO'], $nrorg);
                    if(array_key_exists('exterior', $endereco) && empty($enderecoEx)){
                        $enderecoExterior = (array) $endereco['exterior']; 
                        $cidade = $this->entityManager->getRepository(Repositories::MUNICIPIO)->findOneBy(array('cdmunicibge' => $enderecoExterior['codMunic'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                        $tpLograd = $this->entityManager->getRepository(Repositories::LOGRADOURO)->findOneBy(array('cdesocial' => $enderecoExterior['tpLograd'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                        $pais = $this->entityManager->getRepository(Repositories::PAIS)->findOneBy(array('cdesocial' => $enderecoExterior['paisResid'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                        
                        $enderecoExterior = $this->novoEndereco($pessoa[0]['NRPARCNEGOCIO'], $pais->getCdpais(), null, $cidade->getCdmunicipio(), $tpLograd->getCdlogradouro(), $enderecoExterior['bairro'], 'EXTERIOR', $enderecoExterior['codPostal'], $enderecoExterior['complemento'], 
                                                                $enderecoExterior['nrLograd'], $enderecoExterior['dscLograd'], null);
                        unset($cidade);
                        unset($tpLograd);
                        unset($pais);
                    }
                    // dados pais
                    $relacionaM = $this->entityManager->getRepository(Repositories::RELACIONAPARC)->findOneBy(array('nrorg' => $nrorg, 'nrparcnegocio' => $pessoa[0]['NRPARCNEGOCIO'], 'nrtiporelaciona' => 1));
                    if($nascimento['nmMae'] && empty($relacionaM)){
                        $parceiroMae = $this->novoParcNegocio($nrorg, $nascimento['nmMae'], $nascimento['nmMae'], 'PESSOA', 'S', 'N', 'LIVRE', null, null, 'S', 'N'); 
                        $pessoaMae = $this->novaPessoa($nrorg, $parceiroMae->getNrparcnegocio(), $dtmescompetenc, $nascimento['nmMae'], null, null, null, 'F', null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, 
                                                      null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null);
                        $relacionaMae = $this->novoRelacionaparc($nrorg, 1, $pessoa[0]['NRPARCNEGOCIO'], null, $parceiroMae->getNrparcnegocio());
                    }
                    
                    $relacionaP = $this->entityManager->getRepository(Repositories::RELACIONAPARC)->findOneBy(array('nrorg' => $nrorg, 'nrparcnegocio' => $pessoa[0]['NRPARCNEGOCIO'], 'nrtiporelaciona' => 2));
                    if($nascimento['nmPai'] && empty($relacionaP)){
                        $parceiroPai = $this->novoParcNegocio($nrorg, $nascimento['nmPai'], $nascimento['nmPai'], 'PESSOA', 'S', 'N', 'LIVRE', null, null, 'S', 'N'); 
                        $pessoaPai = $this->novaPessoa($nrorg, $parceiroPai->getNrparcnegocio(), $dtmescompetenc, $nascimento['nmPai'], null, null, null, 'M', null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, 
                                                      null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null);
                        $relacionaPai = $this->novoRelacionaparc($nrorg, 2, $pessoa[0]['NRPARCNEGOCIO'], null, $parceiroPai->getNrparcnegocio());
                    }
                    
                    // daods contato
                    $arrayContato = [];
                    $comunicafonePrinc = $this->entityManager->getRepository(Repositories::COMUNICAPARC)->findOneBy(array('nrorg' => $nrorg, 'nrparcnegocio' => $pessoa[0]['NRPARCNEGOCIO'], 'cdformacomu' => '01'));
                    if(array_key_exists('fonePrinc', $contato) && empty($comunicafonePrinc)){
                        $fonePrinc['CDFORMACOMU'] = '01';
                        $fonePrinc['NRPARCNEGOCIO'] = $pessoa[0]['NRPARCNEGOCIO'];
                        
                        if(strlen($contato['fonePrinc']) > 9){
                            $fonePrinc['CDPREFIXCOMUPARC'] = substr($contato['fonePrinc'], 0, 2);
                            $fonePrinc['CDCOMUNICAPARC'] = substr($contato['fonePrinc'], 2, (strlen($contato['fonePrinc'])-2));
                        }else{
                            $fonePrinc['CDPREFIXCOMUPARC'] = '';
                            $fonePrinc['CDCOMUNICAPARC'] = $contato['fonePrinc'];
                        }
                        
                        array_push($arrayContato, $fonePrinc);
                        unset($fonePrinc);
                    }
                    $comunicafoneAlternat = $this->entityManager->getRepository(Repositories::COMUNICAPARC)->findOneBy(array('nrorg' => $nrorg, 'nrparcnegocio' => $pessoa[0]['NRPARCNEGOCIO'], 'cdformacomu' => '02'));
                    if(array_key_exists('foneAlternat', $contato) && empty($comunicafoneAlternat)){
                        $foneAlternat['CDFORMACOMU'] = '02';
                        $foneAlternat['NRPARCNEGOCIO'] = $pessoa[0]['NRPARCNEGOCIO'];
                        if(strlen($contato['foneAlternat']) > 9){
                            $foneAlternat['CDPREFIXCOMUPARC'] = substr($contato['foneAlternat'], 0, 2);
                            $foneAlternat['CDCOMUNICAPARC'] = substr($contato['foneAlternat'], 2, (strlen($contato['foneAlternat'])-2));
                        }else{
                            $foneAlternat['CDPREFIXCOMUPARC'] = '';
                            $foneAlternat['CDCOMUNICAPARC'] = $contato['foneAlternat'];
                        }
                        
                        array_push($arrayContato, $foneAlternat);
                        unset($foneAlternat);
                    }
                    $comunicaemailPrinc = $this->entityManager->getRepository(Repositories::COMUNICAPARC)->findOneBy(array('nrorg' => $nrorg, 'nrparcnegocio' => $pessoa[0]['NRPARCNEGOCIO'], 'cdformacomu' => '05'));
                    if(array_key_exists('emailPrinc', $contato) && empty($comunicaemailPrinc)){
                        
                        $emailPrinc['CDFORMACOMU'] = '05';
                        $emailPrinc['NRPARCNEGOCIO'] = $pessoa[0]['NRPARCNEGOCIO'];
                        $emailPrinc['CDPREFIXCOMUPARC'] = '';
                        $emailPrinc['CDCOMUNICAPARC'] = $contato['emailPrinc'];
                        
                        array_push($arrayContato, $emailPrinc);
                        unset($emailPrinc);
                    }
                    
                    if(!empty($arrayContato)){
                        $this->novasFormasComunica($arrayContato);
                    }
                    
                    $this->entityManager->commit();
                    
                }catch (\Exception $e) {   
                    $this->entityManager->rollback();
                    throw $e;
                }
                
                
                return array('status' => true,     
                             'messages' => ['Dados importados corretamente! Número da pessoa: '.$pessoah->getNrpessoa()]);
            
            }else{
                return array('status' => false,     
                             'message' => "Não foi encontrada pessoa com esse CPF.");    
            }
        }else{
            return array('status' => false,     
                         'message' => self::MSG_ERRO_XML . " (Ausência da tag 'evtAltCadastral' no arquivo).");
        }
        
    }
    
    public function insertXmlS2206($xml){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
        $competencia = $this->hcmParameters->getDtCompetencia();
        $evtAltContratual  = (array) $xml->evtAltContratual;
        $mensagens = array();
        
        if($evtAltContratual){
            $altContratual = (array) $evtAltContratual['altContratual'];
            $ideVinculo = (array) $evtAltContratual['ideVinculo'];
            $dtAlteracao = $this->alterDateFormat($altContratual['dtAlteracao'], 2);
            $compAlteracao = DateUtil::getDataDeString($this->alterDateFormat($altContratual['dtAlteracao'], 1), DateUtil::FORMATO_BRASILEIRO, true);
            
            $vinculo = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaVinculoPorMatriculaESocial($nrorg, null, $ideVinculo['matricula'], $compAlteracao, null);
            
            if($vinculo){
                
                try{
                    $this->entityManager->beginTransaction();
                    
                    // dados vínculo
                                    
                    $infoCeletista = (array) $altContratual['infoRegimeTrab']->infoCeletista;
                    $infoContrato = (array) $altContratual['infoContrato'];
                    $horario = (array) $infoContrato['horContratual'];
                    $remuneracao = (array) $infoContrato['remuneracao'];
                    $duracao = array_key_exists('duracao', $infoContrato) ? (array) $infoContrato['duracao'] : null;
                    $localTrabGeral = (array) $infoContrato['localTrabalho']->localTrabGeral;
                    
                    
                    $estrutSind = array_key_exists('cnpjSindCategProf', $infoCeletista) ? $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaPorInscricaoTipoEstrutura($nrorg, $infoCeletista['cnpjSindCategProf'], 10, $compAlteracao) : null;
                    $nrestrutsind = !empty($estrutSind) && $estrutSind[0]->getNrestruturam() != $vinculo->getNrestrutsind()? $estrutSind[0]->getNrestruturam() : null; 
                    
                    $estrutLegal = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaPorInscricaoTipoEstrutura($nrorg, $localTrabGeral['nrInsc'], $this->hcmParameters->getTipoEstruturaLegalOrg(), $compAlteracao);
                    $nrestrutlegal = !empty($estrutLegal) && $estrutLegal[0]->getNrestruturam() != $vinculo->getNrestrutlegal()? $estrutLegal[0]->getNrestruturam() : null; 
                    
                    $cargo = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOH)->retornaUltimoHistoricoCdintegracao($infoContrato['codCargo'], $nrorg, $compAlteracao);
                    $nrcargo = !empty($cargo) && $cargo->getNrocupacaom() != $vinculo->getNrcargo() ? $cargo->getNrocupacaom() : null; 
                    
                    $funcao = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOH)->retornaUltimoHistoricoCdintegracao($infoContrato['codFuncao'], $nrorg, $compAlteracao); 
                    $nrfuncao = !empty($funcao)  && $funcao->getNrocupacaom() != $vinculo->getNrfuncao() ? $funcao->getNrocupacaom() : null; 
                    
                    $nrinsc = (array) $infoContrato['localTrabalho']->localTrabGeral->nrInsc;
                    $estrutlegal = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaPorInscricaoTipoEstrutura($nrorg, $nrinsc[0], $this->hcmParameters->getTipoEstruturaLegalOrg(), $compAlteracao);
                    $nrestrutlegal = !empty($estrutlegal) ? $estrutlegal[0]->getNrestruturam() : null; 
                    
                    $hora = !empty($horario['horario']) ? (array) $horario['horario'][0] : null;
                    $escalaTrab = null;
                    if($hora){
                        $escalaTrab = $this->entityManager->getRepository(Repositories::GPE_ESCALATRABM)->retornaEscalaTrabalhoPorHorario($hora['dia'], $hora['codHorContrat'], $horario['qtdHrsSem'], $horario['tpJornada'], $nrorg, $compAlteracao);
                    }
                    $nrEscalaTrab = !empty($escalaTrab) && $escalaTrab['nrescalatrabm'] != $vinculo->getNrescalatrabm()? $escalaTrab['nrescalatrabm'] : null;
                    
                    if($duracao){
                        $dtfimcontrdetermin = !empty($duracao['dtTerm']) ? : null;
                    }else{
                        $dtfimcontrdetermin = null;
                    }
                    
                    if(!empty($nrcargo)){
                        
                        $cargom = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOM)->findOneBy(array('nrorg' => $nrorg, 'nrocupacaom' => $nrcargo));
                        $alteOcupacao = $this->novaAlteOcupacao($nrorg, $vinculo->getNrvinculom(), $nrcargo, DateUtil::getDataDeString($compAlteracao), $cargom->getNrtipoocupacao(), 1, null, null, null, null);
                    }else{
                        $nrcargo = $vinculo->getNrcargo();
                        $nrfuncao = $vinculo->getNrfuncao();
                    }
                    
                    if($nrestrutlegal){
                            $movimLegalsaida = $this->novaMovimentacao($nrorg, 1, $vinculo->getNrvinculom(), $vinculo->getNrestrutlegal(), DateUtil::subtraiIntervalo($dtAlteracao, 1, DateUtil::DIA), $this->hcmParameters->getTipoEstruturaLegalOrg(), 9, null, null, 0, 
                                                                       DateUtil::subtraiIntervalo($dtAlteracao, 1, DateUtil::DIA), '');
                            $movimLegalentrada = $this->novaMovimentacao($nrorg, 1, $vinculo->getNrvinculom(), $nrestrutlegal, DateUtil::getDataDeString($dtAlteracao), $this->hcmParameters->getTipoEstruturaLegalOrg(), 8, null, null, 0, null, '');
                            
                    }else{
                        $nrestrutlegal = $vinculo->getNrestrutlegal(); 
                    }
                    
                    if($nrEscalaTrab){
                        $alteEscala = $this->novaAlteEscala($nrorg, $vinculo->getNrvinculom(), $nrEscalaTrab, DateUtil::getDataDeString($compAlteracao), null, null, null, null, $escalaTrab['nrturno'], null);
                    }else{
                        $nrEscalaTrab = $vinculo->getNrescalatrabm();
                    }
                    
                    $salarioAtual = $this->entityManager->getRepository(Repositories::GPE_ALTESALARIO)->retornaAlteSalarioAtual($nrorg, $vinculo->getNrvinculom(), $compAlteracao);
                    if(array_key_exists('vrSalFx', $remuneracao) && $remuneracao["vrSalFx"] > $salarioAtual->getVrsalario()){
                         switch($remuneracao["undSalFixo"]){
                            case "1":
                                $idTipoSalario = 'HORA';
                                $idremuneracao = 'SAL2';
                                break;
                            case "2":
                                $idTipoSalario = 'DIARIO';
                                $idremuneracao = 'SAL1';
                                break;
                            case "3":
                                $idTipoSalario = 'SEMANAL';
                                $idremuneracao = 'SAL1';
                                break;
                            case "4":
                                $idTipoSalario = 'QUINZENAL';
                                $idremuneracao = 'SAL1';
                                break;
                            case "5":
                                $idTipoSalario = 'MENSAL';
                                $idremuneracao = 'SAL1';
                                break;
                            case "6":
                                $idTipoSalario = null;
                                $idremuneracao = 'SAL1';
                                break;
                        }
                        $observacaoSalario = (array) $infoContrato['remuneracao'];
                        $observacaoSalario = array_key_exists('dscSalVar', $observacaoSalario) ? $observacaoSalario['dscSalVar'] : null;
                        $nrtpmodalidsal = 1;
                        
                        $alteSalario = $this->novaAlteSalario($nrorg, $vinculo->getNrvinculom(), DateUtil::getDataDeString($compAlteracao), 1, 1, $remuneracao["vrSalFx"], $idTipoSalario, 1, $observacaoSalario, null, null, null);
                    }else{
                        $idremuneracao = $vinculo->getIdremuneracao();
                        $idTipoSalario = $vinculo->getIdtppagamento();
                        $nrtpmodalidsal = $vinculo->getNrtpmodalidsal();
                    }
                    
                    if($vinculo->getDtmescompetenc() == $compAlteracao){
                        
                        // $vinculoh->setDtesocial($dtAlteracao);
                        $vinculoh->setNrcargo($nrcargo);
                        $vinculoh->setNrescalatrabm($nrEscalaTrab);
                        $vinculoh->setIdremuneracao($idremuneracao);
                        $vinculoh->setIdtppagamento($idTipoSalario);
                        $vinculoh->setNrestrutlegal($nrestrutlegal);
                        $vinculoh->setNrtpmodalidsal($nrtpmodalidsal);
                        $vinculoh->setNrfuncao($nrfuncao);
                        $vinculoh->setNrestrutsind($nrestrutsind);
                        
                        $this->entityManager->persist($vinculoh);
                        $this->entityManager->flush();
                        
                    }else{
                         
                        $this->vinculoHistorico($nrorg, $vinculo->getNrvinculom(), $compAlteracao, $vinculo->getNrsitufuncm(), $nrcargo, $nrEscalaTrab, $vinculo->getIdcontribuisind(), $vinculo->getNrdependir(), $vinculo->getNrdependsfam(), $vinculo->getDtbaseferias(), $idremuneracao, 
                                                $idTipoSalario, $nrestrutlegal, $vinculo->getNrestrutgeren(), $nrtpmodalidsal, $vinculo->getIdmultvinc(), $nrfuncao, $vinculo->getCdcontribindividual(), $vinculo->getNrtpmovtransfm(), $vinculo->getDtfimcontrdetermin(), $nrestrutsind, 
                                                $vinculo->getConfidencial(), $vinculo->getCdmatriculaEsocial(), $vinculo->getIdpagpropfer13());
                    }
                    
                    $this->entityManager->commit();
                    
                }catch (\Exception $e) {   
                    $this->entityManager->rollback();
                    throw $e;
                }
                
                
                return array('status' => true,     
                             'messages' => ['Dados importados corretamente! Número do vínculo: '.$vinculo->getNrvinculom()]);
            
            }else{
                return array('status' => false,     
                             'message' => "Não foi encontrado vínculo com a matrícula do esocial ".$ideVinculo['matricula']);
            }
            
        }else{
            return array('status' => false,     
                         'message' => self::MSG_ERRO_XML . " (Ausência da tag 'evtAdmissao' no arquivo).");
        }
    }
    
    public function insertXmlS2230($xml){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
        $competencia = $this->hcmParameters->getDtCompetencia();
        $evtAfastTemp  = (array) $xml->evtAfastTemp;
        $dataAtual = DateUtil::getDataAtual();
        $mensagens = array();
        
        if($evtAfastTemp){
            $ideVinculo = (array) $evtAfastTemp['ideVinculo'];
            $infoAfastamento = (array) $evtAfastTemp['infoAfastamento'];
            $iniAfastamento = array_key_exists('iniAfastamento', $infoAfastamento) ? (array) $infoAfastamento['iniAfastamento'] : null;
            $infoRetif = array_key_exists('infoRetif', $infoAfastamento) ? (array) $infoAfastamento['infoRetif'] : null;
            $fimAfastamento = array_key_exists('fimAfastamento', $infoAfastamento) ? (array) $infoAfastamento['fimAfastamento'] : null;
            $infoAtestado = !empty($iniAfastamento) && array_key_exists('infoAtestado', $iniAfastamento) ? (array) $iniAfastamento['infoAtestado'] : null;
            $emitente = !empty($infoAtestado) && array_key_exists('emitente', $infoAtestado) ? (array) $infoAtestado['emitente'] : null;
            $infoCessao = !empty($iniAfastamento) && array_key_exists('infoCessao', $iniAfastamento) ? (array) $iniAfastamento['infoCessao'] : null;
            $infoMandSind = !empty($iniAfastamento) && array_key_exists('infoMandSind', $iniAfastamento) ? (array) $iniAfastamento['infoMandSind'] : null;
            
            $dadosVinculo = '';
            if(!array_key_exists('matricula', $ideVinculo) || is_object($ideVinculo['matricula'])){
                $matricula = null;
            }else{
                $matricula =  $ideVinculo['matricula'];
                $dadosVinculo .= 'matrícula esocial: '.$ideVinculo['matricula'].' | ';
            }
            if(!array_key_exists('codCateg', $ideVinculo) || is_object($ideVinculo['codCateg'])){
                $categoria = null;
            }else{
                $categoria =  $ideVinculo['codCateg'];
                $dadosVinculo .= 'cdesocial da categoria: '.$ideVinculo['codCateg'].' | ';
            }
            if(!array_key_exists('cpfTrab', $ideVinculo) || is_object($ideVinculo['cpfTrab'])){
                $cfppessoa = null;
            }else{
                $cfppessoa =  $ideVinculo['cpfTrab'];
                $dadosVinculo .= 'CPF: '.$ideVinculo['cpfTrab'];
            }
            
            $vinculo = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVinculoPorCPF($nrorg, $cfppessoa, $matricula, $competencia, $categoria);
            
            if(!empty($vinculo)){
                if($iniAfastamento['codMotAfast'] == 15){
                    $dtadmissao = $vinculo[0]->getDtadmissaovinc();
                    $dtbaseferias = $vinculo[1]->getDtbaseferias();
                    $inicio = empty($dtbaseferias) ? DateUtil::truncateData($dtbaseferias) : DateUtil::truncateData($dtadmissao);
                    $fim = DateUtil::truncateData(DateUtil::adicionaIntervalo(DateUtil::subtraiIntervalo($inicio, 1, DateUtil::DIA), 1, DateUtil::ANO));
                    
                    try{
                        
                        $this->entityManager->beginTransaction();
                        
                        $feriasAfastamento = null;
                        while(strtotime($inicio->format('Y-m-d')) < strtotime($dataAtual->format('Y-m-d'))){
                            $ferias = $this->entityManager->getRepository(Repositories::FPA_FERIAS)->findOneBy(array('nrorg' => $nrorg, 'nrvinculom' => $vinculo[0]->getNrvinculom(), 'dtiniaquisicao' => $inicio, 'dtfimaquisicao' => $fim));
                            
                            if(empty($ferias)){
                                if($vinculo[0]->getNrtipovinculom() != 2){
                                    $nrtipoferias = 2;
                                }else{
                                    $nrtipoferias = 1;
                                }
                                $qtdfaltas = 0;
                                $qtddiasmaxferias = 30;
                                $qtddiasmaxabono = 0;
                                $qtddiasafastamento = 0;
                                $idcontrolferias = 'ABERTO';
                                
                                // insere ferias para vínculo
                                $ferias = $this->novaFerias($nrorg, $vinculo[0]->getNrvinculom(), $inicio, $fim, $nrtipoferias, $qtdfaltas, $qtddiasmaxferias, $qtddiasmaxabono, $qtddiasafastamento, $idcontrolferias);
                            }
                            
                            $inicio = DateUtil::truncateData(DateUtil::adicionaIntervalo($fim, 1, DateUtil::DIA));
                            $fim = DateUtil::truncateData(DateUtil::adicionaIntervalo(DateUtil::subtraiIntervalo($inicio, 1, DateUtil::DIA), 1, DateUtil::ANO));
                            
                            if(strtotime($iniAfastamento['dtIniAfast']) >= strtotime($inicio->format('Y-m-d')) && strtotime($iniAfastamento['dtIniAfast']) <= strtotime($fim->format('Y-m-d'))){
                                $feriasAfastamento = $ferias;
                            }
                            
                        }
                        array_push($mensagens, "Os registros de férias entre ".(empty($dtbaseferias) ? $dtbaseferias->format('d/m/Y') : $dtadmissao->format('d/m/Y'))." e ".$fim->format('d/m/Y')." foram inseridos para o vínculo ".$vinculo[0]->getNrvinculom().".");
                        
                        if($feriasAfastamento){
                            $gozoFerias = $this->entityManager->getRepository(Repositories::FPA_GOZOFERIAS)->findBy(array('nrorg' => $nrorg, 'nrferias' => $feriasAfastamento->getNrferias()));
                            $qtdias = 0;
                            
                            $dtinigozoferias = DateUtil::getDataDeString($this->alterDateFormat($iniAfastamento['dtIniAfast'], 2), DateUtil::FORMATO_BRASILEIRO, true);
                            $dtfimgozoferias = DateUtil::getDataDeString($this->alterDateFormat($fimAfastamento['dtTermAfast'], 2), DateUtil::FORMATO_BRASILEIRO, true);
                            $dtretornoferias = DateUtil::adicionaIntervalo($dtfimgozoferias, 1, DateUtil::DIA);
                            $qtdiasferias = $this->retornaQtDias($dtinigozoferias, $dtfimgozoferias);
                            $idadiant13salar = "N";
                            $idnaopagaadic = "N";
                            
                            if($gozoFerias){
                                foreach($gozoFerias as $gozo){
                                    if((DateUtil::getIntervaloEmSegundos($dtinigozoferias,$gozo->getDtinigozoferias()) <= 0 && DateUtil::getIntervaloEmSegundos($dtinigozoferias,$gozo->getDtfimgozoferias()) >= 0) ||
                                       (DateUtil::getIntervaloEmSegundos($dtfimgozoferias,$gozo->getDtinigozoferias()) <= 0 && DateUtil::getIntervaloEmSegundos($dtfimgozoferias,$gozo->getDtfimgozoferias()) >= 0)){
                                        return array('status' => false,     
                                         'message' => "Já existe um gozo de férias para o vínculo ".$vinculo[0]->getNrvinculom()." com o período ".$gozo->getDtinigozoferias()->format('d/m/Y')." a ".$gozo->getDtfimgozoferias()->format('d/m/Y').".");
                                    }
                                    
                                    if($gozo->getQtdiasferias()){
                                        $qtdias += empty($gozo->getQtdiasferias()) ? 0 : $gozo->getQtdiasferias();
                                    }else{
                                        $qtdias += $this->retornaQtDias($gozo->getDtinigozoferias(), $gozo->getDtfimgozoferias());
                                    }
                                }
                            }
                                
                            // insere gozo de ferias 
                            $gozo = $this->novoGozoFerias($nrorg, $feriasAfastamento->getNrferias(), $dtinigozoferias, null, null, $dtfimgozoferias, $dtretornoferias, $idadiant13salar, $idnaopagaadic, null, $qtdiasferias);
                            array_push($mensagens, "Foi inserido um gozo de férias para o vínculo ".$vinculo[0]->getNrvinculom()." com código ".$gozo->getNrgozoferias().".");
                            
                            if(($qtdias + $qtdiasferias) == 30){
                                // altera idcontrolaferias
                                $feriasAfastamento->setIdcontrolferias('GOZADO_INTEGRAL');
                                
                                $this->entityManager->persist($feriasAfastamento);
                                $this->entityManager->flush();
                            }else if(($qtdias + $qtdiasferias) < 30){
                                // altera idcontrolaferias
                                $feriasAfastamento->setIdcontrolferias('GOZADO_PARCIAL');
                                
                                $this->entityManager->persist($feriasAfastamento);
                                $this->entityManager->flush();
                            }else{
                                array_push($mensagens, "A soma de dias de férias ultrapassou o limite de 30 dias. Número das férias: ".$feriasAfastamento->getNrferias().".");
                            }
                        }else{
                            return array('status' => false,     
                                         'message' => "Não foi possível inserir ou encontrar período de ferias referente a esse gozo de férias.");
                        }
                        
                        $this->entityManager->commit();
                    
                    }catch (\Exception $e) {   
                        $this->entityManager->rollback();
                        throw $e;
                    }
                }else{
                    $altesitufunc = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaSituacaoFuncionalImportacao($nrorg, $vinculo[0]->getNrvinculom(), 2);
                    
                    if(empty($altesitufunc)){
                        return array('status' => false,     
                                     'message' => "Não foi encontrada nenhuma situação funcional cadastrada para esse vínculo.");
                    }else{
                        $altesitufunc = is_array($altesitufunc) ? $altesitufunc : [$altesitufunc];
                    }
                    
                    $dtIniAfast = $iniAfastamento && array_key_exists('dtIniAfast', $iniAfastamento) ? $iniAfastamento['dtIniAfast'] : null; 
                    $dtTermAfast = $fimAfastamento && array_key_exists('dtTermAfast', $fimAfastamento) ? $fimAfastamento['dtTermAfast'] : null; 
                    
                    $alteSituAnte = null;
                    $alteSituPos = null;
                    $situRepetida = false;
                    $dtAfastamento = !empty($dtIniAfast) ? $dtIniAfast : $dtTermAfast; 
                    $idInsereFim = empty($dtIniAfast) ? true : false; 
                        
                    // foreach($altesitufunc as $index => $alteracao){
                    //     $inialtesitu = strtotime($alteracao->getDtinisitufunc()->format('Y-m-d')); 
                    //     $inialteprox = array_key_exists($index+1, $altesitufunc) ? strtotime($altesitufunc[$index+1]->getDtinisitufunc()->format('Y-m-d')) : 0;
                        
                        
                    //     if($inialtesitu == strtotime($dtAfastamento) && !$idInsereFim){
                    //         $situRepetida = true;
                    //     }else if(($inialtesitu < strtotime($dtAfastamento) || ($inialtesitu == strtotime($dtAfastamento) && $idInsereFim))  && (($inialteprox > strtotime($dtAfastamento)) || $inialteprox == 0)){
                    //         $alteSituAnte = $alteracao;
                    //         $alteSituPos = array_key_exists($index+1, $altesitufunc) ? $altesitufunc[$index+1] : null;
                    //     }
                    // }
                    foreach($altesitufunc as $index => $alteracao){
                        $inialtesitu = strtotime($alteracao->getDtinisitufunc()->format('Y-m-d')); 
                        $inialteprox = array_key_exists($index+1, $altesitufunc) ? strtotime($altesitufunc[$index+1]->getDtinisitufunc()->format('Y-m-d')) : 0;
                        
                        
                        if(!empty($dtIniAfast) && $inialtesitu == strtotime($dtIniAfast)){
                            $situRepetida = true;
                        }else if(!empty($dtIniAfast) && $inialtesitu < strtotime($dtIniAfast) && ($inialteprox > strtotime($dtIniAfast) || $inialteprox == 0)){
                            $alteSituAnte = $alteracao;
                            $alteSituPos = array_key_exists($index+1, $altesitufunc) ? $altesitufunc[$index+1] : null;
                        }else if(empty($dtIniAfast) && !empty($dtTermAfast) && ($inialtesitu == strtotime($dtTermAfast) || $inialtesitu < strtotime($dtTermAfast)) && (($inialteprox > strtotime($dtTermAfast)) || $inialteprox == 0)){
                            $alteSituAnte = $alteracao;
                            $alteSituPos = array_key_exists($index+1, $altesitufunc) ? $altesitufunc[$index+1] : null;
                        }
                    }
                    
                    if($situRepetida == true){
                        return array('status' => false,     
                                     'message' => "Foi identificada uma situação com início na mesma data da situação que está no XML.");
                    }
                    
                    if(!$alteSituAnte){
                        return array('status' => false,     
                                     'message' => "Não foi encontrado registro de situação funcional para o vínculo anterior a ".$this->alterDateFormat($dtAfastamento, 2).".");
                    }
                    
                    if(empty($dtIniAfast) && !empty($dtTermAfast)){
                        
                        if($alteSituAnte && ((!empty($alteSituAnte->getDtfimSituFunc()) && $alteSituAnte->getNrsitufuncm() != 1) || $alteSituAnte->getNrsitufuncm() == 1)){
                            return array('status' => false,     
                                         'message' => "Não foi encontrada alteração de situação funcional sem data fim para inserir a data ".$this->alterDateFormat($dtTermAfast, 2).".");
                        }
                        
                        $dtinisitunovo = DateUtil::adicionaIntervalo($this->alterDateFormat($dtTermAfast, 2), 1, DateUtil::DIA);
                        $dtfimsitunovo = !empty($alteSituPos) ? DateUtil::subtraiIntervalo($alteSituPos->getDtinisitufunc(), 1, DateUtil::DIA): null;
                        
                        try{
                            
                            $this->entityManager->beginTransaction();
                            
                            // alteração data fim 
                            $alteSituAnte->setDtfimSituFunc(DateUtil::getDataDeString($this->alterDateFormat($dtTermAfast, 2), DateUtil::FORMATO_BRASILEIRO, true));
                        
                            $this->entityManager->persist($alteSituAnte);
                            $this->entityManager->flush();
                        
                            // nova situação normal
                            $novaAlteSituFunc = $this->novaAltesitufunc($nrorg, $vinculo[0]->getNrvinculom(), 1, $dtinisitunovo, null, null, null, $dtfimsitunovo);
                            array_push($mensagens, "Foi realizada a inserção da data fim da alteração de situação funcional para o vínculo ".$vinculo[0]->getNrvinculom()." de código: ".$alteSituAnte->getNraltesitufunc().".");
                            
                            $this->entityManager->commit();
                    
                        }catch (\Exception $e) {   
                            $this->entityManager->rollback();
                            throw $e;
                        }
                        
                    }else if(!empty($dtIniAfast)){
                        
                        if($alteSituAnte->getNrsitufuncm() != 1 && strtotime($alteSituAnte->getDtfimsitufunc()->format('Y-m-d')) >= strtotime($dtIniAfast)){
                            return array('status' => false,     
                                         'message' => "Vínculo já se encontra afastada na data ".$this->alterDateFormat($dtIniAfast, 2).".");
                        }else if(!empty($alteSituPos) && !empty($dtTermAfast) && $alteSituPos->getNrsitufuncm() != 1 && strtotime($alteSituPos->getDtinisitufunc()->format('Y-m-d')) <= strtotime($dtTermAfast)){
                            return array('status' => false,     
                                         'message' => "Vínculo já se encontra afastada na data ".$this->alterDateFormat($dtTermAfast, 2).".");
                        }
                        
                        $dtfimsitunormal = DateUtil::subtraiIntervalo($this->alterDateFormat($dtIniAfast, 2), 1, DateUtil::DIA);
                        
                        if(is_array($infoAtestado) && array_key_exists('qtdDiasAfast', $infoAtestado)){
                            $qtDiasAfastamento = $infoAtestado['qtdDiasAfast'];  
                        }else if(!empty($dtTermAfast)){
                            $qtDiasAfastamento = $this->retornaQtDias(DateUtil::getDataDeString($this->alterDateFormat($dtIniAfast, 2), DateUtil::FORMATO_BRASILEIRO, true), DateUtil::getDataDeString($this->alterDateFormat($dtTermAfast, 2), 
                                                                      DateUtil::FORMATO_BRASILEIRO, true));  
                        }else{
                            $qtDiasAfastamento = null;
                        }
                        $codMotAfast = $iniAfastamento && array_key_exists('codMotAfast', $iniAfastamento) ? $iniAfastamento['codMotAfast'] : null; 
                        if($codMotAfast == '03'){
                            $motivoAfastamento = 22;
                            $nrsitufuncm = 15;
                        }else{
                            $motivoAfastamento = 70;
                            $nrsitufuncm = 16;
                        }
                        $cddiagnost = !empty($infoAtestado) ? $infoAtestado['codCID']: null;
                        
                        if($cddiagnost){
                            $diagnost = $this->entityManager->getRepository(Repositories::DIAGNOST)->findOneBy(array('cddiagnost' => $cddiagnost));
                            $codigoCDI = $diagnost->getCdtabecdi();
                        }else{
                            $codigoCDI = null;
                        }
                        
                        if($emitente){
                            
                            if($emitente['ideOC'] == '1'){
                                $nrconselhoreg = 3;
                            }else if($emitente['ideOC'] == '2'){
                                $nrconselhoreg = 9;
                            }else{
                                $nrconselhoreg = null;
                            }
                            
                            $pessoaMedico = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->retornaPessoaConselhoRegional($nrorg, $nrconselhoreg, $emitente['nrOc'], $competencia); 
                            $medico = $pessoaMedico[0]['nrpessoa'];
                        }else{
                            $medico = null;
                        }
                        
                        $observacao = array_key_exists('observacao', $iniAfastamento) ? $iniAfastamento['observacao'] : null;
                        
                        try{
                            
                            $this->entityManager->beginTransaction();
                            
                            // alteração do fim da situação funcional anterior
                            $alteSituAnte->setDtfimsitufunc($dtfimsitunormal);
                            
                            $this->entityManager->persist($alteSituAnte);
                            $this->entityManager->flush();
                            
                            // inclusão da situação funcional xml
                            $novaaltesitu = $this->novaAltesitufunc($nrorg, $vinculo[0]->getNrvinculom(), $nrsitufuncm, DateUtil::getDataDeString($this->alterDateFormat($dtIniAfast, 2), DateUtil::FORMATO_BRASILEIRO, true), $motivoAfastamento, $cddiagnost, $codigoCDI, 
                                                                    DateUtil::getDataDeString($this->alterDateFormat($dtTermAfast, 2), DateUtil::FORMATO_BRASILEIRO, true), null, null, null, 1, $qtDiasAfastamento, $observacao, $medico);
                            
                            array_push($mensagens, "Foi realizada a inserção da alteração de situação funcional para o vínculo ".$vinculo[0]->getNrvinculom()." de código: ".$novaaltesitu->getNraltesitufunc().".");
                            array_push($mensagens, "Verifique o campo de motivo de afastamento, foi inserido o tipo ".$motivoAfastamento." como default de acordo com o 'codMotAfst': ".$codMotAfast.".");
                            array_push($mensagens, "Verifique o campo de situação funcional, foi inserido o tipo ".$nrsitufuncm." como default de acordo com o 'codMotAfst': ".$codMotAfast.".");
                            
                            if(!empty($dtTermAfast)){
                                if(!empty($alteSituPos) && $alteSituPos->getNraltesitufunc() == 1){
                                    $dtinisitunovo = DateUtil::adicionaIntervalo($this->alterDateFormat($dtTermAfast, 2), 1, DateUtil::DIA);
                                    $alteSituPos->setDtinisitufunc($dtinisitunovo);
                                    
                                    $this->entityManager->persist($alteSituPos);
                                    $this->entityManager->flush();
                                }else{
                                    $dtinisitunovo = DateUtil::adicionaIntervalo($this->alterDateFormat($dtTermAfast, 2), 1, DateUtil::DIA);
                                    $dtfimsitunovo = !empty($alteSituPos) ? DateUtil::subtraiIntervalo($alteSituPos->getDtinisitufunc(), 1, DateUtil::DIA): null;
                                    
                                    $novaAlteSituFunc = $this->novaAltesitufunc($nrorg, $vinculo[0]->getNrvinculom(), 1, $dtinisitunovo, null, null, null, $dtfimsitunovo);
                                }
                            
                            }else{
                                array_push($mensagens, "A alteração de situação funcional ".$novaaltesitu->getNraltesitufunc()." foi inserida para o vínculo ".$vinculo[0]->getNrvinculom()." sem uma data fim.");
                            }
                        
                        $this->entityManager->commit();
                    
                        }catch (\Exception $e) {   
                            $this->entityManager->rollback();
                            throw $e;
                        }
                    }
                }
                
                return array('status'   => true,     
                             'messages' => $mensagens);
            }else{
                return array('status' => false,     
                             'message' => "Não foi possível encontrar um vínculo. Verfifique se os dados do XML correspondem aos do vínculo (".$dadosVinculo.").");    
            }
        }else{
            return array('status' => false,     
                         'message' => self::MSG_ERRO_XML . " (Ausência da tag 'evtAfastTemp' no arquivo).");
        }
    }
    
    public function insertXmlS2300($xml, $nrtipovinculom){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
        $competencia = $this->hcmParameters->getDtCompetencia();
        $evtTSVInicio  = (array) $xml->evtTSVInicio;
        $mensagens = array();
        
        if($evtTSVInicio){
            $trabalhador = (array) $evtTSVInicio['trabalhador'];
            $infoTSVInicio = (array) $evtTSVInicio['infoTSVInicio'];
            $dtmescompetenc = DateUtil::getDataDeString($this->alterDateFormat($infoTSVInicio['dtInicio'], 1), DateUtil::FORMATO_BRASILEIRO, true);
            
            $pessoaExistente = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->retornaPessoaCpf($nrorg, $competencia, $trabalhador['cpfTrab']);
            $vinculoExistente = !empty($pessoaExistente) ? $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaOutrosVinculosAtivosPorPessoa($nrorg, $pessoaExistente[0]['NRPESSOA']) : null;
             
            if(!empty($pessoa) && !empty($vinculoExistente)){
                return array('status' => false,     
                             'message' => "Esse vínculo já foi inserido. Número:".$vinculoExistente->getNrvinculom()."");
            }
            
            try{
                $this->entityManager->beginTransaction();
            
                // inserção de Pessoa
                if(empty($pessoaExistente)){
                    
                    $nascimento = (array) $trabalhador['nascimento'];
                    $documentos =  array_key_exists('documentos', $trabalhador) ? (array) $trabalhador['documentos'] : null;
                    $ctps = !empty($documentos) && array_key_exists('CTPS', $documentos) ? (array) $documentos['CTPS'] : null;
                    $rg = !empty($documentos) && array_key_exists('RG', $documentos) ? (array) $documentos['RG'] : null;
                    $rne = !empty($documentos) && array_key_exists('RNE', $documentos) ? (array) $documentos['RNE'] : null;
                    $cnh = !empty($documentos) && array_key_exists('CNH', $documentos) ? (array) $documentos['CNH'] : null;
                    $endereco = (array) $trabalhador['endereco'];
                    $infoDeficiencia = (array) $trabalhador['infoDeficiencia'];
                    $contato = array_key_exists('contato', $trabalhador) ? (array) $trabalhador['contato'] : null;
                    $trabEstrangeiro = array_key_exists('trabEstrangeiro', $trabalhador) ? (array) $trabalhador['trabEstrangeiro'] : null;
                    
                    $estadoCivil = $this->entityManager->getRepository(Repositories::ESTADOCIVIL)->findOneBy(array('cdesocial' => $trabalhador['estCiv']));
                    $raca = $this->entityManager->getRepository(Repositories::GPE_RACAPESSOA)->findOneBy(array('cdesocial' => $trabalhador['racaCor']));
                    $grauinstr = $this->entityManager->getRepository(Repositories::GPE_GRAUINSTR)->findOneBy(array('cdesocial' => $trabalhador['grauInstr']));
                    $paisNac = $this->entityManager->getRepository(Repositories::PAIS)->findOneBy(array('cdesocial' => $nascimento['paisNac'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                    $nacionalidade = $this->entityManager->getRepository(Repositories::GPE_NACIONALIDADE)->findOneBy(array('cdpais' => $paisNac->getCdpais())); 
                    $paisNascto = $this->entityManager->getRepository(Repositories::PAIS)->findOneBy(array('cdesocial' => $nascimento['paisNascto'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                    $codMunic = $this->entityManager->getRepository(Repositories::MUNICIPIO)->findOneBy(array('cdmunicibge' => $nascimento['codMunic'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                    
                    $dtnascimento = $this->alterDateFormat($nascimento['dtNascto'], 2);
                    $nrCondicaoFisica = $this->retornaCondicaoFisica($infoDeficiencia);
                    
                    if(array_key_exists('nisTrab', $trabalhador)){
                        $validaPis = $this->validaPis($trabalhador['nisTrab']);
                        
                        if($validaPis){
                            $nrPIS = $trabalhador['nisTrab'];
                            $nrnitpessoa = null;
                        }else{
                            $nrPIS = null;
                            $nrnitpessoa = $trabalhador['nisTrab'];
                        }
                        
                    }else{
                        $nrPIS = null;
                        $nrnitpessoa = null;
                    }
                    
                    if(empty($ctps)){
                        $ctps['nrCtps'] = null;
                        $ctps['serieCtps'] = null;
                        $ctps['ufCtps'] = null;
                    }
                    
                    if(empty($rg)){
                        $rg['nrRg'] = null;
                        $rg['orgaoEmissor'] = null;
                        $rg['dtExped'] = null;
                    }else{
                        $rg['dtExped'] = $this->alterDateFormat($rg['dtExped'], 2);
                    }
                    
                    if(empty($rne)){
                        $rne['nrRne'] = null;
                        $rne['orgaoEmissor'] = null;
                        $rne['dtExped'] = null;
                    }else{
                        $rne['dtExped'] = $this->alterDateFormat($rne['dtExped'], 2);
                    }
                    
                    if(empty($cnh)){
                        $cnh['nrRegCnh'] = null;
                        $cnh['dtExped'] = null;
                        $cnh['ufCnh'] = null;
                        $cnh['dtValid'] = null;
                        $cnh['dtPriHab'] = null;
                        $cnh['categoriaCnh'] = null;
                    }else{
                        $cnh['dtExped'] = $this->alterDateFormat($cnh['dtExped'], 2);
                        
                    }
                    
                    if(empty($trabEstrangeiro)){
                        $trabEstrangeiro['dtChegada'] = null;
                        $trabEstrangeiro['classTrabEstrang'] = null;
                    }else{
                        $classTrabEstrang = $this->entityManager->getRepository(Repositories::GPE_CLASSESTRANG)->findOneBy(array('cdesocial' => $trabEstrangeiro['classTrabEstrang']));
                        $trabEstrangeiro['classTrabEstrang'] = $classTrabEstrang->getNrclassestrang();
                    }
                        
                    // insere parceiro de negocio
                    $parceiro = $this->novoParcNegocio($nrorg, $trabalhador['nmTrab'], $trabalhador['nmTrab'], 'PESSOA', 'S', 'N',
                                                        'CPF', $trabalhador['cpfTrab'], DateUtil::getDataDeString($dtnascimento), 'S', 'N');
                    
                    // insere pessoa 
                    $pessoa = $this->novaPessoa($nrorg, $parceiro->getNrparcnegocio(), $dtmescompetenc, $trabalhador['nmTrab'], null, null, DateUtil::getDataDeString($dtnascimento), $trabalhador['sexo'], $nrCondicaoFisica, $trabalhador['cpfTrab'], $ctps['nrCtps'], $ctps['serieCtps'], 
                                                $ctps['ufCtps'], null, $nrPIS, null, null, null, $estadoCivil->getCdestacivil(), null, null, null, null, null, null, $cnh['nrRegCnh'], DateUtil::getDataDeString($cnh['dtExped']), DateUtil::getDataDeString($cnh['dtValid']), $cnh['categoriaCnh'],
                                                $cnh['ufCnh'], null, null, null, null, null, $raca->getNrracapessoa(), null, null, $paisNascto->getCdpais(), $nascimento['uf'], $codMunic->getCdmunicipio(), $rg['nrRg'], $rg['orgaoEmissor'], null, null, DateUtil::getDataDeString($rg['dtExped']),
                                                $nacionalidade->getNrnacionalidade(), $grauinstr->getNrgrauinstr(), null, null, $rne['nrRne'], null, null, null, null, null, null, null, null, null, null, null, null, null, $trabEstrangeiro['classTrabEstrang'], $rne['orgaoEmissor'], 
                                                DateUtil::getDataDeString($trabEstrangeiro['dtChegada']), null, $nrnitpessoa);
                    
                    // insere enderecoparc
                    if(array_key_exists('brasil', $endereco)){
                        $enderecoBrasil = (array) $endereco['brasil'];
                        $cidade = $this->entityManager->getRepository(Repositories::MUNICIPIO)->findOneBy(array('cdmunicibge' => $enderecoBrasil['codMunic'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                        $tpLograd = $this->entityManager->getRepository(Repositories::LOGRADOURO)->findOneBy(array('cdesocial' => $enderecoBrasil['tpLograd'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                        
                        $enderecoBrasil = $this->novoEndereco($parceiro->getNrparcnegocio(), '0055', $enderecoBrasil['uf'], $cidade->getCdmunicipio(), $tpLograd->getCdlogradouro(), $enderecoBrasil['bairro'], 'PRINCIPAL', $enderecoBrasil['cep'], $enderecoBrasil['complemento'], 
                                                              $enderecoBrasil['nrLograd'], $enderecoBrasil['dscLograd'], null);
                        unset($cidade);
                        unset($tpLograd);
                    }
                    if(array_key_exists('exterior', $endereco)){
                        $enderecoExterior = (array) $endereco['exterior']; 
                        $cidade = $this->entityManager->getRepository(Repositories::MUNICIPIO)->findOneBy(array('cdmunicibge' => $enderecoExterior['codMunic'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                        $tpLograd = $this->entityManager->getRepository(Repositories::LOGRADOURO)->findOneBy(array('cdesocial' => $enderecoExterior['tpLograd'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                        $pais = $this->entityManager->getRepository(Repositories::PAIS)->findOneBy(array('cdesocial' => $enderecoExterior['paisResid'], 'nrorg' => array($nrorg, $nrorgpadrao)));
                        
                        $enderecoExterior = $this->novoEndereco($parceiro->getNrparcnegocio(), $pais->getCdpais(), null, $cidade->getCdmunicipio(), $tpLograd->getCdlogradouro(), $enderecoExterior['bairro'], 'EXTERIOR', $enderecoExterior['codPostal'], $enderecoExterior['complemento'], 
                                                                $enderecoExterior['nrLograd'], $enderecoExterior['dscLograd'], null);
                        unset($cidade);
                        unset($tpLograd);
                        unset($pais);
                    }
                    // insere formas de contato
                    
                    $arrayContato = [];
                    if(array_key_exists('fonePrinc', $contato)){
                        $fonePrinc['CDFORMACOMU'] = '01';
                        $fonePrinc['NRPARCNEGOCIO'] = $parceiro->getNrparcnegocio();
                        
                        if(strlen($contato['fonePrinc']) > 9){
                            $fonePrinc['CDPREFIXCOMUPARC'] = substr($contato['fonePrinc'], 0, 2);
                            $fonePrinc['CDCOMUNICAPARC'] = substr($contato['fonePrinc'], 2, (strlen($contato['fonePrinc'])-2));
                        }else{
                            $fonePrinc['CDPREFIXCOMUPARC'] = '';
                            $fonePrinc['CDCOMUNICAPARC'] = $contato['fonePrinc'];
                        }
                        
                        array_push($arrayContato, $fonePrinc);
                        unset($fonePrinc);
                    }
                    if(array_key_exists('foneAlternat', $contato)){
                        $foneAlternat['CDFORMACOMU'] = '02';
                        $foneAlternat['NRPARCNEGOCIO'] = $parceiro->getNrparcnegocio();
                        if(strlen($contato['foneAlternat']) > 9){
                            $foneAlternat['CDPREFIXCOMUPARC'] = substr($contato['foneAlternat'], 0, 2);
                            $foneAlternat['CDCOMUNICAPARC'] = substr($contato['foneAlternat'], 2, (strlen($contato['foneAlternat'])-2));
                        }else{
                            $foneAlternat['CDPREFIXCOMUPARC'] = '';
                            $foneAlternat['CDCOMUNICAPARC'] = $contato['foneAlternat'];
                        }
                        
                        array_push($arrayContato, $foneAlternat);
                        unset($foneAlternat);
                    }
                    if(array_key_exists('emailPrinc', $contato)){
                        
                        $emailPrinc['CDFORMACOMU'] = '05';
                        $emailPrinc['NRPARCNEGOCIO'] = $parceiro->getNrparcnegocio();
                        $emailPrinc['CDPREFIXCOMUPARC'] = '';
                        $emailPrinc['CDCOMUNICAPARC'] = $contato['emailPrinc'];
                        
                        array_push($arrayContato, $emailPrinc);
                        unset($emailPrinc);
                    }
                    
                    if(!empty($arrayContato)){
                        $this->novasFormasComunica($arrayContato);
                    }
                    
                    // inserir o pais
                    if($nascimento['nmMae']){
                        $parceiroMae = $this->novoParcNegocio($nrorg, $nascimento['nmMae'], $nascimento['nmMae'], 'PESSOA', 'S', 'N', 'LIVRE', null, null, 'S', 'N'); 
                        $pessoaMae = $this->novaPessoa($nrorg, $parceiroMae->getNrparcnegocio(), $dtmescompetenc, $nascimento['nmMae'], null, null, null, 'F', null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, 
                                                       null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null);
                        $relacionaMae = $this->novoRelacionaparc($nrorg, 1, $parceiro->getNrparcnegocio(), null, $parceiroMae->getNrparcnegocio());
                    }
                    if($nascimento['nmPai']){
                        $parceiroPai = $this->novoParcNegocio($nrorg, $nascimento['nmPai'], $nascimento['nmPai'], 'PESSOA', 'S', 'N', 'LIVRE', null, null, 'S', 'N'); 
                        $pessoaPai = $this->novaPessoa($nrorg, $parceiroPai->getNrparcnegocio(), $dtmescompetenc, $nascimento['nmPai'], null, null, null, 'M', null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, 
                                                       null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null);
                        $relacionaPai = $this->novoRelacionaparc($nrorg, 2, $parceiro->getNrparcnegocio(), null, $parceiroPai->getNrparcnegocio());
                    }
                    
                    $nrPessoa = $pessoa[0]->getNrpessoa();
                    $nrParcnegocio = $pessoa[0]->getNrparcnegocio();
                    
                }else{
                    $nrPessoa = $pessoaExistente[0]['NRPESSOA'];
                    $nrParcnegocio = $pessoaExistente[0]['NRPARCNEGOCIO'];
                }
                
                $ideEmpregador = array_key_exists('ideEmpregador', $evtTSVInicio) ? (array) $evtTSVInicio['ideEmpregador'] : null;
                $infoComplementares = (array) $infoTSVInicio['infoComplementares'];
                $cargoFuncao = array_key_exists('cargoFuncao', $infoComplementares) ? (array) $infoComplementares['cargoFuncao'] : null;
                $remuneracao = array_key_exists('remuneracao', $infoComplementares) ? (array) $infoComplementares['remuneracao'] : null;
                $FGTS = array_key_exists('fgts', $infoComplementares) ? (array) $infoComplementares['fgts'] : null;
                $infoDirigenteSindical = array_key_exists('infoDirigenteSindical', $infoComplementares) ? (array) $infoComplementares['infoDirigenteSindical'] : null;
                $infoTrabCedido = array_key_exists('infoTrabCedido', $infoComplementares) ? (array) $infoComplementares['infoTrabCedido'] : null;
                $infoEstagiario = array_key_exists('infoEstagiario', $infoComplementares) ? (array) $infoComplementares['infoEstagiario'] : null;
                $instEnsino = !empty($infoEstagiario) && array_key_exists('instEnsino', $infoEstagiario) ? (array) $infoEstagiario['instEnsino'] : null;
                $ageIntegracao = !empty($infoEstagiario) && array_key_exists('ageIntegracao', $infoEstagiario) ? (array) $infoEstagiario['ageIntegracao'] : null;
                $supervisorEstagio = !empty($infoEstagiario) && array_key_exists('supervisorEstagio', $infoEstagiario) ? (array) $infoEstagiario['supervisorEstagio'] : null;
                $termino = array_key_exists('termino', $infoTSVInicio) ? (array) $infoTSVInicio['termino'] : null;
                
                $nrestrutsind = null;
                $nrEscalaTrab = null;
                
                if($nrtipovinculom == "2"){
                    $nrvinculoempreg = 20;
                    $idTipoSalario = 'MENSAL';
                    $idremuneracao = 'ESTAGIO';
                }else if($nrtipovinculom == "3"){
                    $nrvinculoempreg = 14;
                    $idTipoSalario = 'MENSAL';
                    $idremuneracao = 'SOCIO';
                }else{
                    $nrtipovinculom = "10";
                    $nrvinculoempreg = 21;
                    $idTipoSalario = 'MENSAL';
                    $idremuneracao = 'AUTONOMO';
                } 
                
                $dtAdmissao = $this->alterDateFormat($infoTSVInicio['dtInicio'], 2);
                $nrtpmovtransfm = 1;
                $nrtpregimeprev = $infoTrabCedido && array_key_exists('$infoTrabCedido', $infoTrabCedido) ? $infoTrabCedido['$infoTrabCedido'] : null; 
                
                if(!empty($infoDirigenteSindical)){
                    $estrutSind = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->retornaEstruturaPorInscricaoTipoEstrutura($nrorg, $infoDirigenteSindical['cnpjOrigem'], 10, $dtmescompetenc);
                    $nrestrutsind = !empty($estrutSind) ? $estrutSind[0]->getNrestruturam() : null; 
                    $validaSindical = true;
                }else{
                    $validaSindical = false;
                    $nrestrutsind = null;
                }
                
                $dtOpcaoFGTS = $FGTS && array_key_exists('opcFGTS', $FGTS) && $FGTS['opcFGTS'] == 1 ? DateUtil::getDataDeString($this->alterDateFormat($FGTS['dtOpcFGTS'], 2)) : DateUtil::getDataDeString($dtAdmissao);
                
                if(!empty($cargoFuncao)){
                    $cargo = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOH)->retornaUltimoHistoricoCdintegracao($cargoFuncao['codCargo'], $nrorg, $competencia);
                    $nrcargo = !empty($cargo) ? $cargo->Nrocupacaom() : null; 
                    
                    $funcao = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOH)->retornaUltimoHistoricoCdintegracao($cargoFuncao['codFuncao'], $nrorg, $dtmescompetenc); 
                    $nrfuncao = !empty($funcao) ? $funcao->Nrocupacaom() : null; 
                }else{
                    $nrcargo = null;
                    $nrfuncao = null;
                }
                
                if($infoDirigenteSindical && array_key_exists('matricOrig', $infoDirigenteSindical)){
                    $cdmatricula = $infoDirigenteSindical['matricOrig'];
                }else if($infoTrabCedido && array_key_exists('matricCed', $infoTrabCedido)){
                    $cdmatricula = $infoDirigenteSindical['matricCed'];
                }else{
                    $cdmatricula = null;
                }
                
                $observacaoSalario = !empty($remuneracao) && array_key_exists('dscSalVar', $remuneracao) ? $remuneracao['dscSalVar'] : null;
                if(!empty($remuneracao)){
                    $vrSalario = $remuneracao['vrSalFx'];
                }else if($infoEstagiario && array_key_exists('vlrBolsa', $infoEstagiario)){
                    $vrSalario = $infoEstagiario['vlrBolsa'];
                }else{
                    $vrSalario = null;
                }
                
                $estrutlegal = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->getEstruturaPorNrInsc($nrorg, $competencia, $ideEmpregador['nrInsc'], $ideEmpregador['tpInsc']);
                $nrestrutlegal = !empty($estrutlegal) ? $estrutlegal->getNrestruturam() : null; 
                
                if($termino){
                    $nrsitufuncm = 13;
                    $dtRescisao = $termino['dtTerm'];
                }else{
                    $nrsitufuncm = 1;
                    $dtRescisao = null;
                }
                
                // insere vínculo
                $vinculo = $this->novoVinculo($nrPessoa, DateUtil::getDataDeString($dtAdmissao, DateUtil::FORMATO_BRASILEIRO, true), $cdmatricula, $nrtipovinculom, 1, $nrvinculoempreg, DateUtil::getDataDeString($dtOpcaoFGTS, DateUtil::FORMATO_BRASILEIRO, true), 
                                              DateUtil::getDataDeString($dtAdmissao, DateUtil::FORMATO_BRASILEIRO, true), DateUtil::getDataDeString($dtAdmissao, DateUtil::FORMATO_BRASILEIRO, true), $trabalhador['nmTrab'],null, null, 
                                              DateUtil::getDataDeString($dtAdmissao, DateUtil::FORMATO_BRASILEIRO, true), $dtmescompetenc, $nrsitufuncm, $nrcargo, $nrEscalaTrab, null, null, null, DateUtil::getDataDeString($dtAdmissao, DateUtil::FORMATO_BRASILEIRO, true), $idremuneracao, 
                                              $idTipoSalario, $nrestrutlegal, null, 1, 'UNICO', $nrfuncao, null, $nrtpmovtransfm, null, $nrestrutsind, 'N', $nrorg, DateUtil::getDataDeString($dtRescisao, DateUtil::FORMATO_BRASILEIRO, true), null, 
                                              DateUtil::getDataDeString($dtAdmissao, DateUtil::FORMATO_BRASILEIRO, true), null, null, null, null);
                array_push($mensagens, 'Dados importados corretamente! Número do vínculo: '.$vinculo[0]->getNrvinculom().'.');
                array_push($mensagens, $vinculo[0]->getNrvinculom().': Inserir movimentação para uma estrutura gerencial.');
                array_push($mensagens, $vinculo[0]->getNrvinculom().': Inserir escala de trabalho para esse vínculo.');
                
                // insere altesalario
                if(!empty($vrSalario)){
                    $alteSalario = $this->novaAlteSalario($nrorg, $vinculo[0]->getNrvinculom(), DateUtil::getDataDeString($dtAdmissao), 1, 1, $vrSalario, $idTipoSalario, 1, $observacaoSalario, null, null, null);
                }else{
                    array_push($mensagens, $vinculo[0]->getNrvinculom().': Não foi possível cadastrar uma alteração de salário, pois o valor nao foi informado.');
                }
                
                // insere movimentacao sindical
                if($validaSindical == true){
                    $movimSind = $this->novaMovimentacao($nrorg, 1, $vinculo[0]->getNrvinculom(), $nrestrutsind, DateUtil::getDataDeString($dtAdmissao), 10, $nrtpmovtransfm, null, null, 0, null, null);
                }else if($validaSindical == false){
                    array_push($mensagens, $vinculo[0]->getNrvinculom().': Não foi possível cadastrar uma movimentação com a estrutura sindical de CNPJ '.$infoDirigenteSindical['cnpjOrigem'].'.');
                }
                
                //  insere movimentacao legal
                if($nrestrutlegal){
                    $movimLegal = $this->novaMovimentacao($nrorg, 1, $vinculo[0]->getNrvinculom(), $nrestrutlegal, DateUtil::getDataDeString($dtAdmissao), $this->hcmParameters->getTipoEstruturaLegalOrg(), $nrtpmovtransfm, null, null, 0, null, null);
                    array_push($mensagens, $vinculo[0]->getNrvinculom().': Foi inserida uma movimentação com a estrutura legal matriz com início de CNPJ '.$ideEmpregador['nrInsc'].'.');
                }else{
                    array_push($mensagens, $vinculo[0]->getNrvinculom().': Não foi possível cadastrar uma movimentação com a estrutura legal com início de CNPJ '.$ideEmpregador['nrInsc'].'.');
                }
                
                // insere instituicão de ensino
                if($instEnsino){
                    $msg = $this->insereInstituicaoEnsino($nrorg, $instEnsino, $nrParcnegocio, $dtmescompetenc, $vinculo[0]->getNrvinculom());
                    array_push($mensagens, $msg);
                }else if (empty($instEnsino) && $nrtipovinculom == "2"){
                    array_push($mensagens, $vinculo[0]->getNrvinculom().': Não foi informada a instituição de ensino para esse vínculo.');
                }
                
                // insere altesitufunc
                $situFuncional = $this->novaAltesitufunc($nrorg, $vinculo[0]->getNrvinculom(), 1, DateUtil::getDataDeString($dtAdmissao), null, null, null, null, null, null, null, null, null, null);
                
                // if($afastamento){
                //     $iniAfastamento = $this->alterDateFormat($afastamento['dtIniAfast'], 2);
                //     $motivoAfastamento = $this->entityManager->getRepository(Repositories::GPE_MOTIVOAFASTA)->findBy(array('nrorg' => array($nrorg, $nrorgpadrao), 'cdesocial' => $afastamento['codMotAfast']));
                //     if(count($motivoAfastamento) == 1){
                //         $situFuncional->setDtfimsitufunc(DateUtil::subtraiIntervalo($iniAfastamento, 1, DateUtil::DIA));
                //         $alteafastamento = $this->novaAltesitufunc($nrorg, $vinculo->getNrvinculom(), $motivoAfastamento[0]->getNrmotivoafasta(), DateUtil::getDataDeString($iniAfastamento), null, null, null, null, null, null, null, null, null, null);
                //         array_push($mensagens, $vinculo->getNrvinculom().': Complete o cadastro de alteração de situação funcional (número:'.$alteafastamento->getNraltesitufunc().').');
                //     }else if(count($motivoAfastamento) == 0){
                //         array_push($mensagens, $vinculo->getNrvinculom().': Não foi possível cadastrar uma alteração de situação funcional, pois não foi encontrado afastamento de CDESOCIAL '.$afastamento['codMotAfast'].'.');
                //     }else if(count($motivoAfastamento) > 1){
                //         array_push($mensagens, $vinculo->getNrvinculom().': Não foi possível cadastrar uma alteração de situação funcional, pois foram encontrados vários afastamentos de CDESOCIAL '.$afastamento['codMotAfast'].'.');
                //     }
                // }
                
                if($termino){
                    $situFuncional->setDtfimsitufunc(DateUtil::subtraiIntervalo($dtRescisao, 1, DateUtil::DIA));
                    $situFuncDesligamento = $this->novaAltesitufunc($nrorg, $vinculo[0]->getNrvinculom(), 13, DateUtil::getDataDeString($dtRescisao), null, null, null, null, null, null, null, null, null, null);
                }
                
                // insere alteocupacao
                if($nrcargo){
                    $alteOcupacao = $this->novaAlteOcupacao($nrorg, $vinculo[0]->getNrvinculom(), $nrcargo, DateUtil::getDataDeString($dtAdmissao), $cargo->getNrtipoocupacao(), 1, null, null, null, null);
                }
                
                $this->entityManager->commit(); 
                
            }catch (\Exception $e) {   
                $this->entityManager->rollback();
                throw $e;
            }    
            
            return array('status' => true,     
                         'messages' => $mensagens);
            
            
        }else{
            return array('status' => false,     
                         'message' => self::MSG_ERRO_XML . " (Ausência da tag 'evtTSVInicio' no arquivo).");
        }
    }
    
    public function insertXmlS2306($xml){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
        $competencia = $this->hcmParameters->getDtCompetencia();
        $evtTSVAltContr  = (array) $xml->evtTSVAltContr;
        $mensagens = array();
        
        if($evtTSVAltContr){
            $infoTSVAlteracao = (array) $evtTSVAltContr['infoTSVAlteracao'];
            $ideTrabSemVinculo = (array) $evtTSVAltContr['ideTrabSemVinculo'];
            $dtAlteracao = $this->alterDateFormat($infoTSVAlteracao['dtAlteracao'], 2);
            $compAlteracao = DateUtil::getDataDeString($this->alterDateFormat($infoTSVAlteracao['dtAlteracao'], 1), DateUtil::FORMATO_BRASILEIRO, true);
            
            $pessoa = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->retornaPessoaCpf($nrorg, $compAlteracao->format('d/m/Y'), $ideTrabSemVinculo['cpfTrab']);
            $vinculo = !empty($pessoa) ? $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVinculoAtivoPorPessoa($nrorg, $pessoa[0]['NRPESSOA'], $compAlteracao->format('d/m/Y')) : null;
            
            if($vinculo){
                
                try{
                    $this->entityManager->beginTransaction();
                    
                    // dados vínculo
                                    
                    $infoComplementares = (array) $infoTSVAlteracao['infoComplementares'];
                    $cargoFuncao = array_key_exists('cargoFuncao', $infoComplementares) ? (array) $infoComplementares['cargoFuncao'] : null; 
                    $remuneracao = array_key_exists('remuneracao', $infoComplementares) ? (array) $infoComplementares['remuneracao'] : null;
                    $infoEstagiario = array_key_exists('infoEstagiario', $infoComplementares) ? (array) $infoComplementares['infoEstagiario'] : null; 
                    $instEnsino = is_array($infoEstagiario) && array_key_exists('instEnsino', $infoEstagiario) ? (array) $infoEstagiario['instEnsino'] : null; 
                    $ageIntegracao = is_array($infoEstagiario) && array_key_exists('ageIntegracao', $infoEstagiario) ? (array) $infoEstagiario['ageIntegracao'] : null;
                    $nrtipovinculom = $vinculo[0]->getNrtipovinculom();
                    
                    
                    // insere instituicão de ensino
                    if($instEnsino){
                        $msg = $this->insereInstituicaoEnsino($nrorg, $instEnsino, $pessoa[0]['NRPARCNEGOCIO'], $compAlteracao, $vinculo[0]->getNrvinculom());
                        // array_push($mensagens, $msg);
                    }
                    // else if (empty($instEnsino) && $nrtipovinculom == "2"){
                    //     array_push($mensagens, $vinculo[0]->getNrvinculom().': Não foi informada a instituição de ensino para esse vínculo.');
                    // }
                    
                    if(!empty($cargoFuncao)){
                        $cargo = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOH)->retornaUltimoHistoricoCdintegracao($cargoFuncao['codCargo'], $nrorg, $compAlteracao);
                        $nrcargo = !empty($cargo) && $cargo->getNrocupacaom() != $vinculo[1]->getNrcargo() ? $cargo->getNrocupacaom() : null; 
                        
                        $funcao = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOH)->retornaUltimoHistoricoCdintegracao($cargoFuncao['codFuncao'], $nrorg, $compAlteracao); 
                        $nrfuncao = !empty($funcao)  && $funcao->getNrocupacaom() != $vinculo[1]->getNrfuncao() ? $funcao->getNrocupacaom() : null; 
                    }else{
                        $nrcargo = null;
                        $nrfuncao = null;
                    }
                    
                    if(!empty($nrcargo)){
                        $cargom = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOM)->findOneBy(array('nrorg' => $nrorg, 'nrocupacaom' => $nrcargo));
                        $alteOcupacao = $this->novaAlteOcupacao($nrorg, $vinculo[0]->getNrvinculom(), $nrcargo, DateUtil::getDataDeString($compAlteracao), $cargom->getNrtipoocupacao(), 1, null, null, null, null);
                    }else{
                        $nrcargo = $vinculo[1]->getNrcargo();
                        $nrfuncao = empty($nrfuncao) ? $vinculo[1]->getNrfuncao() : $nrfuncao;
                    }
                    
                    if(!empty($remuneracao)){
                        $vrSalario = $remuneracao['vrSalFx'];
                    }else if($infoEstagiario && array_key_exists('vlrBolsa', $infoEstagiario)){
                        $vrSalario = $infoEstagiario['vlrBolsa'];
                    }else{
                        $vrSalario = null;
                    }
                    
                    $salarioAtual = $this->entityManager->getRepository(Repositories::GPE_ALTESALARIO)->retornaAlteSalarioAtual($nrorg, $vinculo[0]->getNrvinculom(), $compAlteracao);
                    // $salarioAtual = is_array($salarioAtual) && !empty($salarioAtual) ? $salarioAtual[0] : $salarioAtual;
                    
                    if(!empty($vrSalario) && !empty($salarioAtual) && $vrSalario > $salarioAtual->getVrsalario()){
                        if($nrtipovinculom == 2){
                            $nrvinculoempreg = 20;
                            $idTipoSalario = 'MENSAL';
                            $idremuneracao = 'ESTAGIO';
                            $nrtpmodalidsal = $vinculo[0]->getNrtpmodalidsal();
                        }else if($nrtipovinculom == 3){
                            $nrvinculoempreg = 14;
                            $idTipoSalario = 'MENSAL';
                            $idremuneracao = 'SOCIO';
                            $nrtpmodalidsal = $vinculo[1]->getNrtpmodalidsal();
                        }else if($nrtipovinculom == 10){
                            $nrvinculoempreg = 21;
                            $idTipoSalario = 'MENSAL';
                            $idremuneracao = 'AUTONOMO';
                            $nrtpmodalidsal = $vinculo[1]->getNrtpmodalidsal();
                        }else{
                            $idremuneracao = $vinculo[1]->getIdremuneracao(); 
                            $idTipoSalario = $vinculo[1]->getIdtppagamento();
                            $nrtpmodalidsal = $vinculo[1]->getNrtpmodalidsal();
                            
                        } 
                        
                        $observacaoSalario = !empty($remuneracao) && array_key_exists('dscSalVar', $remuneracao) ? $remuneracao['dscSalVar'] : null;
                        $alteSalario = $this->novaAlteSalario($nrorg, $vinculo[0]->getNrvinculom(), DateUtil::getDataDeString($compAlteracao), 1, 1, $vrSalario, $idTipoSalario, 1, $observacaoSalario, null, null, null);
                    }else{
                        $idremuneracao = $vinculo[1]->getIdremuneracao(); 
                        $idTipoSalario = $vinculo[1]->getIdtppagamento();
                        $nrtpmodalidsal = $vinculo[1]->getNrtpmodalidsal();
                    }
                    
                    if($vinculo[1]->getDtmescompetenc() == $compAlteracao){
                        
                        $vinculoh = $vinculo[1];
                        $vinculom = $vinculo[0];
                        
                        $vinculom->setDtesocial(DateUtil::getDataDeString($dtAlteracao, DateUtil::FORMATO_BRASILEIRO, true));
                        $vinculoh->setNrcargo($nrcargo);
                        $vinculoh->setIdremuneracao($idremuneracao);
                        $vinculoh->setIdtppagamento($idTipoSalario);
                        $vinculoh->setNrtpmodalidsal($nrtpmodalidsal);
                        $vinculoh->setNrfuncao($nrfuncao);
                        
                        $this->entityManager->persist($vinculoh);
                        $this->entityManager->flush();
                        
                    }else{
                         
                        $historicoNovo = $this->vinculoHistorico($nrorg, $vinculo[0]->getNrvinculom(), $compAlteracao, $vinculo[1]->getNrsitufuncm(), $nrcargo, $vinculo[1]->getNrescalatrabm(), $vinculo[1]->getIdcontribuisind(), $vinculo[1]->getNrdependir(), $vinculo[1]->getNrdependsfam(), 
                                                                 $vinculo[1]->getDtbaseferias(), $idremuneracao, $idTipoSalario, $vinculo[1]->getNrestrutlegal(), $vinculo[1]->getNrestrutgeren(), $nrtpmodalidsal, $vinculo[1]->getIdmultvinc(), $nrfuncao, $vinculo[1]->getCdcontribindividual(), 
                                                                 $vinculo[1]->getNrtpmovtransfm(), $vinculo[1]->getDtfimcontrdetermin(), $vinculo[1]->getNrestrutsind(), $vinculo[1]->getConfidencial(), $vinculo[1]->getCdmatriculaesocial(), $vinculo[1]->getIdpagpropfer13());
                    }
                    
                    $this->entityManager->commit();
                    
                }catch (\Exception $e) {   
                    $this->entityManager->rollback();
                    throw $e;
                }
                
                
                return array('status' => true,     
                             'messages' => ['Dados importados corretamente! Número do vínculo: '.$vinculo[0]->getNrvinculom()]);
            
            }else{
                return array('status' => false,     
                             'message' => "Não foi encontrado vínculo com o CPF ".$ideTrabSemVinculo['cpfTrab']);
            }
            
        }else{
            return array('status' => false,     
                         'message' => self::MSG_ERRO_XML . " (Ausência da tag 'evtTSVAltContr' no arquivo).");
        }
    }
    
    public static function consultaDadoRF($NrInscricao){

        /*
          $opc = array('proxy_host'=>"192.168.122.3",
          'proxy_port'=>8080,
          'proxy_login'=>"",
          'proxy_password'=>"");
        */
        $client = new \SoapClient('http://www.soawebservices.com.br/webservices/producao/cdc/cdc.asmx?WSDL');

        if (strlen($NrInscricao) == 14)
            $function = 'PessoaJuridicaNFe';
        else
            $function = 'PessoaFisicaSimplificada';

        $arguments = array(array('Credenciais' => array('Email' => 'guilherme.martins@teknisa.com',
                                                          'Senha' => 'TxVLWUB6'),
                                   'Documento'    => $NrInscricao));
        $options = array('location' => 'http://www.soawebservices.com.br/webservices/producao/cdc/cdc.asmx');

        try {
            $result = $client->__soapCall($function, $arguments, $options);
        }catch (\Exception $e) {
            return  array("Status" => false, "CONEXAO" => true);
        }

        $result = json_encode($result);
        $resultadoConsulta = json_decode($result,true);
        return $resultadoConsulta[$function.'Result'];
    }
    
    public function verificaEstruturaMesmaInscricao($nrInsc, $nrtipoestrutura, $nrorg, $dtcompetencia){
        
        $estrutura = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaPorInscricaoTipoEstrutura($nrorg, $nrInsc, $nrtipoestrutura, $dtcompetencia);
            
        if(!empty($estrutura)){
            return $estrutura;
        }else{
            return false;
        }    
    }
    
    public function ordenaDados($intervalos){
        $sortArray = array();
        
        foreach($intervalos as $k => $intervalo){
            $intervalo = (array) $intervalo;
            foreach($intervalo as $key=>$value){
                if(!isset($sortArray[$key])){
                    $sortArray[$key] = array();
                }
                $sortArray[$key][] = $value;
            }
        }
        $orderby = "iniInterv"; 

        array_multisort($sortArray[$orderby],SORT_ASC,$intervalos);
        
        return $intervalos;
    }
    
    public function alterDateFormat($data, $tpdata = 1){
        if($tpdata == 1){
            $year = substr($data, 0, 4);
            $month = substr($data, 5, 2);
            $newDate = '01/'.$month.'/'.$year;
            
            return $newDate;
        }else if($tpdata == 2){
            $year = substr($data, 0, 4);
            $month = substr($data, 5, 2);
            $day = substr($data, 8, 2);
            
            $newDate = $day.'/'.$month.'/'.$year;
            
            return $newDate; 
        }
    }
    
    public function getHoraDeMinuto($minutos){
        if($minutos < 60){
            $hora = '00'.$minutos;
        }else if($minutos == 60){
            $hora = '0100';
        }else{
            // Arredonda a hora
            $h = floor($minutos / 60); 
            $m = ($minutos - ($h * 60)) / 100; 
            $horas = $h + $m; 
            // Matemática da quinta série
            // Detalhe: Aqui também pode se usar o abs()
            if ($minutos < 0)
                $horas *= -1; 
            // Separa a hora dos minutos
            $sep = explode('.', $horas); 
            $h = $sep[0]; 
            
            if (empty($sep[1])){
                $m = '00'; 
            }else if(strlen($sep[1]) < 2){
                $m = $m . '0'; 
            }else{
                $m = $sep[1]; 
            }
            
            $hora = '0'.$h.$m;  
        }
        
        return $hora;
    }
    
    
    public function retornaQtDias($date1, $date2){
        $timeDate1 = strtotime($date1->format('Y-m-d'));
        $timeDate2 = strtotime($date2->format('Y-m-d'));
        $time = ($timeDate2 - $timeDate1)/3600;
        $days = $time/24;
        
        return $days + 1;
    }
    
    public function retonaAtivo($data){
        $dataFormat = !empty($data) ? DateTime::createFromFormat('d/m/Y', $data) : '';
        $dataAtual = new DateTime();
        
        if(empty($data) || strtotime($dataFormat->format('Y-m-d')) > strtotime($dataAtual->format('Y-m-d'))){
            return 'S';
        }else{
            return 'N';
        }
        
    }
    
    public function montaNomeEstrutura($nomeMatriz, $nrInsc, $infixo){
        if($nomeMatriz == ''){
            $nome = substr($infixo.' '.$nrInsc, 0, 100);
        }else{
            $sufixo = ' - '.$infixo.' '.$nrInsc.'';
            $prefixo = substr($nomeMatriz, 0, (100 - strlen($sufixo)));
            $nome = $prefixo.$sufixo;
        }
        
        return $nome;
    }
    
    public function decompoeNomePessoa($nmpessoa){
        $arrayNmpessoa = explode(' ', $nmpessoa);
        $nomeMeio      = '';

        $primeiroNome = $arrayNmpessoa[0];
        for ($i = 1; $i < count($arrayNmpessoa) - 1; $i++) {
            $nomeMeio .= ' ' . $arrayNmpessoa[$i];
            $nomeMeio = trim($nomeMeio);
        }
        $ultimoNome = $arrayNmpessoa[count($arrayNmpessoa) - 1];
        $nomeMeio   = ($nomeMeio) ? : $ultimoNome;
        return array($primeiroNome, $nomeMeio, $ultimoNome);
    }
    
    public function getPronomeTratamento($cdEstadoCivil,$idSexoPessoa) {
        if ($idSexoPessoa == 'M'){
            $pronomeTratamento = 'Sr.';
            return $pronomeTratamento;
        }
        else if ($idSexoPessoa == 'F' && $cdEstadoCivil == 'C'){
            $pronomeTratamento = 'Sra.';
            return $pronomeTratamento;
        }
        else {
            $pronomeTratamento = 'Srta.';
            return $pronomeTratamento;
        }
    }
    
    public function retonarTipoEstruturaInserida($tipoestruturaLotacao){
        $tipoLegal = $this->hcmParameters->getTipoEstruturaLegalOrg();
        $tipoTomador = !empty($this->hcmParameters->getTipoEstruturaTomadorOrg()) ? $this->hcmParameters->getTipoEstruturaTomadorOrg() : '7';
        foreach($tipoestruturaLotacao as $tipo){
            if($tipo->getNrtipoestrutura() == $tipoLegal){
                return array('tipo'   => 'LEGAL',
                             'numero' => $tipoLegal);
            }
            if($tipo->getNrtipoestrutura() == $tipoTomador){
                return array('tipo'   =>'TOMADOR',
                             'numero' => $tipoTomador);
            }            
        }
    }
    
    public function retornaCondicaoFisica($infoDeficiencia){
        $deficiencias = $infoDeficiencia['defFisica'].$infoDeficiencia['defVisual'].$infoDeficiencia['defAuditiva'].$infoDeficiencia['defMental'].$infoDeficiencia['defIntelectual'].$infoDeficiencia['reabReadap'];

        if(substr_count($deficiencias, 'S') > 1){
            return 6;
        }else if($infoDeficiencia['defFisica'] == 'S'){
            return 2;
        }else if($infoDeficiencia['defVisual'] == 'S'){
            return 4;
        }else if($infoDeficiencia['defAuditiva'] == 'S'){
            return 3;
        }else if($infoDeficiencia['defMental'] == 'S'){
            return 5;
        }else if($infoDeficiencia['defIntelectual'] == 'S'){
            return 8;
        }else if($infoDeficiencia['reabReadap'] == 'S'){
            return 7;
        }else{
            return 1;
        }
    }
    
    public function validaPIS($pis) {
        $multiplicadorBase = "3298765432";
        $total = 0;
        $resto = 0;
        $multiplicando = 0;
        $multiplicador = 0;
        $digito = 99;
        
        // Retira a mascara
        $numeroPIS = preg_replace('/[^0-9]/', '', $pis);
    
        if (strlen($numeroPIS) !== 11 || 
            $numeroPIS === "00000000000" || 
            $numeroPIS === "11111111111" || 
            $numeroPIS === "22222222222" || 
            $numeroPIS === "33333333333" || 
            $numeroPIS === "44444444444" || 
            $numeroPIS === "55555555555" || 
            $numeroPIS === "66666666666" || 
            $numeroPIS === "77777777777" || 
            $numeroPIS === "88888888888" || 
            $numeroPIS === "99999999999") {
            return false;
        } else {
            for ($i = 0; $i < 10; $i++) {
                $multiplicando = intval( substr($numeroPIS, $i, 1));
                $multiplicador = intval( substr($multiplicadorBase, $i, 1));
                $multiplicacao = $multiplicando * $multiplicador;
                $total += $multiplicacao;
            }
    
            $resto = 11 - $total % 11;
            $resto = $resto === 10 || $resto === 11 ? 0 : $resto;
    
            $digito = intval(substr($numeroPIS, 10));
            return $resto === $digito;
        }
}

    public function insereInstituicaoEnsino($nrorg, $instEnsino, $nrparcnegocio, $dtmescompetenc, $nrvinculom){
        $cnpjInstEnsino = array_key_exists('cnpjInstEnsino', $instEnsino) ? $instEnsino['cnpjInstEnsino'] : null; 
        $nmRazao = array_key_exists('nmRazao', $instEnsino) ? $instEnsino['nmRazao'] : null; 
        $mensagem = '';
        
        $instituicao = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->retornaEstruturaEducacional($nrorg, $dtmescompetenc, $cnpjInstEnsino, $nmRazao);
        
        if(!empty($instituicao)){
            $formacao = $this->entityManager->getRepository(Repositories::DES_FORMACAO)->retornaFormacaoPorCompetenciaEstruturaParc($nrorg, DateUtil::getUltimoDiaDoMes($dtmescompetenc), $instituicao[0]->getNrestruturam(), $nrparcnegocio, $nmRazao);
            $formacao = !empty($formacao) ? $formacao[0] : null;
            if(empty($formacao)){
                $formacao = $this->novaFormacao($nrorg, $nrparcnegocio, $instituicao[0]->getNrestruturam(), $nmRazao, null, null, null, null, null, null, null, null, null, null, null, null, null, $instituicao[0]->getNrparcnegocio(), 1);
            }    
        }else{
            
            $parcEnsino = $this->novoParcNegocio($nrorg, $nmRazao, $nmRazao, 'ESTRUTURA', 'N', 'N', 'LIVRE', $cnpjInstEnsino, DateUtil::getDataDeString('01/01/2020', DateUtil::FORMATO_BRASILEIRO, true), 'S', 'N');
            
            $estrutEnsino = $this->novaEstrutura(32, DateUtil::getDataDeString('01/01/2000', DateUtil::FORMATO_BRASILEIRO, true), 'S', 'N', $parcEnsino->getNrparcnegocio(), $nmRazao, $cnpjInstEnsino, null, null, null, $nmRazao, null, null, null, null, null, null, null, null,
                                                 null, null, null, $nmRazao, null, null, null, null, null, null, null, null, null, null, null, $nmRazao, $nmRazao, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, 
                                                 null, null);
            $formacao = $this->novaFormacao($nrorg, $nrparcnegocio, $estrutEnsino[0]->getNrestruturam(), $nmRazao, null, null, null, null, null, null, null, null, null, null, null, null, null, $parcEnsino->getNrparcnegocio(), 1);
        }
            $mensagem = $nrvinculom.': Complete o cadastro da formação (número: '.$formacao->getNrformacao().').';
            return $mensagem; 
    }
    
    public function novoParcNegocio($nrorg, $nmprincipalparc, $nmsecundariparc, $cdtipoparceiro, $idpessoafisica, $idinstituicao,
                                    $cdtipoinscricao, $nrinscricaoparc, $dtnascifundparc, $idativo, $idparcfundido = 'N'){
        
        $parceiro = new $this->parcnegocio;
        $parceiro->setNrorg($nrorg);
        $parceiro->setIdativo($idativo);
        $parceiro->setNmprincipalparc($nmprincipalparc);
        $parceiro->setNmsecundariparc($nmsecundariparc);
        $parceiro->setCdtipoparcprincipal($cdtipoparceiro);
        $parceiro->setIdpessoafisica($idpessoafisica);
        $parceiro->setIdinstituicao($idinstituicao);
        $parceiro->setCdtipoinscricao($cdtipoinscricao);
        $parceiro->setNrinscricaoparc($nrinscricaoparc);
        $parceiro->setDtnascifundparc($dtnascifundparc);
        $parceiro->setIdparcfundido($idparcfundido);

        $parceiro->setDtinclusao(DateUtil::getDataAtual());
        $parceiro->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $parceiro->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $parceiro->setDtultatu(DateUtil::getDataAtual());
        $parceiro->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $parceiro->setCdoperultatu($this->hcmParameters->getCdoperador());

        $parceiro->setNrparcnegocio(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'PARCNEGOCIO', 1, 12, true));
        
        $this->entityManager->persist($parceiro);
        $this->entityManager->flush();
        
        return $parceiro;
        
    }
    
    public function novaEstrutura($nrtipoestrutura, $dtinivigencia, $idativo, $idoptsimples, $nrparcnegocio, $nmestruturam = null, $cdcnpjestrut = null, $dtfimvigencia = null, $cdintestrutura = null, $nrestruturamold = null, $dsestrutura = null, $cdintestrutexp = null,
                                  $cdintestrutexpfil = null, $cdcentcust = null, $dsurllogoestrutura = null, $nrtpeventoctbl = null, $nrregimeapur = null, $dtesocial = null, $nrgrupoesocial = null, $dtextincao = null, $nrcalendario = null, $cdceiestrut = null, $nmrazsocestrut = null,
                                  $cdnatujuri = null, $cdcnae = null, $nrcaged = null, $cdemprcaixa = null, $idtipoempr = null, $idparticipat = null, $nrpropriempr = null, $nrparcrespinf = null, $nrparcentisin = null, $cdfpas = null, $cdreducont = null, $nmfantasia = null, 
                                  $nmestruturah = null, $cdcpfestrutura = null, $dtbasesindical = null, $cdsindical = null, $cdterceiro = null, $nrclasstribut = null, $vrfap = null, $nrtipotribut = null, $vraliqdeson = null, $cdfederalestrut = null, $nrparcregamb = null, 
                                  $nrparcmonbio = null, $dtiniresamb = null, $dtfimresamb = null, $nrtpobrigaprendiz = null, $nrtpobrigapcd = null, $cdcaepfestrut = null, $codconvenio = null){
        
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $mesCompetencia = DateUtil::truncateData(DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataDeString($dtinivigencia,DateUtil::FORMATO_BRASILEIRO,true)));
        
        $estruturam = $this->estruturaMestre($nrorg, $idativo, $nrtipoestrutura, $dtinivigencia, $nrparcnegocio, $nmestruturam, $dtfimvigencia, $cdintestrutura, $nrestruturamold, $dsestrutura, $cdintestrutexp,
                                             $cdintestrutexpfil, $cdcentcust, $dsurllogoestrutura, $nrtpeventoctbl, $nrregimeapur, $dtesocial, $nrgrupoesocial);
        $estruturah = $this->estruturaHistorico($nrorg, $idativo, $estruturam->getNrestruturam(), $mesCompetencia, $idoptsimples, $nrparcnegocio, $cdcnpjestrut, $dtextincao, $nrcalendario, $cdceiestrut, $nmrazsocestrut,
                                                $cdnatujuri, $cdcnae, $nrcaged, $cdemprcaixa, $idtipoempr, $idparticipat, $nrpropriempr, $nrparcrespinf, $nrparcentisin, $cdfpas, $cdreducont, $nmfantasia, 
                                                $nmestruturah, $cdcpfestrutura, $dtbasesindical, $cdsindical, $cdterceiro, $nrclasstribut, $vrfap, $nrtipotribut, $vraliqdeson, $cdfederalestrut, $nrparcregamb, 
                                                $nrparcmonbio, $dtiniresamb, $dtfimresamb, $nrtpobrigaprendiz, $nrtpobrigapcd, $cdcaepfestrut, $codconvenio);
        
        return array($estruturam, $estruturah);
    } 
   
    public function estruturaMestre($nrorg, $idativo, $nrtipoestrutura, $dtinivigencia, $nrparcnegocio, $nmestruturam, $dtfimvigencia = null, $cdintestrutura = null, $nrestruturamold = null, $dsestrutura = null, $cdintestrutexp = null,
                                $cdintestrutexpfil = null, $cdcentcust = null, $dsurllogoestrutura = null, $nrtpeventoctbl = null, $nrregimeapur = null, $dtesocial = null, $nrgrupoesocial = null){
                                    
        $estruturam = new $this->estruturam;
        
        $estruturam->setNrorg($nrorg);
        $estruturam->setNrtipoestrutura($nrtipoestrutura);
        $estruturam->setDtinivigencia(DateUtil::getDataDeString($dtinivigencia,DateUtil::FORMATO_BRASILEIRO,true));
        $estruturam->setNrparcnegocio($nrparcnegocio);
        $estruturam->setNmestruturam($nmestruturam);
        $estruturam->setIdativo($idativo);
        
        $estruturam->setDtfimvigencia(DateUtil::getDataDeString($dtfimvigencia,DateUtil::FORMATO_BRASILEIRO,true));
        $estruturam->setCdintestrutura($cdintestrutura);
        $estruturam->setNrestruturamold($nrestruturamold);
        // $estruturam->setDsestrutura($dsestrutura);
        // $estruturam->setCdintestrutexp($cdintestrutexp);
        // $estruturam->setCdintestrutexpfil($cdintestrutexpfil);
        // $estruturam->setCdcentcust($cdcentcust);
        // $estruturam->setDsurllogoestrutura($dsurllogoestrutura);
        $estruturam->setNrtpeventoctbl($nrtpeventoctbl);
        // $estruturam->setNrregimeapur($nrregimeapur);
        // $estruturam->setDtesocial($dtesocial);
        // $estruturam->setNrgrupoesocial($nrgrupoesocial);       
        

        $estruturam->setDtinclusao(DateUtil::getDataAtual());
        $estruturam->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $estruturam->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $estruturam->setDtultatu(DateUtil::getDataAtual());
        $estruturam->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $estruturam->setCdoperultatu($this->hcmParameters->getCdoperador());

        $estruturam->setNrestruturam(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESTRUTURAM', 1, 12, true));

        $this->entityManager->persist($estruturam);
        $this->entityManager->flush();
    
        return $estruturam;
        
    }
    
    public function estruturaHistorico($nrorg, $idativo, $nrestruturam, $mesCompetencia, $idoptsimples, $nrparcnegocio, $cdcnpjestrut = null, $dtextincao = null, $nrcalendario = null, $cdceiestrut = null, $nmrazsocestrut = null,
                                  $cdnatujuri = null, $cdcnae = null, $nrcaged = null, $cdemprcaixa = null, $idtipoempr = null, $idparticipat = null, $nrpropriempr = null, $nrparcrespinf = null, $nrparcentisin = null, $cdfpas = null, $cdreducont = null, $nmfantasia = null, 
                                  $nmestruturah = null, $cdcpfestrutura = null, $dtbasesindical = null, $cdsindical = null, $cdterceiro = null, $nrclasstribut = null, $vrfap = null, $nrtipotribut = null, $vraliqdeson = null, $cdfederalestrut = null, $nrparcregamb = null, 
                                  $nrparcmonbio = null, $dtiniresamb = null, $dtfimresamb = null, $nrtpobrigaprendiz = null, $nrtpobrigapcd = null, $cdcaepfestrut = null, $codconvenio = null){
        
        $estruturah = new $this->estruturah;
        
        $estruturah->setNrorg($nrorg);
        $estruturah->setNrestruturam($nrestruturam);
        $estruturah->setDtmescompetenc($mesCompetencia);
        $estruturah->setCdcnpjestrut($cdcnpjestrut);
        $estruturah->setIdoptsimples($idoptsimples);
        
        $estruturah->setDtextincao(DateUtil::getDataDeString($dtextincao));
        $estruturah->setNrcalendario($nrcalendario);
        $estruturah->setCdceiestrut($cdceiestrut);
        $estruturah->setNmrazsocestrut($nmrazsocestrut);
        $estruturah->setCdnatujuri($cdnatujuri);
        $estruturah->setCdcnae($cdcnae);
        $estruturah->setNrcaged($nrcaged);
        $estruturah->setCdemprcaixa($cdemprcaixa);
        $estruturah->setIdtipoempr($idtipoempr);
        $estruturah->setIdparticipat($idparticipat);
        $estruturah->setNrparcnegocio($nrparcnegocio);
        $estruturah->setNrpropriempr($nrpropriempr);
        $estruturah->setNrparcrespinf($nrparcrespinf);
        $estruturah->setNrparcentisin($nrparcentisin);
        $estruturah->setCdfpas($cdfpas);
        $estruturah->setCdreducont($cdreducont);
        $estruturah->setNmfantasia($nmfantasia);
        $estruturah->setNmestruturah($nmestruturah);
        $estruturah->setCdcpfestrutura($cdcpfestrutura);
        $estruturah->setDtbasesindical($dtbasesindical);
        $estruturah->setCdsindical($cdsindical);
        $estruturah->setCdterceiro($cdterceiro);
        $estruturah->setNrclasstribut($nrclasstribut);
        $estruturah->setVrfap($vrfap);
        $estruturah->setNrtipotribut($nrtipotribut);
        $estruturah->setVraliqdeson($vraliqdeson);
        $estruturah->setCdfederalestrut($cdfederalestrut);
        $estruturah->setIdativo($idativo);
        $estruturah->setNrparcregamb($nrparcregamb);
        $estruturah->setNrparcmonbio($nrparcmonbio);
        $estruturah->setDtiniresamb($dtiniresamb);
        $estruturah->setDtfimresamb($dtfimresamb);
        // $estruturah->setNrtpobrigaprendiz($nrtpobrigaprendiz);
        // $estruturah->setNrtpobrigapcd($nrtpobrigapcd);
        $estruturah->setCdcaepfestrut($cdcaepfestrut);
        // $estruturah->setCodconvenio($codconvenio);
        
        $estruturah->setDtinclusao(DateUtil::getDataAtual());
        $estruturah->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $estruturah->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $estruturah->setDtultatu(DateUtil::getDataAtual());
        $estruturah->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $estruturah->setCdoperultatu($this->hcmParameters->getCdoperador());

        $estruturah->setNrestruturah(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESTRUTURAH', 1, 12, true));

        $this->entityManager->persist($estruturah);
        $this->entityManager->flush();
        
        return $estruturah;
        
    }
    
    public function novaOcupacao($nrtipoocupacao, $dtinivigencia, $dtmescompetenc, $nmocupacaoh, $dtfimvigencia = null, $dtesocial = null, $nrocupacaomsup = null, $cdintegracao = null, 
                                 $nrcbo = null, $dsobjetivocargo = null, $dsatividadescargo = null, $dscondtrabalho = null, $nrtempoexpano = null , $nrtempoexpmes = null){
        
        $ocupacaom = $this->ocupacaoMestre($nrtipoocupacao, $dtinivigencia, $dtfimvigencia, $dtesocial, $nrocupacaomsup);
    
        $ocupacaoh = $this->ocupacaoHistorico($ocupacaom->getNrocupacaom(), $dtmescompetenc, $nmocupacaoh, $cdintegracao, $nrcbo, $dsobjetivocargo, $dsatividadescargo, $dscondtrabalho, $nrtempoexpano , $nrtempoexpmes);
        
        return array($ocupacaom, $ocupacaoh);
    }
    
    public function ocupacaoMestre($nrtipoocupacao, $dtinivigencia, $dtfimvigencia = null, $dtesocial = null, $nrocupacaomsup = null){
        
        $ocupacaom = new $this->ocupacaom;
        
        $ocupacaom->setNrorg($this->hcmParameters->getNrorgTrabalho());
        $ocupacaom->setNrtipoocupacao($nrtipoocupacao);
        $ocupacaom->setDtinivigencia(DateUtil::getDataDeString($dtinivigencia, DateUtil::FORMATO_BRASILEIRO, true));
        $ocupacaom->setDtfimvigencia(DateUtil::getDataDeString($dtfimvigencia, DateUtil::FORMATO_BRASILEIRO, true));
        // $ocupacaom->setDtesocial($dtesocial);
        // $ocupacaom->setNrocupacaomsup($nrocupacaomsup);
        
        $ocupacaom->setDtinclusao(DateUtil::getDataAtual());
        $ocupacaom->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $ocupacaom->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $ocupacaom->setDtultatu(DateUtil::getDataAtual());
        $ocupacaom->setCdoperultatu($this->hcmParameters->getCdoperador());
        $ocupacaom->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        
        $ocupacaom->setNrocupacaom(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_OCUPACAOM', 1, 12, true));
        
        $this->entityManager->persist($ocupacaom);
        $this->entityManager->flush();
        
        return $ocupacaom; 
    }
    
    public function ocupacaoHistorico($nrocupacaom, $dtmescompetenc, $nmocupacaoh, $cdintegracao = null, $nrcbo = null, $dsobjetivocargo = null, $dsatividadescargo = null, $dscondtrabalho = null, $nrtempoexpano = null , $nrtempoexpmes = null){
        
        $ocupacaoh = new $this->ocupacaoh;
        
        $ocupacaoh->setNrorg($this->hcmParameters->getNrorgTrabalho());
        $ocupacaoh->setNrocupacaom($nrocupacaom);
        $ocupacaoh->setDtmescompetenc(DateUtil::getDataDeString($dtmescompetenc));
        $ocupacaoh->setNmocupacaoh($nmocupacaoh);
        $ocupacaoh->setCdintegracao($cdintegracao);
        $ocupacaoh->setNrcbo($nrcbo);
        // $ocupacaoh->setDsobjetivocargo($dsobjetivocargo);
        // $ocupacaoh->setDsatividadescargo($dsatividadescargo);
        // $ocupacaoh->setDscondtrabalho($dscondtrabalho);
        // $ocupacaoh->setNrtempoexpano($nrtempoexpano);
        // $ocupacaoh->setNrtempoexpmes($nrtempoexpmes);
        
        $ocupacaoh->setDtinclusao(DateUtil::getDataAtual());
        $ocupacaoh->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $ocupacaoh->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $ocupacaoh->setDtultatu(DateUtil::getDataAtual());
        $ocupacaoh->setCdoperultatu($this->hcmParameters->getCdoperador());
        $ocupacaoh->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        
        $ocupacaoh->setNrocupacaoh(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_OCUPACAOH', 1, 12, true));
        
        $this->entityManager->persist($ocupacaoh);
        $this->entityManager->flush();
        
        return $ocupacaoh;
    }
    
    public function novoHorario($nrorg, $dtinivigencia, $dtmescompetenc, $nmhordiah, $idtipohorario, $dtfimvigencia = null, $cdintegracao = null, $hrinicoletamarcacao = null){
        
        $gpehordiam = $this->horarioMestre($nrorg, $dtinivigencia, $dtfimvigencia, $cdintegracao);
    
        $gpehordiah = $this->horarioHistorico($nrorg, $gpehordiam->getNrhordiam(), $dtmescompetenc, $nmhordiah, $idtipohorario, $hrinicoletamarcacao);
        
        return array($gpehordiam, $gpehordiah);
        
    }
    
    public function horarioMestre($nrorg, $dtinivigencia, $dtfimvigencia = null, $cdintegracao = null){
        
        $gpehordiam = new $this->gpehordiam;
        
        $gpehordiam->setNrorg($nrorg);
        $gpehordiam->setDtfimvigencia($dtfimvigencia);
        $gpehordiam->setDtinivigencia($dtinivigencia);
        $gpehordiam->setCdintegracao($cdintegracao);
        
        $gpehordiam->setDtinclusao(DateUtil::getDataAtual());
        $gpehordiam->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $gpehordiam->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $gpehordiam->setDtultatu(DateUtil::getDataAtual());
        $gpehordiam->setCdoperultatu($this->hcmParameters->getCdoperador());
        $gpehordiam->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        
        $gpehordiam->setNrhordiam(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_HORDIAM', 1, 12, true));
        
        $this->entityManager->persist($gpehordiam);
        $this->entityManager->flush();
        
        return $gpehordiam;
        
    }
    
    public function horarioHistorico($nrorg, $nrhordiam, $dtmescompetenc, $nmhordiah, $idtipohorario, $hrinicoletamarcacao = null){
        
        $gpehordiah = new $this->gpehordiah;
        
        $gpehordiah->setNrorg($nrorg);
        $gpehordiah->setNrhordiam($nrhordiam);
        $gpehordiah->setDtmescompetenc($dtmescompetenc);
        $gpehordiah->setNmhordiah($nmhordiah);
        $gpehordiah->setIdtipohorario($idtipohorario);
        $gpehordiah->setHrinicoletamarcacao($hrinicoletamarcacao);
        
        $gpehordiah->setDtinclusao(DateUtil::getDataAtual());
        $gpehordiah->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $gpehordiah->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $gpehordiah->setDtultatu(DateUtil::getDataAtual());
        $gpehordiah->setCdoperultatu($this->hcmParameters->getCdoperador());
        $gpehordiah->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        
        $gpehordiah->setNrhordiah(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_HORDIAH', 1, 12, true));
        
        $this->entityManager->persist($gpehordiah);
        $this->entityManager->flush();
        
        return $gpehordiah; 
    }

    
    public function novaPessoa($nrorg, $nrparcnegocio, $dtmescompetencia, $nmpessoa, $apelido, $nrCertidaoNascimento, $dtNascimento, $idSexo, $nrCondicaoFisica, $nrCPF, $nrCTPS, $nrSerieCTPS, $sgufCTPS, $dtCTPS, $nrPIS, $dtPIS, $nrInscricaoINSS, $nrInscricaoISS, $cdEstadoCivil, 
                               $nrCertidaoCasamento, $dtCasamento, $idIsentoTituloEleitor, $nrTituloEleitor, $nrSecaoEleitoral, $nrZonaEleitoral, $nrHabilitacao, $dtHabilitacao, $dtValidadeHabilitacao, $categoriaHabilitacao, $sgufhabcarpes, $nrCategoriaMilitar, $nrCertificadoReservista, 
                               $dsCertificadoReservista, $dtCertificadoReservista, $cdExpedicaoCertificadoReservista, $nrRaca, $idGrupoSanguineo, $idFatorRH, $cdpais, $sgEstado, $cdMunicipio, $nrRg, $orgaoExpedidorRG, $sgExpedidorRG, $dsLocalExpedicaoRG, $dtExpedicaoRG,
                               $nrNacionalidade, $nrGrauInstrucao, $pronometrat = null, $nranochegapais = null, $nrdocuestrang = null, $idpesnaturaliza = null, $dtnaturalizapes = null, $idposvistoperm = null, $dtpermavisto = null, $nrconselhoreg = null, $nrinscrconsreg = null, 
                               $dtexconsreg = null, $sgufconsreg = null, $cdtpbaixapag = null, $nrlivrocertnasc = null, $nrfolhacertnasc = null, $dscomplcertnasc = null,$nrregctpspes = null, $nrclassestrang = null, $dsexpdocestr = null, $dtchegadapais = null, $idcotapcd = null,
                               $nrnitpessoa = null){
        
        $pessoa = $this->pessoaMestre($nrorg, $nrparcnegocio);
        $pessoah = $this->pessoaHistorico($nrorg, $pessoa->getNrpessoa(), $dtmescompetencia, $nmpessoa, $apelido, $nrCertidaoNascimento, $dtNascimento, $idSexo, $nrCondicaoFisica, $nrCPF, $nrCTPS, $nrSerieCTPS, $sgufCTPS, $dtCTPS, $nrPIS, $dtPIS, $nrInscricaoINSS, $nrInscricaoISS, 
                                        $cdEstadoCivil, $nrCertidaoCasamento, $dtCasamento, $idIsentoTituloEleitor, $nrTituloEleitor, $nrSecaoEleitoral, $nrZonaEleitoral, $nrHabilitacao, $dtHabilitacao, $dtValidadeHabilitacao, $sgufhabcarpes, $categoriaHabilitacao, $nrCategoriaMilitar, 
                                        $nrCertificadoReservista, $dsCertificadoReservista, $dtCertificadoReservista, $cdExpedicaoCertificadoReservista, $nrRaca, $idGrupoSanguineo, $idFatorRH, $cdpais, $sgEstado, $cdMunicipio, $nrRg, $orgaoExpedidorRG, $sgExpedidorRG,
                                        $dsLocalExpedicaoRG, $dtExpedicaoRG, $nrNacionalidade, $nrGrauInstrucao, $pronometrat, $nranochegapais, $nrdocuestrang, $idpesnaturaliza, $dtnaturalizapes, $idposvistoperm, $dtpermavisto, $nrconselhoreg, $nrinscrconsreg, $dtexconsreg,
                                        $sgufconsreg, $cdtpbaixapag, $nrlivrocertnasc, $nrfolhacertnasc, $dscomplcertnasc,$nrregctpspes, $nrclassestrang, $dsexpdocestr, $dtchegadapais, $idcotapcd, $nrnitpessoa);
                                        
        return array($pessoa, $pessoah);
    }
    
    public function pessoaMestre($nrorg, $nrparcnegocio) {
        $pessoa = new $this->gpepessoa;

        $pessoa->setNrorg($nrorg);
        $pessoa->setNrparcnegocio($nrparcnegocio);

        $pessoa->setDtinclusao(DateUtil::getDataAtual());
        $pessoa->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $pessoa->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $pessoa->setDtultatu(DateUtil::getDataAtual());
        $pessoa->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $pessoa->setCdoperultatu($this->hcmParameters->getCdoperador());

        $pessoa->setNrpessoa(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_PESSOA', 1, 12, true));

        $this->entityManager->persist($pessoa);
        $this->entityManager->flush($pessoa);
        return $pessoa;
    }
    
    public function pessoaHistorico($nrorg, $nrpessoa, $dtmescompetencia, $nmpessoa, $apelido, $nrCertidaoNascimento, $dtNascimento, $idSexo, $nrCondicaoFisica, $nrCPF, $nrCTPS, $nrSerieCTPS, $sgufCTPS, $dtCTPS, $nrPIS, $dtPIS, $nrInscricaoINSS, $nrInscricaoISS, $cdEstadoCivil, 
                                  $nrCertidaoCasamento, $dtCasamento, $idIsentoTituloEleitor, $nrTituloEleitor, $nrSecaoEleitoral, $nrZonaEleitoral, $nrHabilitacao, $dtHabilitacao, $dtValidadeHabilitacao, $categoriaHabilitacao, $sgufhabcarpes,$nrCategoriaMilitar, $nrCertificadoReservista, 
                                  $dsCertificadoReservista, $dtCertificadoReservista, $cdExpedicaoCertificadoReservista, $nrRaca, $idGrupoSanguineo, $idFatorRH, $cdpais, $sgEstado, $cdMunicipio, $nrRg, $orgaoExpedidorRG, $sgExpedidorRG, $dsLocalExpedicaoRG, $dtExpedicaoRG, $nrNacionalidade, 
                                  $nrGrauInstrucao, $pronometrat = null,  $nranochegapais = null, $nrdocuestrang = null, $idpesnaturaliza = null, $dtnaturalizapes = null, $idposvistoperm = null, $dtpermavisto = null, $nrconselhoreg = null, $nrinscrconsreg = null, $dtexconsreg = null, 
                                  $sgufconsreg = null, $cdtpbaixapag = null, $nrlivrocertnasc = null, $nrfolhacertnasc = null, $dscomplcertnasc = null,$nrregctpspes = null, $nrclassestrang = null, $dsexpdocestr = null, $dtchegadapais = null, $idcotapcd = null, $nrnitpessoa = null){
        
        $pessoah = new $this->gpepessoah;

        //setDadosPrincipais
        list($primeiroNome, $meioNome, $ultimoNome) = self::decompoeNomePessoa($nmpessoa);
        
        $pessoah->setNrorg($nrorg);
        $pessoah->setNrpessoa($nrpessoa);
        $pessoah->setNmpessoa($nmpessoa);
        $pessoah->setDsprinomepess($primeiroNome);
        $pessoah->setDsnomemeiopess((strlen($meioNome)> 20) ? substr($meioNome, 0, 20) : $meioNome);
        $pessoah->setDsultnomepess($ultimoNome);
        $pessoah->setDsapelidopess($apelido);
        $pessoah->setDtmescompetenc($dtmescompetencia);
        $pessoah->setDtnascpessoa($dtNascimento);
        $pessoah->setNrcertnascpes($nrCertidaoNascimento);
        $pessoah->setNrlivrocertnasc($nrlivrocertnasc);
        $pessoah->setNrfolhacertnasc($nrfolhacertnasc);
        $pessoah->setDscomplcertnasc($dscomplcertnasc);

        //setRegistroIdentidade
        $pessoah->setNrcpfpessoa($nrCPF);
        $pessoah->setNrrgpessoa($nrRg);
        $pessoah->setCdexrgpessoa($orgaoExpedidorRG);
        $pessoah->setSgufrgpessoa($sgExpedidorRG);
        $pessoah->setDslocalexrgpes($dsLocalExpedicaoRG);
        $pessoah->setDtexrgpessoa($dtExpedicaoRG);

        //setLocal
        $pessoah->setNrnacionalid($nrNacionalidade);
        $pessoah->setCdpais($cdpais);
        $pessoah->setSgestado($sgEstado);
        $pessoah->setCdmunicipio($cdMunicipio);

        //setImigrante
        $pessoah->setNranochegapais($nranochegapais);
        $pessoah->setNrdocuestrang($nrdocuestrang);
        $pessoah->setIdpesnaturaliza($idpesnaturaliza);
        $pessoah->setDtnaturalizapes($dtnaturalizapes);
        $pessoah->setIdposvistoperm($idposvistoperm);
        $pessoah->setDtpermavisto($dtpermavisto);
        $pessoah->setNrclassestrang($nrclassestrang);
        $pessoah->setDsexpdocestr($dsexpdocestr);
        $pessoah->setDtchegadapais($dtchegadapais ? DateUtil::getDataDeString($dtchegadapais) : null);

        //setDadosCarteiraTrabalho
        $pessoah->setNrctpspessoa($nrCTPS);
        $pessoah->setNrseriectpspes($nrSerieCTPS);
        $pessoah->setSgufctpspes($sgufCTPS);
        $pessoah->setDtctpspessoa($dtCTPS);
        $pessoah->setNrregctpspes($nrregctpspes);

        //setDadosImpostos
        $pessoah->setNrpispaseppes($nrPIS);
        $pessoah->setDtinscpispasep($dtPIS);
        $pessoah->setNrinscinsspes($nrInscricaoINSS);
        $pessoah->setNrinscisspes($nrInscricaoISS);

        //setDadosCivis
        $pessoah->setCdestacivil($cdEstadoCivil);
        $pessoah->setNrcertcasapes($nrCertidaoCasamento);
        $pessoah->setDtcasapessoa($dtCasamento);

        //setDadosEleitorais
        $pessoah->setIdisentituelei($idIsentoTituloEleitor);
        $pessoah->setNrtitueleipes($nrTituloEleitor);
        $pessoah->setNrsecaeleipes($nrSecaoEleitoral);
        $pessoah->setNrzonaeleipes($nrZonaEleitoral);

        //setDadosHabilitacaoVeiculo
        $pessoah->setNrcarthabpes($nrHabilitacao);
        $pessoah->setDthabcartpes($dtHabilitacao);
        $pessoah->setDtvalhabcartpes($dtValidadeHabilitacao);
        $pessoah->setDscateghabcart($categoriaHabilitacao);

        //setDadosMilitares
        $pessoah->setNrcatmilitarpe($nrCategoriaMilitar);
        $pessoah->setNrcertresepes($nrCertificadoReservista);
        $pessoah->setDscategcertrese($dsCertificadoReservista);
        $pessoah->setDtcertresepes($dtCertificadoReservista);
        $pessoah->setCdexcertresepes($cdExpedicaoCertificadoReservista);

        //setDadosFisicos
        $pessoah->setIdsexopessoa($idSexo);
        $pessoah->setNrracapessoa($nrRaca);
        $pessoah->setNrcondfispes($nrCondicaoFisica);
        $pessoah->setIdgrupsangupes($idGrupoSanguineo);
        $pessoah->setIdcotapcd($idcotapcd);
        $pessoah->setIdfatorrhpes($idFatorRH);

        //setDadosAdicionais
        $pessoah->setNrgrauinstr($nrGrauInstrucao);
        $pessoah->setNrconselhoreg($nrconselhoreg);
        $pessoah->setNrinscrconsreg($nrinscrconsreg);
        $pessoah->setDtexconsreg($dtexconsreg);
        $pessoah->setSgufconsreg($sgufconsreg);
        $pessoah->setCdtpbaixapag($cdtpbaixapag);
        $pessoah->setNrnitpessoa($nrnitpessoa);

        if($pronometrat) {
            $pessoah->setDspronometrat($pronometrat);
        }else{
            $pronomeTratamento = $this->getPronomeTratamento($cdEstadoCivil, $idSexo);
            $pessoah->setDspronometrat($pronomeTratamento);
        }

        $pessoah->setDtinclusao(DateUtil::getDataAtual());
        $pessoah->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $pessoah->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $pessoah->setDtultatu(DateUtil::getDataAtual());
        $pessoah->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $pessoah->setCdoperultatu($this->hcmParameters->getCdoperador());

        $pessoah->setNrpessoah(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_PESSOAH', 1, 12, true));
        
        $this->entityManager->persist($pessoah);
        $this->entityManager->flush($pessoah);
        return $pessoah;
    }
    
    public function novoVinculo($nrPessoa, $dtAdmissaoVinculo, $cdMatricula, $nrTipoVinculom, $nrTipoAdmissao, $nrVinculoEmpregaticio, $dtOpcaoFGTS, $dtAposentadoriaFGTS, $dtPrimeiraAdmissao,$nmVinculom, $nrDiasContratoExp, $nrDiasContratoExpPro, $dtExameMedico, $dtmescompetenc, $nrsitufuncm,
                                $nrcargo, $nrescalatrabm, $idcontribuisind, $nrdependir, $nrdependsfam, $dtbaseferias, $idremuneracao, $idtppagamento, $nrestrutlegal, $nrestrutgeren, $nrtpmodalidsal, $idmultvinc, $nrfuncao, $cdcontribindividual, $nrtpmovtransfm, $dtfimcontrdetermin, 
                                $nrestrutsind, $confidencial = null,$nrorg = null,$dtRescisao = null, $dslivroadmissao = null, $dtentregaepi = null, $cdmatriculaEsocial = null, $dtvenccontrexper = null, $nrclausassecurat = null, $idpagpropfer13 = null){
        
                             
        $vinculom = $this->vinculoMestre($nrPessoa, $nrorg, $dtAdmissaoVinculo, $dtRescisao, $cdMatricula, $nrTipoVinculom, $nrVinculoEmpregaticio, $nrTipoAdmissao, $nmVinculom, $dtPrimeiraAdmissao, $dtOpcaoFGTS, $dtExameMedico, $dtAposentadoriaFGTS, $nrDiasContratoExp, $nrDiasContratoExpPro,
                                         $dslivroadmissao, $dtentregaepi, $dtvenccontrexper, $nrclausassecurat);
        
        $vinculoh = $this->vinculoHistorico($nrorg, $vinculom->getNrvinculom(), $dtmescompetenc, $nrsitufuncm, $nrcargo, $nrescalatrabm, $idcontribuisind, $nrdependir, $nrdependsfam, $dtbaseferias, $idremuneracao, $idtppagamento, $nrestrutlegal, $nrestrutgeren, $nrtpmodalidsal, $idmultvinc, 
                                            $nrfuncao, $cdcontribindividual, $nrtpmovtransfm, $dtfimcontrdetermin, $nrestrutsind, $confidencial, $cdmatriculaEsocial, $idpagpropfer13
        );
        
        return array($vinculom, $vinculoh);
        
    }
    
    public function vinculoMestre($nrpessoa, $nrorgTrab, $dtAdmissao, $dtRescisao = null, $matricula = null, $tipoVinculo = null, $nrvinculoempreg = null, $nrtpadmissao = null, $nmvinculom = null, $dtPrimeiraAdmissao = null, $dtOpcaoFGTS = null, $dtExameMedico = null, 
                                  $dtAposentadoriaFGTS = null, $nrDiasContratoExp = null, $nrDiasContratoExpPro = null, $dslivroadmissao = null, $dtentregaepi = null, $dtvenccontrexper = null, $nrclausassecurat = null){
        
        $vinculo = new $this->gpevinculom;
        
        if(empty($matricula)){
            $matricula = $vinculo->getNrvinculom();
        }
    
        $vinculo->setNrorg($nrorgTrab);
        $vinculo->setNrpessoa($nrpessoa);
        $vinculo->setNrtipovinculom($tipoVinculo);
        $vinculo->setDtadmissaovinc($dtAdmissao);
        $vinculo->setDtrescisaovinc($dtRescisao);
        $vinculo->setCdmatricula($matricula);
        $vinculo->setNrvinculoempreg($nrvinculoempreg);
        $vinculo->setNrtpadmissao($nrtpadmissao);
        $vinculo->setNmvinculom($nmvinculom);
        $vinculo->setDtprimadmissao($dtPrimeiraAdmissao);
        $vinculo->setDtopcaofgts($dtOpcaoFGTS);
        $vinculo->setDtaposentafgts($dtAposentadoriaFGTS);
        $vinculo->setNrdiascontratoexp($nrDiasContratoExp);
        $vinculo->setNrdiascontratoexppro($nrDiasContratoExpPro);
        $vinculo->setDtexamemedico($dtExameMedico);
        $vinculo->setDslivroadmissao($dslivroadmissao);
        $vinculo->setDtentregaepi($dtentregaepi);
        $vinculo->setDtvenccontrexper(DateUtil::getDataDeString($dtvenccontrexper));
        $vinculo->setNrclausassecurat($nrclausassecurat);
        
        $vinculo->setDtinclusao(DateUtil::getDataAtual());
        $vinculo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $vinculo->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $vinculo->setDtultatu(DateUtil::getDataAtual());
        $vinculo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $vinculo->setCdoperultatu($this->hcmParameters->getCdoperador());
        
        $vinculo->setNrvinculom(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_VINCULOM', 1, 12, true));
        
        $this->entityManager->persist($vinculo);
        $this->entityManager->flush();
        
        return $vinculo;
        
    }
    
    public function vinculoHistorico($nrorg, $nrvinculom, $dtmescompetenc, $nrsitufuncm, $nrcargo, $nrescalatrabm, $idcontribuisind, $nrdependir, $nrdependsfam, $dtbaseferias, $idremuneracao, $idtppagamento, $nrestrutlegal, $nrestrutgeren, $nrtpmodalidsal, $idmultvinc, $nrfuncao, 
                                     $cdcontribindividual, $nrtpmovtransfm, $dtfimcontrdetermin, $nrestrutsind, $confidencial, $cdmatriculaEsocial = null, $idpagpropfer13 = null){
                                                                   
        $vinculoh = new $this->gpevinculoh;

        $vinculoh->setNrorg($nrorg);
        $vinculoh->setNrvinculom($nrvinculom);
        $vinculoh->setDtmescompetenc($dtmescompetenc);

        $vinculoh->setDtbaseferias($dtbaseferias);
        $vinculoh->setIdmultvinc($idmultvinc);
        $vinculoh->setNrsitufuncm($nrsitufuncm);
        $vinculoh->setNrtpmovtransfm($nrtpmovtransfm);
        $vinculoh->setCdcontribindividual($cdcontribindividual);
        $vinculoh->setDtfimcontrdetermin($dtfimcontrdetermin);

        $vinculoh->setNrcargo($nrcargo);
        $vinculoh->setNrfuncao($nrfuncao);
        $vinculoh->setNrescalatrabm($nrescalatrabm);
        $vinculoh->setIdremuneracao($idremuneracao);
        $vinculoh->setIdtppagamento($idtppagamento);
        $vinculoh->setNrtpmodalidsal($nrtpmodalidsal);

        $vinculoh->setNrdependir($nrdependir);
        $vinculoh->setNrdependsfam($nrdependsfam);
        $vinculoh->setConfidencial($confidencial);

        $vinculoh->setNrestrutlegal($nrestrutlegal);
        $vinculoh->setNrestrutgeren($nrestrutgeren);
        $vinculoh->setNrestrutsind($nrestrutsind);
        $vinculoh->setIdcontribuisind(($idcontribuisind) ? : 'NUNCA');
        $vinculoh->setCdmatriculaesocial($cdmatriculaEsocial);
        $vinculoh->setIdpagpropfer13($idpagpropfer13);

        $vinculoh->setDtinclusao(DateUtil::getDataAtual());
        $vinculoh->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $vinculoh->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $vinculoh->setDtultatu(DateUtil::getDataAtual());
        $vinculoh->setCdoperultatu($this->hcmParameters->getCdoperador());
        $vinculoh->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());

        $vinculoh->setNrvinculoh(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_VINCULOH', 1, 12, true));
        
        $this->entityManager->persist($vinculoh);
        $this->entityManager->flush();
        
        return $vinculoh;
    }
    
    public function novoEvento($nrorg, $dtinivigencia, $dtfimvigencia, $idtipoevento, $cdintegraevento, $dtmescompetenc, $idativo, $nmeventoh, $iddtocorrencia, $ideventodemonst, $dsobservacao, $idfaltasferias, $idfaltas13resc, $idgeravrzero, $idimpressevento, $idmediaevenferi, 
                               $idmediavalref, $idprocdifmovant, $idprocdifrescis, $idrepdecterc, $idrepdsr, $idrepferias, $idreprecisao, $idretrocedeprov, $idretrocedevent, $idtipoarredonda, $idtipoocorreven, $idunidrefereven, $idzeraevenfecha, $nrdigarredonda, $nreventoestorno, 
                               $nreventoneg, $nreventorescis, $nreventotransf, $nrhomologrubrica, $nrincidcp, $nrincidfgts, $nrincidirrf, $nrincidsind, $nrnaturubr, $nrordimpreseven, $nrpriorestorno, $qtlimirefereven, $qtminocorconsec, $qtminocormedfer, $vrlimitearred, $vrlimiteevento, 
                               $vrpelimitevento, $vrpercentualevento, $vrpisoevento, $vrtetoevento, $idcompsalfer, $idcompsalprov){
        
        $eventom = $this->eventoMestre($nrorg, $dtinivigencia, $dtfimvigencia, $idtipoevento);
        
        $eventoh = $this->eventoHistorico($nrorg, $eventom->getNreventom(), $cdintegraevento, $dtmescompetenc, $dtinivigencia, $idativo, $nmeventoh, $iddtocorrencia, $ideventodemonst, $dsobservacao, $idfaltasferias, $idfaltas13resc, $idgeravrzero, $idimpressevento, $idmediaevenferi, 
                                          $idmediavalref, $idprocdifmovant, $idprocdifrescis, $idrepdecterc, $idrepdsr, $idrepferias, $idreprecisao, $idretrocedeprov, $idretrocedevent, $idtipoarredonda, $idtipoocorreven, $idunidrefereven, $idzeraevenfecha, $nrdigarredonda, $nreventoestorno,
                                          $nreventoneg, $nreventorescis, $nreventotransf, $nrhomologrubrica, $nrincidcp, $nrincidfgts, $nrincidirrf, $nrincidsind, $nrnaturubr, $nrordimpreseven, $nrpriorestorno, $qtlimirefereven, $qtminocorconsec, $qtminocormedfer, $vrlimitearred,
                                          $vrlimiteevento, $vrpelimitevento, $vrpercentualevento, $vrpisoevento, $vrtetoevento, $idcompsalfer, $idcompsalprov);
    
        return array($eventom, $eventoh);
    }
    
    public function eventoMestre($nrorg, $dtinivigencia, $dtfimvigencia, $idtipoevento){
        
        $eventom = new $this->fpaeventom;
    
        $eventom->setNrorg($nrorg);
        $eventom->setDtinivigencia($dtinivigencia);
        $eventom->setDtfimvigencia($dtfimvigencia);
        $eventom->setIdtipoevento($idtipoevento);


        $eventom->setDtinclusao(DateUtil::getDataAtual());
        $eventom->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $eventom->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $eventom->setDtultatu(DateUtil::getDataAtual());
        $eventom->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $eventom->setCdoperultatu($this->hcmParameters->getCdoperador());

        $eventom->setNreventom(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_EVENTOM', 1, 12, true));
        
        $this->entityManager->persist($eventom);
        $this->entityManager->flush();
        
        return $eventom;
    }
    
    public function eventoHistorico($nrorg, $nreventom, $cdintegraevento, $dtmescompetenc, $dtinivigencia, $idativo, $nmeventoh, $iddtocorrencia, $ideventodemonst, $dsobservacao, $idfaltasferias, $idfaltas13resc, $idgeravrzero, $idimpressevento, $idmediaevenferi, $idmediavalref, 
                                    $idprocdifmovant, $idprocdifrescis, $idrepdecterc, $idrepdsr, $idrepferias, $idreprecisao, $idretrocedeprov, $idretrocedevent, $idtipoarredonda, $idtipoocorreven, $idunidrefereven, $idzeraevenfecha, $nrdigarredonda, $nreventoestorno, $nreventoneg, 
                                    $nreventorescis, $nreventotransf, $nrhomologrubrica, $nrincidcp, $nrincidfgts, $nrincidirrf, $nrincidsind, $nrnaturubr, $nrordimpreseven, $nrpriorestorno, $qtlimirefereven, $qtminocorconsec, $qtminocormedfer, $vrlimitearred, $vrlimiteevento, 
                                    $vrpelimitevento, $vrpercentualevento, $vrpisoevento, $vrtetoevento, $idcompsalfer, $idcompsalprov){

        $eventoh = new $this->fpaeventoh;

        $eventoh->setNrorg($nrorg);
        $eventoh->setNreventom($nreventom);
        $eventoh->setCdintegraevento($cdintegraevento);
        
        $eventoh->setDtmescompetenc($dtmescompetenc);
        $eventoh->setDtinivigencia($dtinivigencia);
        $eventoh->setIdativo($idativo);
        $eventoh->setNmeventoh($nmeventoh);
        $eventoh->setIddtocorrencia($iddtocorrencia);
        $eventoh->setIdeventodemonst($ideventodemonst);
        $eventoh->setDsobservacao($dsobservacao);
        $eventoh->setIdfaltasferias($idfaltasferias);
        $eventoh->setIdfaltas13resc($idfaltas13resc);
        $eventoh->setIdgeravrzero($idgeravrzero);
        $eventoh->setIdimpressevento($idimpressevento);
        $eventoh->setIdmediaevenferi($idmediaevenferi);
        $eventoh->setIdmediavalref($idmediavalref);
        $eventoh->setIdprocdifmovant($idprocdifmovant);
        $eventoh->setIdprocdifrescis($idprocdifrescis);
        $eventoh->setIdrepdecterc($idrepdecterc);
        $eventoh->setIdrepdsr($idrepdsr);
        $eventoh->setIdrepferias($idrepferias);
        $eventoh->setIdreprecisao($idreprecisao);
        $eventoh->setIdretrocedeprov($idretrocedeprov);
        $eventoh->setIdretrocedevent($idretrocedevent);
        $eventoh->setIdtipoarredonda($idtipoarredonda);
        $eventoh->setIdtipoocorreven($idtipoocorreven);
        $eventoh->setIdunidrefereven($idunidrefereven);
        $eventoh->setIdzeraevenfecha($idzeraevenfecha);
        $eventoh->setNrdigarredonda($nrdigarredonda);
        $eventoh->setNreventoestorno($nreventoestorno);
        $eventoh->setNreventoneg($nreventoneg);
        $eventoh->setNreventorescis($nreventorescis);
        $eventoh->setNreventotransf($nreventotransf);
        $eventoh->setNrhomologrubrica($nrhomologrubrica);
        $eventoh->setNrincidcp($nrincidcp);
        $eventoh->setNrincidfgts($nrincidfgts);
        $eventoh->setNrincidirrf($nrincidirrf);
        $eventoh->setNrincidsind($nrincidsind);
        $eventoh->setNrnaturubr($nrnaturubr);
        $eventoh->setNrordimpreseven($nrordimpreseven);
        $eventoh->setNrpriorestorno($nrpriorestorno);
        $eventoh->setQtlimirefereven($qtlimirefereven);
        $eventoh->setQtminocorconsec($qtminocorconsec);
        $eventoh->setQtminocormedfer($qtminocormedfer);
        $eventoh->setVrlimitearred($vrlimitearred);
        $eventoh->setVrlimiteevento($vrlimiteevento);
        $eventoh->setVrpelimitevento($vrpelimitevento);
        $eventoh->setVrpercentualevento($vrpercentualevento);
        $eventoh->setVrpisoevento($vrpisoevento);
        $eventoh->setVrtetoevento($vrtetoevento);
        $eventoh->setIdcompsalfer($idcompsalfer);
        $eventoh->setIdcompsalprov($idcompsalprov);
    
        $eventoh->setDtinclusao(DateUtil::getDataAtual());
        $eventoh->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $eventoh->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $eventoh->setDtultatu(DateUtil::getDataAtual());
        $eventoh->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $eventoh->setCdoperultatu($this->hcmParameters->getCdoperador());

        $eventoh->setNreventoh(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_EVENTOH', 1, 12, true));
        
        $this->entityManager->persist($eventoh);
        $this->entityManager->flush();
        
        return $eventoh;
    
    }
    
    public function formatComunicaParc($contatos, $parcnegocio){
        $arrayComunicaParc = array();
        if(array_key_exists('foneFixo', $contatos)){
            $telefone = array();
            $telefone['CDFORMACOMU'] = '01';
            $telefone['NRPARCNEGOCIO'] = $parcnegocio->getNrparcnegocio();
            if(strlen($contatos['foneFixo']) > 9){
                $telefone['CDPREFIXCOMUPARC'] = substr($contatos['foneFixo'], 0, 2);
                $telefone['CDCOMUNICAPARC'] = substr($contatos['foneFixo'], 2, (strlen($contatos['foneFixo'])-2));
            }else{
                $telefone['CDPREFIXCOMUPARC'] = '';
                $telefone['CDCOMUNICAPARC'] = $contatos['foneFixo'];
            }
            array_push($arrayComunicaParc, $telefone);
        }
        if(array_key_exists('email', $contatos)){
            $email = array();
            $email['CDFORMACOMU'] = '05';
            $email['NRPARCNEGOCIO'] = $parcnegocio->getNrparcnegocio();
            $email['CDPREFIXCOMUPARC'] = '';
            $email['CDCOMUNICAPARC'] = $contatos['email'];
            
            array_push($arrayComunicaParc, $email);
        }
        return $arrayComunicaParc;
    }

    
    
    public function novaPessoaResponsável($nrorg, $nomePessoa, $cpfPessoa, $dtmescompetenc, $parcnegocio){
        $pessoa = new $this->gpepessoa;
        
        $pessoa->setNrorg($nrorg);
        $pessoa->setNrparcnegocio($parcnegocio->getNrparcnegocio());

        $pessoa->setDtinclusao(DateUtil::getDataAtual());
        $pessoa->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $pessoa->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $pessoa->setDtultatu(DateUtil::getDataAtual());
        $pessoa->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $pessoa->setCdoperultatu($this->hcmParameters->getCdoperador());

        $pessoa->setNrpessoa(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_PESSOA', 1, 12, true));

        $this->entityManager->persist($pessoa);
        $this->entityManager->flush($pessoa);
        
        
        
        $pessoah = new $this->gpepessoah;
        
        $pessoah->setNrorg($nrorg);
        $pessoah->setNrpessoa($pessoa->getNrpessoa());
        $pessoah->setNmpessoa($nomePessoa);
        $pessoah->setNrcpfpessoa($cpfPessoa);
        $pessoah->setDtmescompetenc($dtmescompetenc);
        
        $pessoah->setDtinclusao(DateUtil::getDataAtual());
        $pessoah->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $pessoah->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $pessoah->setDtultatu(DateUtil::getDataAtual());
        $pessoah->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $pessoah->setCdoperultatu($this->hcmParameters->getCdoperador());

        $pessoah->setNrpessoah(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_PESSOAH', 1, 12, true));
        
        $this->entityManager->persist($pessoah);
        $this->entityManager->flush($pessoah);
        
        return $pessoa;
    }
    
    public function novasFormasComunica($rows) {
        foreach ($rows as $row) {
            
            $obj = new $this->comunicaParc;
            
            $obj->setNrorg($this->hcmParameters->getNrorgTrabalho());
            $obj->setIdativo('S');
            $obj->setNrparcnegocio($row['NRPARCNEGOCIO']);
            $obj->setCdformacomu($row['CDFORMACOMU']);
            $obj->setCdprefixcomuparc($row['CDPREFIXCOMUPARC']);
            $obj->setCdcomunicaparc($row['CDCOMUNICAPARC']);
            $obj->setCdcomplecomuparc(null);
            $obj->setNrenderecoparc(null);
            
            $obj->setDtinclusao(DateUtil::getDataAtual());
            $obj->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $obj->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $obj->setDtultatu(DateUtil::getDataAtual());
            $obj->setCdoperultatu($this->hcmParameters->getCdoperador());
            $obj->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            
            $obj->setNrcomunicaparc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'COMUNICAPARC', 1, 12, true));
            
            $this->entityManager->persist($obj);
            $this->entityManager->flush();
        }    
    }
    
    public function novoIntervaloHorario($nrorg, $nrhordiam, $nrseqintervhorario, $hrentrada, $hrtoleraanteentrada, $hrtolerapostentrada, $hrsaida, $hrtoleraantesaida, $hrtolerapostsaida, $hrinicionucleo = null, $hrfimnucleo = null, $iddiaocorreentrada = null, $iddiaocorresaida = null, 
                                         $idctrlmarcentrada = null, $idctrlmarcsaida = null, $idajustemarcentrada = null, $idajustemarcsaida = null, $hrprojetarintervalo = null, $idativo = null, $dsintervaloprojetado = null, $nrtpintervalo = null){
        
        $intervalohorario = new $this->gpentervaloshorario;
        
        $intervalohorario->setNrorg($nrorg);
        $intervalohorario->setNrhordiam($nrhordiam);
        $intervalohorario->setNrseqintervhorario($nrseqintervhorario);
        $intervalohorario->setHrentrada($hrentrada);
        $intervalohorario->setHrtoleraanteentrada($hrtoleraanteentrada);
        $intervalohorario->setHrtolerapostentrada($hrtolerapostentrada);
        $intervalohorario->setHrsaida($hrsaida);
        $intervalohorario->setHrtoleraantesaida($hrtoleraantesaida);
        $intervalohorario->setHrtolerapostsaida($hrtolerapostsaida);
        $intervalohorario->setHrinicionucleo($hrinicionucleo);
        $intervalohorario->setHrfimnucleo($hrfimnucleo);
        $intervalohorario->setIddiaocorreentrada($iddiaocorreentrada);
        $intervalohorario->setIddiaocorresaida($iddiaocorresaida);
        $intervalohorario->setIdctrlmarcentrada($idctrlmarcentrada);
        $intervalohorario->setIdctrlmarcsaida($idctrlmarcsaida);
        $intervalohorario->setIdajustemarcentrada($idajustemarcentrada);
        $intervalohorario->setIdajustemarcsaida($idajustemarcsaida);
        $intervalohorario->setHrprojetarintervalo($hrprojetarintervalo);
        $intervalohorario->setIdativo($idativo);
        $intervalohorario->setDsintervaloprojetado($dsintervaloprojetado);
        $intervalohorario->setNrtpintervalo($nrtpintervalo);
        
        $intervalohorario->setDtinclusao(DateUtil::getDataAtual());
        $intervalohorario->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $intervalohorario->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $intervalohorario->setDtultatu(DateUtil::getDataAtual());
        $intervalohorario->setCdoperultatu($this->hcmParameters->getCdoperador());
        $intervalohorario->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        
        $intervalohorario->setNrintervalohorario(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_INTERVALOSHORARIO', 1, 12, true));
    
        $this->entityManager->persist($intervalohorario);
        $this->entityManager->flush();
        
        return $intervalohorario;
    }

    public function novoEndereco($nrparcnegocio, $cdpais, $sgestado, $cdmunicipio, $cdlogradouro, $nmbairroendereco, $cdtipoendereco, $nrcependereco, $dscompleendereco, $nrimovelendereco, $dsendereco, $dsreferenciaende){
       
        $endereco = new $this->enderecoParc;

        $endereco->setNrorg($this->hcmParameters->getNrorgTrabalho());
        $endereco->setNrparcnegocio($nrparcnegocio);
        $endereco->setCdpais($cdpais);
        $endereco->setIdativo('S');
        $endereco->setSgestado($sgestado);
        $endereco->setCdmunicipio($cdmunicipio);
        $endereco->setCdlogradouro($cdlogradouro);
        $endereco->setNmbairroendereco($nmbairroendereco);
        $endereco->setCdtipoendereco($cdtipoendereco);
        $endereco->setNrcependereco($nrcependereco);
        $endereco->setDscompleendereco($dscompleendereco);
        $endereco->setNrimovelendereco($nrimovelendereco);
        $endereco->setDsendereco($dsendereco);
        $endereco->setDsreferenciaende($dsreferenciaende);
        
        $endereco->setDtinclusao(DateUtil::getDataAtual());
        $endereco->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $endereco->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $endereco->setDtultatu(DateUtil::getDataAtual());
        $endereco->setCdoperultatu($this->hcmParameters->getCdoperador());
        $endereco->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        
        $endereco->setNrenderecoparc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ENDERECOPARC', 1, 10, true));
        
        $this->entityManager->persist($endereco);
        $this->entityManager->flush();
        
        return $endereco;
    }
    
    public function novoRelacionaparc($nrorg, $nrtiporelaciona, $nrparcnegocio, $dtfimrelacionamento, $nrparcnegrelac){
        
        $relacionaparc = new $this->relacionaparc;

        $relacionaparc->setNrorg($nrorg);
        $relacionaparc->setNrparcnegocio($nrparcnegocio);
        $relacionaparc->setNrtiporelaciona($nrtiporelaciona);
        $relacionaparc->setNrparcnegrelac($nrparcnegrelac);
        //$relacionaparc->setDtfimrelacionamento($dtfimrelacionamento);
        $relacionaparc->setIdativo('S');
        
        $relacionaparc->setDtinclusao(DateUtil::getDataAtual());
        $relacionaparc->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $relacionaparc->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $relacionaparc->setDtultatu(DateUtil::getDataAtual());
        $relacionaparc->setCdoperultatu($this->hcmParameters->getCdoperador());
        $relacionaparc->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        
        $relacionaparc->setNrrelacionaparc(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'RELACIONAPARC', 1, 10, true));

        $this->entityManager->persist($relacionaparc);
        $this->entityManager->flush();
        return $relacionaparc;
        
    }
    
    public function novaMovimentacao($nrorg, $nrtipotransfer, $nrvinculom, $nrestruturam, $dtinimoviment, $nrtipoestrutura = null, $nrtpmovtransf = null, $nrmotivotransf = null, $dtretroativa = null, $idatualizaacerto = 0, $dtfimmovimento = null, $dsobservacao = null){
        
        $movimentacao = new $this->gpemovimentacao;
        
        $movimentacao->setNrorg($nrorg);
        $movimentacao->setNrtipotransfer($nrtipotransfer);
        $movimentacao->setNrvinculom($nrvinculom);
        $movimentacao->setNrestruturam($nrestruturam);
        $movimentacao->setDtinimoviment($dtinimoviment);
        $movimentacao->setNrtipoestrutura($nrtipoestrutura);
        $movimentacao->setNrtpmovtransfm($nrtpmovtransf);
        $movimentacao->setNrmotivotransf($nrmotivotransf);
        $movimentacao->setDtmovimentretro($dtretroativa);
        $movimentacao->setIdatualizaacerto($idatualizaacerto);
        $movimentacao->setDtfimmoviment($dtfimmovimento);
        $movimentacao->setDsobservacao($dsobservacao);

        $movimentacao->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $movimentacao->setDtinclusao(DateUtil::getDataAtual());
        $movimentacao->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $movimentacao->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $movimentacao->setDtultatu(DateUtil::getDataAtual());
        $movimentacao->setCdoperultatu($this->hcmParameters->getCdoperador());

        $movimentacao->setNrmovimentacao(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_MOVIMENTACAO', 1, 12, true));

        $this->entityManager->persist($movimentacao);
        $this->entityManager->flush();

        return $movimentacao;
    }
    
    public function novaAlteOcupacao($nrorg, $nrvinculom, $nrOcupacao, $dtIniOcupacao, $nrTipoOcupacao, $nrTpModalidade, $nrMotivoOcupacao = null, $nrNivelOcupacao = null, $dtFimOcupacao = null, $observacao = null){
        
        $alteOcupacao = new $this->gpealteOcupacao;

        $alteOcupacao->setNrorg($nrorg);
        $alteOcupacao->setNrvinculom($nrvinculom);
        $alteOcupacao->setNrocupacaom($nrOcupacao);
        $alteOcupacao->setDtiniocupacao($dtIniOcupacao);
        $alteOcupacao->setNrtipoocupacao($nrTipoOcupacao);
        $alteOcupacao->setNrmotivoocupa($nrMotivoOcupacao);
        $alteOcupacao->setNrnivelocupa($nrNivelOcupacao);
        $alteOcupacao->setNrtpmodalidsal($nrTpModalidade);
        $alteOcupacao->setDsobservacao($observacao);
        $alteOcupacao->setDtfimocupacao($dtFimOcupacao);

        $alteOcupacao->setDtinclusao(DateUtil::getDataAtual());
        $alteOcupacao->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $alteOcupacao->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $alteOcupacao->setDtultatu(DateUtil::getDataAtual());
        $alteOcupacao->setCdoperultatu($this->hcmParameters->getCdoperador());
        $alteOcupacao->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());

        $alteOcupacao->setNralteocupacao(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTEOCUPACAO', 1, 12, true));
        
        $this->entityManager->persist($alteOcupacao);
        $this->entityManager->flush();
        
        return $alteOcupacao;
    }
    
    public function novaAltesitufunc($nrorg, $nrvinculom, $nrSituacaoFuncional, $dtIniSituacaoFunc, $motivoAfastamento = null, $cdDiagnostico = null, $codigoCDI = null, $dtFimSituacaoFunc = null, $dtPreviretoSitu = null, $dtProrrSituacaoFunc = null,
                                     $dtVerifSituacaoFunc = null, $nrProxSituFunc = null, $qntDias = null, $dsObs = null, $nrpessoa = null){
        
        $altesitufunc = new $this->gpealtesitufunc;
        
        $altesitufunc->setNrorg($nrorg);
        $altesitufunc->setNrvinculom($nrvinculom);
        $altesitufunc->setNrsitufuncm($nrSituacaoFuncional);
        $altesitufunc->setDtinisitufunc($dtIniSituacaoFunc);
        $altesitufunc->setNrqtdiasitufunc($qntDias);

        $altesitufunc->setNrmotivoafasta($motivoAfastamento);
        $altesitufunc->setCddiagnost($cdDiagnostico);
        $altesitufunc->setCdtabecdi($codigoCDI);
        $altesitufunc->setDtfimsitufunc($dtFimSituacaoFunc);
        $altesitufunc->setDtpreviretositu($dtPreviretoSitu);
        $altesitufunc->setDtprorrsitufunc($dtProrrSituacaoFunc);
        $altesitufunc->setDtverifsitufunc($dtVerifSituacaoFunc);
        $altesitufunc->setNrproxsitufunc($nrProxSituFunc);
        $altesitufunc->setDsObsAltSitFunc($dsObs);
        $altesitufunc->setNrpessoa($nrpessoa);

        $altesitufunc->setNrorginclusao($nrorg);
        $altesitufunc->setDtinclusao(DateUtil::getDataAtual());
        $altesitufunc->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $altesitufunc->setNrorgultatu($nrorg);
        $altesitufunc->setDtultatu(DateUtil::getDataAtual());
        $altesitufunc->setCdoperultatu($this->hcmParameters->getCdoperador());

        $altesitufunc->setNraltesitufunc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTESITUFUNC', 1, 12, true));

        $this->entityManager->persist($altesitufunc);
        $this->entityManager->flush();
        
        return $altesitufunc;
    }
    
    public function novaAlteSalario($nrorg, $nrvinculom, $dtAlteracaoSalario, $nrOcorrencia, $nrTipoAlteracao, $vrSalario, $idTipoSalario, $nrTipoModalideSalario, $observacao = null, $vrDiferencaReal = null, $vrPercentualReal = null, $vrpercaltera = null){
        
        $alteSalario = new $this->gpealtesalario;
        
        $alteSalario->setNrorg($nrorg);
        $alteSalario->setNrvinculom($nrvinculom);
        $alteSalario->setDtaltesalario($dtAlteracaoSalario);
        $alteSalario->setNrocorrencia($nrOcorrencia);
        $alteSalario->setNrtipoaltera($nrTipoAlteracao);
        $alteSalario->setVrpercaltera($vrpercaltera);
        $alteSalario->setVrsalario($vrSalario);
        $alteSalario->setIdtpsalario($idTipoSalario);
        $alteSalario->setNrtpmodalidsal($nrTipoModalideSalario);
        $alteSalario->setDsobservacao($observacao);
        $alteSalario->setVrdiferencareal($vrDiferencaReal);
        $alteSalario->setVrpercreal($vrPercentualReal);

        $alteSalario->setDtinclusao(DateUtil::getDataAtual());
        $alteSalario->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $alteSalario->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $alteSalario->setDtultatu(DateUtil::getDataAtual());
        $alteSalario->setCdoperultatu($this->hcmParameters->getCdoperador());
        $alteSalario->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        
        
        $alteSalario->setNraltesalario(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTESALARIO', 1, 12, true));
        
        $this->entityManager->persist($alteSalario);
        $this->entityManager->flush();
        
        return $alteSalario;
    }
    
    public function novoIncideEvento($nrorg, $dtmescompetenc, $idincidevento, $nreventom, $nrtipoincide){
        
        $incideEvento = new $this->fpaincidevento;
    
        $incideEvento->setNrorg($nrorg);
        $incideEvento->setDtmescompetenc($dtmescompetenc);
        $incideEvento->setIdincidevento($idincidevento);
        $incideEvento->setNreventom($nreventom);
        $incideEvento->setNrtipoincide($nrtipoincide);
        
        $incideEvento->setDtinclusao(DateUtil::getDataAtual());
        $incideEvento->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $incideEvento->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $incideEvento->setDtultatu(DateUtil::getDataAtual());
        $incideEvento->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $incideEvento->setCdoperultatu($this->hcmParameters->getCdoperador());
        
        $incideEvento->setNrincidevento(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_INCIDEVENTO', 1, 12, true));
        
        $this->entityManager->persist($incideEvento);
        $this->entityManager->flush();
        
        return $incideEvento;
    }
    
    public function novaFerias($nrorg, $nrvinculom, $dtiniaquisicao, $dtfimaquisicao, $nrtipoferias, $qtdfaltas, $qtddiasmaxferias, $qtddiasmaxabono, $qtddiasafastamento, $idcontrolferias){
        
        $ferias = new $this->fpaferias;
        
        $ferias->setNrorg($nrorg);
        $ferias->setNrvinculom($nrvinculom);
        $ferias->setDtiniaquisicao($dtiniaquisicao);
        $ferias->setDtfimaquisicao($dtfimaquisicao);
        $ferias->setNrtipoferias($nrtipoferias);
        $ferias->setQtfaltasperiodo($qtdfaltas);
        $ferias->setQtdiasmaxfe($qtddiasmaxferias);
        $ferias->setQtdiasmaxabon($qtddiasmaxabono);
        $ferias->setQtdiaafastament($qtddiasafastamento);
        $ferias->setIdcontrolferias($idcontrolferias);
        $ferias->setQtmesafastament(0);
        
        
        $ferias->setDtinclusao(DateUtil::getDataAtual());
        $ferias->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $ferias->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $ferias->setDtultatu(DateUtil::getDataAtual());
        $ferias->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $ferias->setCdoperultatu($this->hcmParameters->getCdoperador());
        
        $ferias->setNrferias(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_FERIAS', 1, 12, true));
        
        $this->entityManager->persist($ferias);
        $this->entityManager->flush();
        
        return $ferias;
    }
    
    public function novoGozoFerias($nrorg, $nrferias, $dtinigozoferias, $dsgozoferias, $dtavisaoprevio, $dtfimgozoferias, $dtretornoferias, $idadiant13salar, $idnaopagaadic, $qtdiasabonoferi, $qtdiasferias){
    
        $gozoFerias = new $this->fpagozoferias;
        
        $gozoFerias->setNrorg($nrorg);
        $gozoFerias->setNrferias($nrferias);
        $gozoFerias->setDtinigozoferias($dtinigozoferias);
        $gozoFerias->setDsgozoferias($dsgozoferias);
        $gozoFerias->setDtavisaoprevio($dtavisaoprevio);
        $gozoFerias->setDtfimgozoferias($dtfimgozoferias);
        $gozoFerias->setDtretornoferias($dtretornoferias);
        $gozoFerias->setIdadiant13salar($idadiant13salar);
        $gozoFerias->setIdnaopagaadic($idnaopagaadic);
        $gozoFerias->setQtdiasabonoferi($qtdiasabonoferi);
        $gozoFerias->setQtdiasferias($qtdiasferias);
        
        $gozoFerias->setDtinclusao(DateUtil::getDataAtual());
        $gozoFerias->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $gozoFerias->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $gozoFerias->setDtultatu(DateUtil::getDataAtual());
        $gozoFerias->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $gozoFerias->setCdoperultatu($this->hcmParameters->getCdoperador());
        
        $gozoFerias->setNrgozoferias(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_GOZOFERIAS', 1, 12, true));
        
        $this->entityManager->persist($gozoFerias);
        $this->entityManager->flush();
        
        return $gozoFerias;
    }
    
    public function novaEscalaTrabalho($nrorg, $dtfimvigencia, $dtinivigencia, $idativo, $nrescalatrabm, $dtmescompetenc, $nmescalatrabh, $descansosemanal, $idexigeauthoraextra, $qthrlimitesephextra, $idturnoextraferiado, $idmantempercvirada, $idatrasosoprimarc, 
                                       $idprojetaintervalo, $qthrprojetaintervalo, $iddescintervalofolga, $iddescintervaloferiado, $iddescintervalosabado, $idturnointervaloextra, $idhextranmarcinterv, $hriniperiodonoturno, $hrfimperiodonoturno, 
                                       $idgeramarcviradadia, $idadcnoturnointervalo, $idcompensaatraso, $idutapuratest, $idutapuradsr, $idnaocontrolajornada, $idgerbenfer, $qthrescalatrabh = null, $qthrsemesctrabh = null, $prientrada = null, 
                                       $prisaida = null, $segentrada = null, $segsaida = null, $nrdiasapurvales = null, $nrtpjornada = null, $nrtpintervalo = null, $nrdiainiapur = null, $nrdiafinapur = null, $idfaltasreflexo = null, $qthrsemenageren = null,
                                       $qthrescalageren = null ){
        
        
        $gpeescalatrabm = $this->escalaMestre($nrorg, $dtfimvigencia, $dtinivigencia);
        
        $gpeescalatrabh = $this->escalaHistorico($nrorg, $idativo, $gpeescalatrabm->getNrescalatrabm(), $dtmescompetenc, $nmescalatrabh, $descansosemanal, $idexigeauthoraextra, $qthrlimitesephextra, $idturnoextraferiado, $idmantempercvirada, $idatrasosoprimarc, 
                                                 $idprojetaintervalo, $qthrprojetaintervalo, $iddescintervalofolga, $iddescintervaloferiado, $iddescintervalosabado, $idturnointervaloextra, $idhextranmarcinterv, $hriniperiodonoturno, $hrfimperiodonoturno, 
                                                 $idgeramarcviradadia, $idadcnoturnointervalo, $idcompensaatraso, $idutapuratest, $idutapuradsr, $idnaocontrolajornada, $idgerbenfer, $qthrescalatrabh, $qthrsemesctrabh, $prientrada, $prisaida, $segentrada,
                                                 $segsaida, $nrdiasapurvales, $nrtpjornada, $nrtpintervalo, $nrdiainiapur, $nrdiafinapur, $idfaltasreflexo, $qthrsemenageren, $qthrescalageren);
    
        return array($gpeescalatrabm, $gpeescalatrabh);
    }
    
    public function escalaMestre($nrorg, $dtfimvigencia, $dtinivigencia){
        $gpeescalatrabm = new $this->gpe_escalatrabm;
        
        $gpeescalatrabm->setNrorg($nrorg);
        $gpeescalatrabm->setDtfimvigencia($dtfimvigencia);
        $gpeescalatrabm->setDtinivigencia($dtinivigencia);

        $gpeescalatrabm->setDtinclusao(DateUtil::getDataAtual());
        $gpeescalatrabm->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $gpeescalatrabm->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $gpeescalatrabm->setDtultatu(DateUtil::getDataAtual());
        $gpeescalatrabm->setCdoperultatu($this->hcmParameters->getCdoperador());
        $gpeescalatrabm->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        
        
        $gpeescalatrabm->setNrescalatrabm(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ESCALATRABM', 1, 12, true));
        
        $this->entityManager->persist($gpeescalatrabm);
        $this->entityManager->flush();
        
        return $gpeescalatrabm;
    }
    
    public function escalaHistorico($nrorg, $idativo, $nrescalatrabm, $dtmescompetenc, $nmescalatrabh, $descansosemanal, $idexigeauthoraextra, $qthrlimitesephextra, $idturnoextraferiado, $idmantempercvirada, $idatrasosoprimarc, $idprojetaintervalo, 
                                    $qthrprojetaintervalo, $iddescintervalofolga, $iddescintervaloferiado, $iddescintervalosabado, $idturnointervaloextra, $idhextranmarcinterv, $hriniperiodonoturno, $hrfimperiodonoturno, $idgeramarcviradadia,
                                    $idadcnoturnointervalo, $idcompensaatraso, $idutapuratest, $idutapuradsr, $idnaocontrolajornada, $idgerbenfer, $qthrescalatrabh = null, $qthrsemesctrabh = null, $prientrada = null, $prisaida = null, $segentrada = null,
                                    $segsaida = null, $nrdiasapurvales = null, $nrtpjornada = null, $nrtpintervalo = null, $nrdiainiapur = null, $nrdiafinapur = null, $idfaltasreflexo = null, $qthrsemenageren = null, $qthrescalageren = null ){
                                        
        $gpeescalatrabh = new $this->gpe_escalatrabh;
        
        $gpeescalatrabh->setNrorg($nrorg);
        $gpeescalatrabh->setIdativo($idativo);
        $gpeescalatrabh->setNrescalatrabm($nrescalatrabm);
        $gpeescalatrabh->setDescansosemanal($descansosemanal);
        $gpeescalatrabh->setDtmescompetenc($dtmescompetenc);
        $gpeescalatrabh->setHrfimperiodonoturno($hrfimperiodonoturno);
        $gpeescalatrabh->setHriniperiodonoturno($hriniperiodonoturno);
        $gpeescalatrabh->setIdadcnoturnointervalo($idadcnoturnointervalo);
        $gpeescalatrabh->setIdatrasosoprimarc($idatrasosoprimarc);
        $gpeescalatrabh->setIdcompensaatraso($idcompensaatraso);
        $gpeescalatrabh->setIddescintervaloferiado($iddescintervaloferiado);
        $gpeescalatrabh->setIddescintervalofolga($iddescintervalofolga);
        $gpeescalatrabh->setIddescintervalosabado($iddescintervalosabado);
        $gpeescalatrabh->setIdexigeauthoraextra($idexigeauthoraextra);
        $gpeescalatrabh->setIdfaltasreflexo($idfaltasreflexo);
        $gpeescalatrabh->setIdgeramarcviradadia($idgeramarcviradadia);
        $gpeescalatrabh->setIdhextranmarcinterv($idhextranmarcinterv);
        $gpeescalatrabh->setIdmantempercvirada($idmantempercvirada);
        $gpeescalatrabh->setIdnaocontrolajornada($idnaocontrolajornada);
        $gpeescalatrabh->setIdprojetaintervalo($idprojetaintervalo);
        $gpeescalatrabh->setIdturnoextraferiado($idturnoextraferiado);
        $gpeescalatrabh->setIdturnointervaloextra($idturnointervaloextra);
        $gpeescalatrabh->setIdutapuradsr($idutapuradsr);
        $gpeescalatrabh->setIdutapuratest($idutapuratest);
        $gpeescalatrabh->setNmescalatrabh($nmescalatrabh);
        $gpeescalatrabh->setNrdiafinapur($nrdiafinapur);
        $gpeescalatrabh->setNrdiainiapur($nrdiainiapur);
        $gpeescalatrabh->setNrdiasapurvales($nrdiasapurvales);
        $gpeescalatrabh->setNrtpintervalo($nrtpintervalo);
        $gpeescalatrabh->setNrtpjornada($nrtpjornada);
        $gpeescalatrabh->setPrientrada($prientrada);
        $gpeescalatrabh->setPrisaida($prisaida);
        $gpeescalatrabh->setQthrescalageren($qthrescalageren);
        $gpeescalatrabh->setQthrescalatrabh($qthrescalatrabh);
        $gpeescalatrabh->setQthrlimitesephextra($qthrlimitesephextra);
        $gpeescalatrabh->setQthrprojetaintervalo($qthrprojetaintervalo);
        $gpeescalatrabh->setQthrsemenageren($qthrsemenageren);
        $gpeescalatrabh->setQthrsemesctrabh($qthrsemesctrabh);
        $gpeescalatrabh->setSegentrada($segentrada);
        $gpeescalatrabh->setSegsaida($segsaida);
        $gpeescalatrabh->setIdgerbenfer($idgerbenfer);
        
        $gpeescalatrabh->setDtinclusao(DateUtil::getDataAtual());
        $gpeescalatrabh->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $gpeescalatrabh->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $gpeescalatrabh->setDtultatu(DateUtil::getDataAtual());
        $gpeescalatrabh->setCdoperultatu($this->hcmParameters->getCdoperador());
        $gpeescalatrabh->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        
        $gpeescalatrabh->setNrescalatrabh(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ESCALATRABH', 1, 12, true));
        
        $this->entityManager->persist($gpeescalatrabh);
        $this->entityManager->flush();
        
        return $gpeescalatrabh;
    }
    
    public function novoDepVinculo($nrorg, $nrpessoa, $nrtipodepende, $nrvinculom){
        
        $depvinculo = new $this->fpadepvinculo;

        $depvinculo->setNrorg($nrorg);
        $depvinculo->setNrpessoa($nrpessoa);
        $depvinculo->setNrtipodepende($nrtipodepende);
        $depvinculo->setNrvinculom($nrvinculom);

        $depvinculo->setDtinclusao(DateUtil::getDataAtual());
        $depvinculo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $depvinculo->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $depvinculo->setDtultatu(DateUtil::getDataAtual());
        $depvinculo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $depvinculo->setCdoperultatu($this->hcmParameters->getCdoperador());

        $depvinculo->setNrdepvinculo(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_DEPVINCULO', 1, 12, true));
        
        $this->entityManager->persist($depvinculo);
        $this->entityManager->flush();
        
        return $depvinculo;
    }
    
    public function novaFormacao($nrorg, $nrparcnegocio, $nrestruturam, $dsinstituicao = null, $cdidioma = null, $cdstatus = null, $desareaformacaonrorg = null, $dsandamentocurso = null, $dscurso = null, $dsobservacao = null, $dtfim = null, $dtinicio = null, 
                                 $idnvidioma = null, $nrareaformacao = null, $nrcargahoraria = null, $nrcurso = null, $nrgrauconferido = null, $nrinstituicaoparc = null, $nrtipoformacao){
        
        $desFormacao = new $this->desFormacao;
        
        $desFormacao->setCdidioma($cdidioma);
        $desFormacao->setCdstatus($cdstatus);
        $desFormacao->setDesAreaformacaoNrorg($desareaformacaonrorg);
        $desFormacao->setDsandamentocurso($dsandamentocurso);
        $desFormacao->setDscurso($dscurso);
        $desFormacao->setDsinstituicao($dsinstituicao);
        $desFormacao->setDsobservacao($dsobservacao);
        $desFormacao->setDtfim($dtfim);
        $desFormacao->setDtinicio($dtinicio);
        $desFormacao->setIdnvidioma($idnvidioma);
        $desFormacao->setNrareaformacao($nrareaformacao);
        $desFormacao->setNrcargahoraria($nrcargahoraria);
        $desFormacao->setNrcurso($nrcurso);
        $desFormacao->setNrestruturam($nrestruturam);
        $desFormacao->setNrgrauconferido($nrgrauconferido);
        $desFormacao->setNrinstituicaoparc($nrinstituicaoparc);
        $desFormacao->setNrorg($nrorg);
        $desFormacao->setNrparcnegocio($nrparcnegocio);
        $desFormacao->setNrtipoformacao($nrtipoformacao);
        
        $desFormacao->setDtinclusao(DateUtil::getDataAtual());
        $desFormacao->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $desFormacao->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $desFormacao->setDtultatu(DateUtil::getDataAtual());
        $desFormacao->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $desFormacao->setCdoperultatu($this->hcmParameters->getCdoperador());

        $desFormacao->setNrformacao(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'DES_FORMACAO', 1, 12, true));
        
        $this->entityManager->persist($desFormacao);
        $this->entityManager->flush();
        
        return $desFormacao;
    }
    
    public function novaAlteEscala($nrorg, $nrvinculom, $nrEscalaTrabalho, $dtIniEscala, $observacao = null, $dtFimEscala = null, $nrhordiam = null, $nrMotivoEscala = null, $nrturno = null, $qthrlimitesephextra = null){
        
        $alteEscala = new $this->gpealteEscala;
        
        $alteEscala->setNrorg($nrorg);
        $alteEscala->setNrvinculom($nrvinculom);
        $alteEscala->setNrescalatrabm($nrEscalaTrabalho);
        $alteEscala->setDtiniescala($dtIniEscala);
        $alteEscala->setNrmotivoescala($nrMotivoEscala);
        $alteEscala->setNrturno($nrturno);
        $alteEscala->setDtfimescala($dtFimEscala);
        $alteEscala->setNrhordiam($nrhordiam);
        $alteEscala->setDsobservacao($observacao);
        $alteEscala->setQthrlimitesephextra($qthrlimitesephextra);

        $alteEscala->setDtinclusao(DateUtil::getDataAtual());
        $alteEscala->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $alteEscala->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $alteEscala->setDtultatu(DateUtil::getDataAtual());
        $alteEscala->setCdoperultatu($this->hcmParameters->getCdoperador());
        $alteEscala->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());

        $alteEscala->setNralteescala(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTEESCALA', 1, 12, true));
        
        $this->entityManager->persist($alteEscala);
        $this->entityManager->flush();
        
        return $alteEscala;
    }
    
    public function insertEstruturaWebservice($nrtipoestrutura, $dados){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        
        if(strlen($dados['Documento']) == 14){
            $idpessoafisica = 'N'; 
            $cdtipoinscricao = 'CNPJ';
        }else{
            $idpessoafisica = 'S'; 
            $cdtipoinscricao = 'CPF';
        }
        
        try{
            $this->entityManager->beginTransaction();
            
            $estrututaMatriz = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaMatrizPorFilial($nrorg, $ideEmpregador['nrInsc'], $nrtipoestrutura, $dtcompetencia)[0];
            if(empty($estrututaMatriz)){
                return array('status' => false, 
                             'message' => 'Não foi encontrada no sistema nenhuma estrutura matriz para essa filial.'); 
            }
            
            if($ideEstab['tpInsc'] == 3){
                $cdceiestrut = null;
                $cdcaepfestrut = $nrInsc; 
                $cdcnpjestrut = $estrututaMatriz['CDCNPJESTRUT'];
            }else if($ideEstab['tpInsc'] == 4){
                $cdceiestrut = $nrInsc;
                $cdcaepfestrut = null; 
                $cdcnpjestrut = $estrututaMatriz['CDCNPJESTRUT'];
            }else{
                $cdceiestrut = null;
                $cdcaepfestrut = null; 
                $cdcnpjestrut = $nrInsc;
            }
            
//            $nomeEstruturaFilial = $this->montaNomeEstrutura($estrututaMatriz['NMRAZSOCESTRUT'], $nrInsc, 'FILIAL');
            
            // Salva Parceiro de Negócio
            $parcnegocio = $this->novoParcNegocio($nrorg, $nomeEstruturaFilial, $nomeEstruturaFilial, 'ESTRUTURA', $idpessoafisica, 'N',
                                            $cdtipoinscricao, $nrInsc, DateUtil::getDataDeString($ideEstab['iniValid'],DateUtil::FORMATO_BRASILEIRO,true), $idativo);
                                            
                                         
            // Salva Estrutura
            $estrutura = $this->novaEstrutura($nrtipoestrutura, $ideEstab['iniValid'], $idativo, $idoptsimples, $parcnegocio->getNrparcnegocio(), $nomeEstruturaFilial, $cdcnpjestrut, $ideEstab['fimValid'], null, null, null, null,
                                              null, null, null, null, null, null, null, null, null, $cdceiestrut, $nomeEstruturaFilial, null, $dadosEstab['cnaePrep'], null, null, null, null, null, null, null, null, null, $nomeEstruturaFilial, $nomeEstruturaFilial,
                                              null, null, null, null, null, $vrfap, null, null, null, null, null, null, null, null, null, $cdcaepfestrut, null);
            
            $this->entityManager->commit();
        }catch (\Exception $e) {   
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
}
